@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -115,8 +115,6 @@ void Arguments::init()
 
     initialization = rocblas_initialization::hpl;
 
-    arithmetic_check = rocblas_arithmetic_check::no_check;
-
     atomics_mode = rocblas_atomics_allowed;
 
     math_mode = rocblas_default_math;
@@ -152,6 +150,9 @@ void Arguments::init()
     outofplace          = false;
     HMM                 = false;
     graph_test          = false;
+    repeatability_check = false;
+
+    use_hipblaslt = -1;
 }
 
 bool Arguments::validate()
