@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,8 @@
 #include "rocblas_parse_data.hpp"
 #include "rocblas_test.hpp"
 #include "test_cleanup.hpp"
-#include "utility.hpp"
+
+#include "client_utility.hpp"
 
 using namespace testing;
 
@@ -221,6 +222,24 @@ static void rocblas_print_version()
     rocblas_cout << "rocBLAS version: " << blas_version << "\n" << std::endl;
 }
 
+// Print rocBLAS and Tensile commit hashes
+static void rocblas_print_commit_hashes()
+{
+    const char* rocblas_tensile_commit_hash[] = {ROCBLAS_TENSILE_COMMIT_ID};
+
+#if BUILD_WITH_TENSILE
+    rocblas_cout << "rocBLAS-commit-hash: " << rocblas_tensile_commit_hash[0] << std::endl
+                 << std::endl;
+    rocblas_cout << "Tensile-commit-hash: " << rocblas_tensile_commit_hash[1] << std::endl
+                 << std::endl;
+#else
+    rocblas_cout << "rocBLAS-commit-hash: " << rocblas_tensile_commit_hash[0] << std::endl
+                 << std::endl;
+    rocblas_cout << "Tensile-commit-hash: N/A, as rocBLAS was built without Tensile" << std::endl
+                 << std::endl;
+#endif
+}
+
 static void rocblas_print_usage_warning()
 {
     std::string warning(
@@ -265,6 +284,8 @@ static void rocblas_set_test_device()
  *****************/
 int main(int argc, char** argv)
 {
+    rocblas_client_init();
+
     std::string args = rocblas_capture_args(argc, argv);
 
     auto* no_signal_handling = getenv("ROCBLAS_TEST_NO_SIGACTION");
@@ -280,6 +301,12 @@ int main(int argc, char** argv)
 
     rocblas_print_version();
 
+    // Print rocBLAS and Tensile commit hashes
+    rocblas_print_commit_hashes();
+
+    // Warn users if using older reference library
+    print_reference_lib_warning();
+
     // Set test device
     rocblas_set_test_device();
 
@@ -308,5 +335,7 @@ int main(int argc, char** argv)
 
     //rocblas_shutdown();
 
+    rocblas_client_shutdown();
+
     return status;
 }
