@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,19 +27,22 @@
 template <typename T>
 void testing_asum_bad_arg(const Arguments& arg)
 {
-    auto rocblas_asum_fn = arg.api == FORTRAN ? rocblas_asum<T, true> : rocblas_asum<T, false>;
+    auto rocblas_asum_fn = arg.api & c_API_FORTRAN ? rocblas_asum<T, true> : rocblas_asum<T, false>;
     auto rocblas_asum_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_asum_64<T, true> : rocblas_asum_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_asum_64<T, true> : rocblas_asum_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
         rocblas_local_handle handle{arg};
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, pointer_mode));
 
-        int64_t    N                = 100;
-        int64_t    incx             = 1;
-        real_t<T>  rocblas_result   = 10;
-        real_t<T>* h_rocblas_result = &rocblas_result;
+        int64_t N    = 100;
+        int64_t incx = 1;
+
+        using RT = real_t<T>;
+
+        RT  rocblas_result   = 10;
+        RT* h_rocblas_result = &rocblas_result;
 
         // Allocate device memory
         device_vector<T> dx(N, incx);
@@ -62,17 +65,21 @@ void testing_asum_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_asum(const Arguments& arg)
 {
-    auto rocblas_asum_fn = arg.api == FORTRAN ? rocblas_asum<T, true> : rocblas_asum<T, false>;
+    auto rocblas_asum_fn = arg.api & c_API_FORTRAN ? rocblas_asum<T, true> : rocblas_asum<T, false>;
     auto rocblas_asum_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_asum_64<T, true> : rocblas_asum_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_asum_64<T, true> : rocblas_asum_64<T, false>;
+
     int64_t N    = arg.N;
     int64_t incx = arg.incx;
 
-    real_t<T>            rocblas_result_1;
-    real_t<T>            rocblas_result_2;
-    real_t<T>            cpu_result;
-    double               rocblas_error_1;
-    double               rocblas_error_2;
+    using RT = real_t<T>;
+
+    RT rocblas_result;
+    RT hr_gold;
+
+    double error_host_ptr;
+    double error_device_ptr;
+
     rocblas_local_handle handle{arg};
 
     // check to prevent undefined memory allocation error
@@ -82,16 +89,16 @@ void testing_asum(const Arguments& arg)
         device_vector<T>    dx(safe_size);
         CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
-        device_vector<real_t<T>> dr(1);
+        device_vector<RT> dr(1);
         CHECK_DEVICE_ALLOCATION(dr.memcheck());
 
-        host_vector<real_t<T>> hr_1(1);
-        host_vector<real_t<T>> hr_2(1);
-        host_vector<real_t<T>> result_0(1);
+        host_vector<RT> hr_1(1);
+        host_vector<RT> hr_2(1);
+        host_vector<RT> result_0(1);
         CHECK_HIP_ERROR(hr_1.memcheck());
         CHECK_HIP_ERROR(hr_2.memcheck());
         CHECK_HIP_ERROR(result_0.memcheck());
-        result_0[0] = real_t<T>(0);
+        result_0[0] = RT(0);
 
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
         DAPI_CHECK(rocblas_asum_fn, (handle, N, dx, incx, hr_1));
@@ -101,8 +108,8 @@ void testing_asum(const Arguments& arg)
         CHECK_HIP_ERROR(hr_2.transfer_from(dr));
 
         // check that result is set to 0
-        unit_check_general<real_t<T>, real_t<T>>(1, 1, 1, result_0, hr_1);
-        unit_check_general<real_t<T>, real_t<T>>(1, 1, 1, result_0, hr_2);
+        unit_check_general<RT, RT>(1, 1, 1, result_0, hr_1);
+        unit_check_general<RT, RT>(1, 1, 1, result_0, hr_2);
 
         return;
     }
@@ -112,8 +119,8 @@ void testing_asum(const Arguments& arg)
     host_vector<T> hx(N, incx);
 
     // Allocate device memory
-    device_vector<T>         dx(N, incx);
-    device_vector<real_t<T>> dr(1);
+    device_vector<T>  dx(N, incx);
+    device_vector<RT> dr(1);
 
     // Check device memory allocation
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
@@ -132,7 +139,7 @@ void testing_asum(const Arguments& arg)
         if(arg.pointer_mode_host)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-            DAPI_CHECK(rocblas_asum_fn, (handle, N, dx, incx, &rocblas_result_1));
+            DAPI_CHECK(rocblas_asum_fn, (handle, N, dx, incx, &rocblas_result));
         }
 
         if(arg.pointer_mode_device)
@@ -141,58 +148,86 @@ void testing_asum(const Arguments& arg)
             handle.pre_test(arg);
             DAPI_CHECK(rocblas_asum_fn, (handle, N, dx, incx, dr));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                RT rocblas_result_copy;
+                CHECK_HIP_ERROR(hipMemcpy(&rocblas_result, dr, sizeof(RT), hipMemcpyDeviceToHost));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_vector<T>  dx_copy(N, incx);
+                    device_vector<RT> dr_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dr_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_asum_fn, (handle_copy, N, dx_copy, incx, dr_copy));
+                        CHECK_HIP_ERROR(hipMemcpy(
+                            &rocblas_result_copy, dr_copy, sizeof(RT), hipMemcpyDeviceToHost));
+                        unit_check_general<RT, RT>(1, 1, 1, &rocblas_result, &rocblas_result_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        cblas_asum<T>(N, hx, incx, &cpu_result);
+        ref_asum<T>(N, hx, incx, &hr_gold);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        // abs_error for large N
-        real_t<T> abs_error = std::numeric_limits<real_t<T>>::epsilon() * cpu_result;
-        real_t<T> tolerance = 20.0;
-        abs_error *= tolerance;
+        auto compare_to_gold = [&] {
+            if(arg.unit_check)
+            {
+                // Near check for asum ILP64 bit
+                bool near_check = arg.initialization == rocblas_initialization::hpl;
+                if(near_check)
+                {
+                    double abs_error = sum_near_tolerance<T>(N, hr_gold);
+                    near_check_general<RT, RT>(1, 1, 1, &hr_gold, &rocblas_result, abs_error);
+                }
+                else
+                    unit_check_general<RT, RT>(1, 1, 1, &hr_gold, &rocblas_result);
+            }
 
-        // Near check for asum ILP64 bit
-        bool near_check = arg.initialization == rocblas_initialization::hpl;
+            double error = 0.0;
+            if(arg.norm_check)
+            {
+                error = std::abs((hr_gold - rocblas_result) / hr_gold);
+            }
+            return error;
+        };
 
         if(arg.pointer_mode_host)
         {
-            if(arg.unit_check)
-            {
-                if(near_check)
-                    near_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, &cpu_result, &rocblas_result_1, abs_error);
-                else
-                    unit_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, &cpu_result, &rocblas_result_1);
-            }
-
-            if(arg.norm_check)
-            {
-                rocblas_error_1 = std::abs((cpu_result - rocblas_result_1) / cpu_result);
-            }
+            error_host_ptr = compare_to_gold();
         }
 
         if(arg.pointer_mode_device)
         {
-            CHECK_HIP_ERROR(
-                hipMemcpy(&rocblas_result_2, dr, sizeof(real_t<T>), hipMemcpyDeviceToHost));
-
-            if(arg.unit_check)
-            {
-                if(near_check)
-                    near_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, &cpu_result, &rocblas_result_2, abs_error);
-                else
-                    unit_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, &cpu_result, &rocblas_result_2);
-            }
-
-            if(arg.norm_check)
-            {
-                rocblas_error_2 = std::abs((cpu_result - rocblas_result_2) / cpu_result);
-            }
+            CHECK_HIP_ERROR(hipMemcpy(&rocblas_result, dr, sizeof(RT), hipMemcpyDeviceToHost));
+            error_device_ptr = compare_to_gold();
         }
     }
 
@@ -201,6 +236,7 @@ void testing_asum(const Arguments& arg)
         double gpu_time_used;
         int    number_cold_calls = arg.cold_iters;
         int    total_calls       = number_cold_calls + arg.iters;
+
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
 
         hipStream_t stream;
@@ -222,7 +258,7 @@ void testing_asum(const Arguments& arg)
                                                  asum_gflop_count<T>(N),
                                                  asum_gflop_count<T>(N),
                                                  cpu_time_used,
-                                                 rocblas_error_1,
-                                                 rocblas_error_2);
+                                                 error_host_ptr,
+                                                 error_device_ptr);
     }
 }
