@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,37 +22,28 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "rocblas.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_asum_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_asum_batched_fn
-        = arg.api == FORTRAN ? rocblas_asum_batched<T, true> : rocblas_asum_batched<T, false>;
-    auto rocblas_asum_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_asum_batched_64<T, true>
-                                                            : rocblas_asum_batched_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_asum_batched<T, true> : rocblas_asum_batched<T, false>;
+    auto rocblas_asum_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_asum_batched_64<T, true>
+                                                              : rocblas_asum_batched_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
         rocblas_local_handle handle{arg};
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, pointer_mode));
 
-        int64_t    N                = 100;
-        int64_t    incx             = 1;
-        int64_t    batch_count      = 2;
-        real_t<T>  rocblas_result   = 10;
-        real_t<T>* h_rocblas_result = &rocblas_result;
+        int64_t N           = 100;
+        int64_t incx        = 1;
+        int64_t batch_count = 2;
+
+        using RT             = real_t<T>;
+        RT  rocblas_result   = 10;
+        RT* h_rocblas_result = &rocblas_result;
 
         // Allocate device memory
         device_batch_vector<T> dx(N, incx, batch_count);
@@ -77,44 +68,44 @@ template <typename T>
 void testing_asum_batched(const Arguments& arg)
 {
     auto rocblas_asum_batched_fn
-        = arg.api == FORTRAN ? rocblas_asum_batched<T, true> : rocblas_asum_batched<T, false>;
-    auto rocblas_asum_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_asum_batched_64<T, true>
-                                                            : rocblas_asum_batched_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_asum_batched<T, true> : rocblas_asum_batched<T, false>;
+    auto rocblas_asum_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_asum_batched_64<T, true>
+                                                              : rocblas_asum_batched_64<T, false>;
 
     int64_t N           = arg.N;
     int64_t incx        = arg.incx;
     int64_t batch_count = arg.batch_count;
 
-    double rocblas_error_1;
-    double rocblas_error_2;
+    using RT = real_t<T>;
+
+    double error_host_ptr;
+    double error_device_ptr;
 
     rocblas_local_handle handle{arg};
 
     // check to prevent undefined memory allocation error
     if(N <= 0 || incx <= 0 || batch_count <= 0)
     {
-        host_vector<real_t<T>> hr_1(std::max(int64_t(1), std::abs(batch_count)));
-        host_vector<real_t<T>> hr_2(std::max(int64_t(1), std::abs(batch_count)));
-        host_vector<real_t<T>> result_0(std::max(int64_t(1), std::abs(batch_count)));
-        CHECK_HIP_ERROR(hr_1.memcheck());
-        CHECK_HIP_ERROR(hr_2.memcheck());
+        host_vector<RT> rocblas_result(std::max(int64_t(1), std::abs(batch_count)));
+        host_vector<RT> result_0(std::max(int64_t(1), std::abs(batch_count)));
+        CHECK_HIP_ERROR(rocblas_result.memcheck());
         CHECK_HIP_ERROR(result_0.memcheck());
 
-        device_vector<real_t<T>> dr(std::max(int64_t(1), std::abs(batch_count)));
+        device_vector<RT> dr(std::max(int64_t(1), std::abs(batch_count)));
         CHECK_DEVICE_ALLOCATION(dr.memcheck());
 
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-        DAPI_CHECK(rocblas_asum_batched_fn, (handle, N, nullptr, incx, batch_count, hr_1));
+        DAPI_CHECK(rocblas_asum_batched_fn,
+                   (handle, N, nullptr, incx, batch_count, rocblas_result));
 
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
         DAPI_CHECK(rocblas_asum_batched_fn, (handle, N, nullptr, incx, batch_count, dr));
 
-        CHECK_HIP_ERROR(hr_2.transfer_from(dr));
-
         if(batch_count > 0)
         {
-            unit_check_general<real_t<T>, real_t<T>>(1, batch_count, 1, result_0, hr_1);
-            unit_check_general<real_t<T>, real_t<T>>(1, batch_count, 1, result_0, hr_2);
+            unit_check_general<RT, RT>(1, batch_count, 1, result_0, rocblas_result);
+            CHECK_HIP_ERROR(rocblas_result.transfer_from(dr));
+            unit_check_general<RT, RT>(1, batch_count, 1, result_0, rocblas_result);
         }
 
         return;
@@ -122,16 +113,16 @@ void testing_asum_batched(const Arguments& arg)
 
     // Naming: `h` is in CPU (host) memory(eg hx), `d` is in GPU (device) memory (eg dx).
     // Allocate host memory
-    host_batch_vector<T>   hx(N, incx, batch_count);
-    host_vector<real_t<T>> hr_1(batch_count);
-    host_vector<real_t<T>> hr_2(batch_count);
+    host_batch_vector<T> hx(N, incx, batch_count);
+    host_vector<RT>      rocblas_result(batch_count);
+    host_vector<RT>      hr_gold(batch_count);
 
     // Check host memory allocation
     CHECK_HIP_ERROR(hx.memcheck());
 
     // Allocate device memory
-    device_batch_vector<T>   dx(N, incx, batch_count);
-    device_vector<real_t<T>> dr(batch_count);
+    device_batch_vector<T> dx(N, incx, batch_count);
+    device_vector<RT>      dr(batch_count);
 
     // Check device memory allocation
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
@@ -150,7 +141,7 @@ void testing_asum_batched(const Arguments& arg)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
             DAPI_CHECK(rocblas_asum_batched_fn,
-                       (handle, N, dx.ptr_on_device(), incx, batch_count, hr_1));
+                       (handle, N, dx.ptr_on_device(), incx, batch_count, rocblas_result));
         }
 
         if(arg.pointer_mode_device)
@@ -160,60 +151,96 @@ void testing_asum_batched(const Arguments& arg)
             DAPI_CHECK(rocblas_asum_batched_fn,
                        (handle, N, dx.ptr_on_device(), incx, batch_count, dr));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<RT> hr_copy(batch_count);
+                // Transfer from device to host.
+                CHECK_HIP_ERROR(rocblas_result.transfer_from(dr));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_vector<T> dx_copy(N, incx, batch_count);
+                    device_vector<RT>      dr_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dr_copy.memcheck());
+
+                    // Transfer from host to device.
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(
+                            rocblas_asum_batched_fn,
+                            (handle_copy, N, dx_copy.ptr_on_device(), incx, batch_count, dr_copy));
+                        // Transfer from device to host.
+                        CHECK_HIP_ERROR(hr_copy.transfer_from(dr_copy));
+                        unit_check_general<RT, RT>(1, batch_count, 1, rocblas_result, hr_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
-        real_t<T> cpu_result[batch_count];
-
         cpu_time_used = get_time_us_no_sync();
         for(size_t b = 0; b < batch_count; b++)
         {
-            cblas_asum<T>(N, hx[b], incx, cpu_result + b);
+            ref_asum<T>(N, hx[b], incx, hr_gold + b);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        real_t<T> abs_error = std::numeric_limits<real_t<T>>::epsilon() * cpu_result[0];
-        real_t<T> tolerance = 20.0;
-        abs_error *= tolerance;
+        auto compare_to_gold = [&] {
+            if(arg.unit_check)
+            {
+                // Near check for asum ILP64 bit
+                bool near_check = arg.initialization == rocblas_initialization::hpl;
+                if(near_check)
+                {
+                    for(int64_t b = 0; b < batch_count; ++b)
+                    {
+                        double abs_error = sum_near_tolerance<T>(N, hr_gold[b]);
+                        near_check_general<RT, RT>(
+                            1, 1, 1, hr_gold + b, rocblas_result + b, abs_error);
+                    }
+                }
+                else
+                    unit_check_general<RT, RT>(1, batch_count, 1, hr_gold, rocblas_result);
+            }
 
-        // Near check for asum ILP64 bit
-        bool near_check = arg.initialization == rocblas_initialization::hpl;
+            double error = 0.0;
+            if(arg.norm_check)
+            {
+                error = std::abs((hr_gold[0] - rocblas_result[0]) / hr_gold[0]);
+            }
+            return error;
+        };
 
         if(arg.pointer_mode_host)
         {
-            if(arg.unit_check)
-            {
-                if(near_check)
-                    near_check_general<real_t<T>, real_t<T>>(
-                        batch_count, 1, 1, cpu_result, hr_1, abs_error);
-                else
-                    unit_check_general<real_t<T>, real_t<T>>(1, batch_count, 1, cpu_result, hr_1);
-            }
-
-            if(arg.norm_check)
-            {
-                rocblas_error_1 = std::abs((cpu_result[0] - hr_1[0]) / cpu_result[0]);
-            }
+            error_host_ptr = compare_to_gold();
         }
 
         if(arg.pointer_mode_device)
         {
             // Transfer from device to host.
-            CHECK_HIP_ERROR(hr_2.transfer_from(dr));
-
-            if(arg.unit_check)
-            {
-                if(near_check)
-                    near_check_general<real_t<T>, real_t<T>>(
-                        batch_count, 1, 1, cpu_result, hr_2, abs_error);
-                else
-                    unit_check_general<real_t<T>, real_t<T>>(1, batch_count, 1, cpu_result, hr_2);
-            }
-
-            if(arg.norm_check)
-            {
-                rocblas_error_2 = std::abs((cpu_result[0] - hr_2[0]) / cpu_result[0]);
-            }
+            CHECK_HIP_ERROR(rocblas_result.transfer_from(dr));
+            error_device_ptr = compare_to_gold();
         }
     }
 
@@ -245,7 +272,7 @@ void testing_asum_batched(const Arguments& arg)
                                                                 asum_gflop_count<T>(N),
                                                                 asum_gbyte_count<T>(N),
                                                                 cpu_time_used,
-                                                                rocblas_error_1,
-                                                                rocblas_error_2);
+                                                                error_host_ptr,
+                                                                error_device_ptr);
     }
 }
