@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #include "bytes.hpp"
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "norm.hpp"
 #include "rocblas.hpp"
@@ -33,16 +34,15 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 /* ============================================================================================ */
 template <typename T>
 void testing_axpy_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_axpy_batched_fn
-        = arg.api == FORTRAN ? rocblas_axpy_batched<T, true> : rocblas_axpy_batched<T, false>;
-    auto rocblas_axpy_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_axpy_batched_64<T, true>
-                                                            : rocblas_axpy_batched_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched<T, true> : rocblas_axpy_batched<T, false>;
+    auto rocblas_axpy_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_64<T, true>
+                                                              : rocblas_axpy_batched_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -116,9 +116,9 @@ template <typename T>
 void testing_axpy_batched(const Arguments& arg)
 {
     auto rocblas_axpy_batched_fn
-        = arg.api == FORTRAN ? rocblas_axpy_batched<T, true> : rocblas_axpy_batched<T, false>;
-    auto rocblas_axpy_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_axpy_batched_64<T, true>
-                                                            : rocblas_axpy_batched_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched<T, true> : rocblas_axpy_batched<T, false>;
+    auto rocblas_axpy_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_64<T, true>
+                                                              : rocblas_axpy_batched_64<T, false>;
 
     rocblas_local_handle handle{arg};
     int64_t              N = arg.N, incx = arg.incx, incy = arg.incy, batch_count = arg.batch_count;
@@ -208,6 +208,56 @@ void testing_axpy_batched(const Arguments& arg)
         // Transfer from device to host.
         CHECK_HIP_ERROR(hy_2.transfer_from(dy));
 
+        if(arg.repeatability_check)
+        {
+            host_batch_vector<T> hy_copy(N, incy, batch_count);
+
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                //Allocate device memory in new device
+                device_batch_vector<T> dx_copy(N, incx, batch_count);
+                device_batch_vector<T> dy_copy(N, incy, batch_count);
+                device_vector<T>       dalpha_copy(1);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dalpha_copy.memcheck());
+
+                CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                CHECK_HIP_ERROR(dalpha_copy.transfer_from(halpha));
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                    DAPI_CHECK(rocblas_axpy_batched_fn,
+                               (handle_copy,
+                                N,
+                                dalpha_copy,
+                                dx_copy.ptr_on_device(),
+                                incx,
+                                dy_copy.ptr_on_device(),
+                                incy,
+                                batch_count));
+                    CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                    unit_check_general<T>(1, N, incy, hy_2, hy_copy, batch_count);
+                }
+            }
+            return;
+        }
+
         // CPU BLAS
         {
             cpu_time_used = get_time_us_no_sync();
@@ -215,7 +265,7 @@ void testing_axpy_batched(const Arguments& arg)
             // Compute the host solution.
             for(int64_t batch_index = 0; batch_index < batch_count; ++batch_index)
             {
-                cblas_axpy<T>(N, h_alpha, hx[batch_index], incx, hy_gold[batch_index], incy);
+                ref_axpy<T>(N, h_alpha, hx[batch_index], incx, hy_gold[batch_index], incy);
             }
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
         }
