@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,7 @@
 
 #pragma once
 
+#include "benchmark.hpp"
 #include "testing_common.hpp"
 
 template <typename T>
@@ -60,6 +61,7 @@ void testing_copy(const Arguments& arg)
     int64_t              N    = arg.N;
     int64_t              incx = arg.incx;
     int64_t              incy = arg.incy;
+    bool                 HMM  = arg.HMM;
     rocblas_local_handle handle{arg};
 
     // Argument sanity check before allocating invalid memory
@@ -75,31 +77,30 @@ void testing_copy(const Arguments& arg)
     host_vector<T> hy(N, incy);
     host_vector<T> hy_gold(N, incy);
 
-    // Allocate device memory
-    device_vector<T> dx(N, incx);
-    device_vector<T> dy(N, incy);
-
-    // Check device memory allocation
-    CHECK_DEVICE_ALLOCATION(dx.memcheck());
-    CHECK_DEVICE_ALLOCATION(dy.memcheck());
-
     // Initialize data on host memory
     rocblas_init_vector(hx, arg, rocblas_client_alpha_sets_nan, true);
     rocblas_init_vector(hy, arg, rocblas_client_alpha_sets_nan, false);
 
-    // copy vector is easy in STL; hy_gold = hx: save a copy in hy_gold which will be output of CPU
-    // BLAS
+    // save a copy in hy_gold which will be output of CPU BLAS
     hy_gold = hy;
 
-    // copy data from CPU to device
-    CHECK_HIP_ERROR(dx.transfer_from(hx));
-    CHECK_HIP_ERROR(dy.transfer_from(hy));
-
     double cpu_time_used;
     double rocblas_error = 0.0;
 
     if(arg.unit_check || arg.norm_check)
     {
+        // Allocate device memory
+        device_vector<T> dx(N, incx, HMM);
+        device_vector<T> dy(N, incy, HMM);
+
+        // Check device memory allocation
+        CHECK_DEVICE_ALLOCATION(dx.memcheck());
+        CHECK_DEVICE_ALLOCATION(dy.memcheck());
+
+        // copy data from CPU to device
+        CHECK_HIP_ERROR(dx.transfer_from(hx));
+        CHECK_HIP_ERROR(dy.transfer_from(hy));
+
         handle.pre_test(arg);
         // GPU BLAS
         DAPI_CHECK(rocblas_copy_fn, (handle, N, dx, incx, dy, incy));
@@ -109,7 +110,7 @@ void testing_copy(const Arguments& arg)
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        cblas_copy<T>(N, hx, incx, hy_gold, incy);
+        ref_copy<T>(N, hx, incx, hy_gold, incy);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         if(arg.unit_check)
@@ -125,22 +126,43 @@ void testing_copy(const Arguments& arg)
 
     if(arg.timing)
     {
+        size_t aligned_stride_x = align_stride<T>(size_t(N) * (incx >= 0 ? incx : -incx));
+        size_t aligned_stride_y = align_stride<T>(size_t(N) * (incy >= 0 ? incy : -incy));
+
+        size_t x_size          = N * incx * sizeof(T);
+        size_t y_size          = N * incy * sizeof(T);
+        size_t x_y_cached_size = x_size + y_size;
+
+        size_t flush_batch_count = calculate_flush_batch_count(
+            arg.flush_batch_count, arg.flush_memory_size, x_y_cached_size);
+
+        // allocate device rotating buffer arrays
+        device_strided_batch_vector<T> dx_rot_buff(
+            N, incx, aligned_stride_x, flush_batch_count, HMM);
+        device_strided_batch_vector<T> dy_rot_buff(
+            N, incy, aligned_stride_y, flush_batch_count, HMM);
+
+        CHECK_DEVICE_ALLOCATION(dx_rot_buff.memcheck());
+        CHECK_DEVICE_ALLOCATION(dy_rot_buff.memcheck());
+
+        CHECK_HIP_ERROR(dx_rot_buff.broadcast_one_vector_from(hx));
+        CHECK_HIP_ERROR(dy_rot_buff.broadcast_one_vector_from(hy));
+
         double gpu_time_used;
-        int    number_cold_calls = arg.cold_iters;
-        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
 
-        for(int iter = 0; iter < total_calls; iter++)
-        {
-            if(iter == number_cold_calls)
-                gpu_time_used = get_time_us_sync(stream);
+        auto lambda_to_benchmark = [&](int flush_index) {
+            DAPI_DISPATCH(
+                rocblas_copy_fn,
+                (handle, N, dx_rot_buff[flush_index], incx, dy_rot_buff[flush_index], incy));
+        };
 
-            DAPI_DISPATCH(rocblas_copy_fn, (handle, N, dx, incx, dy, incy));
-        }
+        Benchmark<decltype(lambda_to_benchmark)> benchmark_copy(
+            lambda_to_benchmark, stream, arg, flush_batch_count);
 
-        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+        gpu_time_used = benchmark_copy.timer();
 
         ArgumentModel<e_N, e_incx, e_incy>{}.log_args<T>(rocblas_cout,
                                                          arg,
