@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,10 +27,10 @@
 template <typename T>
 void testing_copy_strided_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_copy_strided_batched_fn    = arg.api == FORTRAN
+    auto rocblas_copy_strided_batched_fn    = arg.api & c_API_FORTRAN
                                                   ? rocblas_copy_strided_batched<T, true>
                                                   : rocblas_copy_strided_batched<T, false>;
-    auto rocblas_copy_strided_batched_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_copy_strided_batched_fn_64 = arg.api & c_API_FORTRAN
                                                   ? rocblas_copy_strided_batched_64<T, true>
                                                   : rocblas_copy_strided_batched_64<T, false>;
 
@@ -69,10 +69,10 @@ void testing_copy_strided_batched_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_copy_strided_batched(const Arguments& arg)
 {
-    auto rocblas_copy_strided_batched_fn    = arg.api == FORTRAN
+    auto rocblas_copy_strided_batched_fn    = arg.api & c_API_FORTRAN
                                                   ? rocblas_copy_strided_batched<T, true>
                                                   : rocblas_copy_strided_batched<T, false>;
-    auto rocblas_copy_strided_batched_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_copy_strided_batched_fn_64 = arg.api & c_API_FORTRAN
                                                   ? rocblas_copy_strided_batched_64<T, true>
                                                   : rocblas_copy_strided_batched_64<T, false>;
 
@@ -138,7 +138,7 @@ void testing_copy_strided_batched(const Arguments& arg)
         cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < batch_count; ++b)
         {
-            cblas_copy<T>(N, hx[b], incx, hy_gold[b], incy);
+            ref_copy<T>(N, hx[b], incx, hy_gold[b], incy);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
