@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,10 +29,10 @@
 template <typename T, bool CONJ = false>
 void testing_dot_bad_arg(const Arguments& arg)
 {
-    auto rocblas_dot_fn    = arg.api == FORTRAN
+    auto rocblas_dot_fn    = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc<T, true> : rocblas_dot<T, true>)
                                  : (CONJ ? rocblas_dotc<T, false> : rocblas_dot<T, false>);
-    auto rocblas_dot_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_dot_fn_64 = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc_64<T, true> : rocblas_dot_64<T, true>)
                                  : (CONJ ? rocblas_dotc_64<T, false> : rocblas_dot_64<T, false>);
 
@@ -72,19 +72,13 @@ void testing_dot_bad_arg(const Arguments& arg)
     }
 }
 
-template <typename T>
-void testing_dotc_bad_arg(const Arguments& arg)
-{
-    testing_dot_bad_arg<T, true>(arg);
-}
-
 template <typename T, bool CONJ = false>
 void testing_dot(const Arguments& arg)
 {
-    auto rocblas_dot_fn    = arg.api == FORTRAN
+    auto rocblas_dot_fn    = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc<T, true> : rocblas_dot<T, true>)
                                  : (CONJ ? rocblas_dotc<T, false> : rocblas_dot<T, false>);
-    auto rocblas_dot_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_dot_fn_64 = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc_64<T, true> : rocblas_dot_64<T, true>)
                                  : (CONJ ? rocblas_dotc_64<T, false> : rocblas_dot_64<T, false>);
 
@@ -177,6 +171,66 @@ void testing_dot(const Arguments& arg)
             {
                 DAPI_CHECK(rocblas_dot_fn,
                            (handle, N, dx, incx, dy_ptr, incy, d_rocblas_result_device));
+
+                if(arg.repeatability_check)
+                {
+                    T rocblas_result_device_copy;
+
+                    CHECK_HIP_ERROR(hipMemcpy(&rocblas_result_device,
+                                              d_rocblas_result_device,
+                                              sizeof(T),
+                                              hipMemcpyDeviceToHost));
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        //Allocate device memory in new device
+                        device_vector<T> dx_copy(N, incx, HMM);
+                        device_vector<T> dy_copy(N, incy, HMM);
+                        device_vector<T> d_rocblas_result_device_copy(1, 1, HMM);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(d_rocblas_result_device_copy.memcheck());
+
+                        CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                        auto dy_ptr_copy = (arg.algo) ? (T*)(dx_copy) : (T*)(dy_copy);
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            DAPI_CHECK(rocblas_dot_fn,
+                                       (handle_copy,
+                                        N,
+                                        dx_copy,
+                                        incx,
+                                        dy_ptr_copy,
+                                        incy,
+                                        d_rocblas_result_device_copy));
+                            CHECK_HIP_ERROR(hipMemcpy(&rocblas_result_device_copy,
+                                                      d_rocblas_result_device_copy,
+                                                      sizeof(T),
+                                                      hipMemcpyDeviceToHost));
+                            unit_check_general<T>(
+                                1, 1, 1, &rocblas_result_device, &rocblas_result_device_copy);
+                        }
+                    }
+                    return;
+                }
             }
             else if constexpr(std::is_same_v<T, float>)
             {
@@ -202,7 +256,7 @@ void testing_dot(const Arguments& arg)
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        (CONJ ? cblas_dotc<T> : cblas_dot<T>)(N, hx, incx, hy_ptr, incy, &cpu_result);
+        (CONJ ? ref_dotc<T, T> : ref_dot<T, T>)(N, hx, incx, hy_ptr, incy, &cpu_result);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         // For large N, rocblas_half tends to diverge proportional to N
@@ -289,9 +343,3 @@ void testing_dot(const Arguments& arg)
                                                                  rocblas_error_device);
     }
 }
-
-template <typename T>
-void testing_dotc(const Arguments& arg)
-{
-    testing_dot<T, true>(arg);
-}
