@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,16 +27,16 @@
 template <typename T, bool CONJ = false>
 void testing_dot_strided_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_dot_strided_batched_fn = arg.api == FORTRAN
+    auto rocblas_dot_strided_batched_fn = arg.api & c_API_FORTRAN
                                               ? (CONJ ? rocblas_dotc_strided_batched<T, true>
                                                       : rocblas_dot_strided_batched<T, true>)
                                               : (CONJ ? rocblas_dotc_strided_batched<T, false>
                                                       : rocblas_dot_strided_batched<T, false>);
     auto rocblas_dot_strided_batched_fn_64
-        = arg.api == FORTRAN_64 ? (CONJ ? rocblas_dotc_strided_batched_64<T, true>
-                                        : rocblas_dot_strided_batched_64<T, true>)
-                                : (CONJ ? rocblas_dotc_strided_batched_64<T, false>
-                                        : rocblas_dot_strided_batched_64<T, false>);
+        = arg.api & c_API_FORTRAN ? (CONJ ? rocblas_dotc_strided_batched_64<T, true>
+                                          : rocblas_dot_strided_batched_64<T, true>)
+                                  : (CONJ ? rocblas_dotc_strided_batched_64<T, false>
+                                          : rocblas_dot_strided_batched_64<T, false>);
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -95,25 +95,19 @@ void testing_dot_strided_batched_bad_arg(const Arguments& arg)
     }
 }
 
-template <typename T>
-void testing_dotc_strided_batched_bad_arg(const Arguments& arg)
-{
-    testing_dot_strided_batched_bad_arg<T, true>(arg);
-}
-
 template <typename T, bool CONJ = false>
 void testing_dot_strided_batched(const Arguments& arg)
 {
-    auto rocblas_dot_strided_batched_fn = arg.api == FORTRAN
+    auto rocblas_dot_strided_batched_fn = arg.api & c_API_FORTRAN
                                               ? (CONJ ? rocblas_dotc_strided_batched<T, true>
                                                       : rocblas_dot_strided_batched<T, true>)
                                               : (CONJ ? rocblas_dotc_strided_batched<T, false>
                                                       : rocblas_dot_strided_batched<T, false>);
     auto rocblas_dot_strided_batched_fn_64
-        = arg.api == FORTRAN_64 ? (CONJ ? rocblas_dotc_strided_batched_64<T, true>
-                                        : rocblas_dot_strided_batched_64<T, true>)
-                                : (CONJ ? rocblas_dotc_strided_batched_64<T, false>
-                                        : rocblas_dot_strided_batched_64<T, false>);
+        = arg.api & c_API_FORTRAN ? (CONJ ? rocblas_dotc_strided_batched_64<T, true>
+                                          : rocblas_dot_strided_batched_64<T, true>)
+                                  : (CONJ ? rocblas_dotc_strided_batched_64<T, false>
+                                          : rocblas_dot_strided_batched_64<T, false>);
 
     int64_t        N           = arg.N;
     int64_t        incx        = arg.incx;
@@ -233,6 +227,7 @@ void testing_dot_strided_batched(const Arguments& arg)
         {
             // GPU BLAS, rocblas_pointer_mode_device
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
+
             handle.pre_test(arg);
             DAPI_CHECK(rocblas_dot_strided_batched_fn,
                        (handle,
@@ -246,14 +241,78 @@ void testing_dot_strided_batched(const Arguments& arg)
                         batch_count,
                         d_rocblas_result));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                CHECK_HIP_ERROR(rocblas_result_device.transfer_from(d_rocblas_result));
+                host_vector<T> rocblas_result_device_copy(batch_count);
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory in new device
+                    device_strided_batch_vector<T> dx_copy(N, incx, stride_x, batch_count);
+                    device_strided_batch_vector<T> dy_copy(N, incy, stride_y, batch_count);
+                    device_vector<T>               d_rocblas_result_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                    auto dy_ptr_copy = (arg.algo) ? (T*)(dx_copy) : (T*)(dy_copy);
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_dot_strided_batched_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy,
+                                    incx,
+                                    stride_x,
+                                    dy_ptr_copy,
+                                    incy,
+                                    stride_y,
+                                    batch_count,
+                                    d_rocblas_result_copy));
+
+                        CHECK_HIP_ERROR(
+                            rocblas_result_device_copy.transfer_from(d_rocblas_result_copy));
+
+                        unit_check_general<T>(1,
+                                              1,
+                                              1,
+                                              1,
+                                              rocblas_result_device,
+                                              rocblas_result_device_copy,
+                                              batch_count);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
         for(size_t b = 0; b < batch_count; ++b)
         {
-            (CONJ ? cblas_dotc<T>
-                  : cblas_dot<T>)(N, hx[b], incx, hy_ptr + b * stride_y, incy, &cpu_result[b]);
+            (CONJ ? ref_dotc<T, T>
+                  : ref_dot<T, T>)(N, hx[b], incx, hy_ptr + b * stride_y, incy, &cpu_result[b]);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -358,9 +417,3 @@ void testing_dot_strided_batched(const Arguments& arg)
                          rocblas_error_device);
     }
 }
-
-template <typename T>
-void testing_dotc_strided_batched(const Arguments& arg)
-{
-    testing_dot_strided_batched<T, true>(arg);
-}
