@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -129,6 +129,52 @@ void testing_iamax_iamin(const Arguments& arg, FUNC func)
             handle.pre_test(arg);
             CHECK_ROCBLAS_ERROR(func(handle, N, dx, incx, d_rocblas_result));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                R h_rocblas_result_copy;
+                CHECK_HIP_ERROR(hipMemcpy(
+                    &h_rocblas_result_device, d_rocblas_result, sizeof(R), hipMemcpyDeviceToHost));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory in new device
+                    device_vector<T> dx_copy(N, incx);
+                    device_vector<R> d_rocblas_result_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_ROCBLAS_ERROR(
+                            func(handle_copy, N, dx_copy, incx, d_rocblas_result_copy));
+                        CHECK_HIP_ERROR(hipMemcpy(&h_rocblas_result_copy,
+                                                  d_rocblas_result_copy,
+                                                  sizeof(R),
+                                                  hipMemcpyDeviceToHost));
+                        unit_check_general<R>(
+                            1, 1, 1, &h_rocblas_result_device, &h_rocblas_result_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
@@ -207,9 +253,10 @@ void testing_iamax_iamin(const Arguments& arg, FUNC func)
 template <typename T>
 void testing_iamax_bad_arg(const Arguments& arg)
 {
-    auto rocblas_iamax_fn = arg.api == FORTRAN ? rocblas_iamax<T, true> : rocblas_iamax<T, false>;
+    auto rocblas_iamax_fn
+        = arg.api & c_API_FORTRAN ? rocblas_iamax<T, true> : rocblas_iamax<T, false>;
     auto rocblas_iamax_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_iamax_64<T, true> : rocblas_iamax_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_iamax_64<T, true> : rocblas_iamax_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_bad_arg<T, int64_t>(arg, rocblas_iamax_fn_64);
@@ -220,9 +267,10 @@ void testing_iamax_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_iamin_bad_arg(const Arguments& arg)
 {
-    auto rocblas_iamin_fn = arg.api == FORTRAN ? rocblas_iamin<T, true> : rocblas_iamin<T, false>;
+    auto rocblas_iamin_fn
+        = arg.api & c_API_FORTRAN ? rocblas_iamin<T, true> : rocblas_iamin<T, false>;
     auto rocblas_iamin_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_iamin_64<T, true> : rocblas_iamin_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_iamin_64<T, true> : rocblas_iamin_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_bad_arg<T, int64_t>(arg, rocblas_iamin_fn_64);
@@ -233,9 +281,10 @@ void testing_iamin_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_iamax(const Arguments& arg)
 {
-    auto rocblas_iamax_fn = arg.api == FORTRAN ? rocblas_iamax<T, true> : rocblas_iamax<T, false>;
+    auto rocblas_iamax_fn
+        = arg.api & c_API_FORTRAN ? rocblas_iamax<T, true> : rocblas_iamax<T, false>;
     auto rocblas_iamax_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_iamax_64<T, true> : rocblas_iamax_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_iamax_64<T, true> : rocblas_iamax_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin<T, rocblas_iamax_iamin_ref::iamax<T>, int64_t>(arg,
@@ -248,9 +297,10 @@ void testing_iamax(const Arguments& arg)
 template <typename T>
 void testing_iamin(const Arguments& arg)
 {
-    auto rocblas_iamin_fn = arg.api == FORTRAN ? rocblas_iamin<T, true> : rocblas_iamin<T, false>;
+    auto rocblas_iamin_fn
+        = arg.api & c_API_FORTRAN ? rocblas_iamin<T, true> : rocblas_iamin<T, false>;
     auto rocblas_iamin_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_iamin_64<T, true> : rocblas_iamin_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_iamin_64<T, true> : rocblas_iamin_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin<T, rocblas_iamax_iamin_ref::iamin<T>, int64_t>(arg,
