@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -136,6 +136,46 @@ void testing_iamax_iamin_batched(const Arguments& arg, FUNC func)
             handle.pre_test(arg);
             CHECK_ROCBLAS_ERROR(func(handle, N, dx.ptr_on_device(), incx, batch_count, dr));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<R> hr_copy(batch_count);
+                CHECK_HIP_ERROR(hr2.transfer_from(dr));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+                    // Allocate device memory in new device
+                    device_batch_vector<T> dx_copy(N, incx, batch_count);
+                    device_vector<R>       dr_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dr_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_ROCBLAS_ERROR(func(
+                            handle_copy, N, dx_copy.ptr_on_device(), incx, batch_count, dr_copy));
+                        CHECK_HIP_ERROR(hr_copy.transfer_from(dr_copy));
+                        unit_check_general<R>(batch_count, 1, 1, hr2, hr_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
@@ -229,10 +269,10 @@ void testing_iamax_iamin_batched(const Arguments& arg, FUNC func)
 template <typename T>
 void testing_iamax_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_iamax_batched_fn
-        = arg.api == FORTRAN ? rocblas_iamax_batched<T, true> : rocblas_iamax_batched<T, false>;
-    auto rocblas_iamax_batched_fn_64 = arg.api == FORTRAN ? rocblas_iamax_batched_64<T, true>
-                                                          : rocblas_iamax_batched_64<T, false>;
+    auto rocblas_iamax_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_iamax_batched<T, true>
+                                                               : rocblas_iamax_batched<T, false>;
+    auto rocblas_iamax_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_iamax_batched_64<T, true>
+                                                               : rocblas_iamax_batched_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_batched_bad_arg<T, int64_t>(arg, rocblas_iamax_batched_fn_64);
@@ -243,11 +283,11 @@ void testing_iamax_batched_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_iamin_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_iamin_batched_fn
-        = arg.api == FORTRAN ? rocblas_iamin_batched<T, true> : rocblas_iamin_batched<T, false>;
+    auto rocblas_iamin_batched_fn = arg.api & c_API_FORTRAN ? rocblas_iamin_batched<T, true>
+                                                            : rocblas_iamin_batched<T, false>;
 
-    auto rocblas_iamin_batched_fn_64 = arg.api == FORTRAN ? rocblas_iamin_batched_64<T, true>
-                                                          : rocblas_iamin_batched_64<T, false>;
+    auto rocblas_iamin_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_iamin_batched_64<T, true>
+                                                               : rocblas_iamin_batched_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_batched_bad_arg<T, int64_t>(arg, rocblas_iamin_batched_fn_64);
@@ -258,10 +298,10 @@ void testing_iamin_batched_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_iamax_batched(const Arguments& arg)
 {
-    auto rocblas_iamax_batched_fn
-        = arg.api == FORTRAN ? rocblas_iamax_batched<T, true> : rocblas_iamax_batched<T, false>;
-    auto rocblas_iamax_batched_fn_64 = arg.api == FORTRAN ? rocblas_iamax_batched_64<T, true>
-                                                          : rocblas_iamax_batched_64<T, false>;
+    auto rocblas_iamax_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_iamax_batched<T, true>
+                                                               : rocblas_iamax_batched<T, false>;
+    auto rocblas_iamax_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_iamax_batched_64<T, true>
+                                                               : rocblas_iamax_batched_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_batched<T, rocblas_iamax_iamin_ref::iamax<T>, int64_t>(
@@ -274,10 +314,10 @@ void testing_iamax_batched(const Arguments& arg)
 template <typename T>
 void testing_iamin_batched(const Arguments& arg)
 {
-    auto rocblas_iamin_batched_fn
-        = arg.api == FORTRAN ? rocblas_iamin_batched<T, true> : rocblas_iamin_batched<T, false>;
-    auto rocblas_iamin_batched_fn_64 = arg.api == FORTRAN ? rocblas_iamin_batched_64<T, true>
-                                                          : rocblas_iamin_batched_64<T, false>;
+    auto rocblas_iamin_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_iamin_batched<T, true>
+                                                               : rocblas_iamin_batched<T, false>;
+    auto rocblas_iamin_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_iamin_batched_64<T, true>
+                                                               : rocblas_iamin_batched_64<T, false>;
 
     if(arg.api & c_API_64)
         testing_iamax_iamin_batched<T, rocblas_iamax_iamin_ref::iamin<T>, int64_t>(
