@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,10 +28,10 @@ template <typename T>
 void testing_nrm2_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_nrm2_batched_fn
-        = arg.api == FORTRAN ? rocblas_nrm2_batched<T, true> : rocblas_nrm2_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched<T, true> : rocblas_nrm2_batched<T, false>;
 
-    auto rocblas_nrm2_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_nrm2_batched_64<T, true>
-                                                            : rocblas_nrm2_batched_64<T, false>;
+    auto rocblas_nrm2_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_64<T, true>
+                                                              : rocblas_nrm2_batched_64<T, false>;
 
     int64_t N           = 100;
     int64_t incx        = 1;
@@ -65,17 +65,17 @@ void testing_nrm2_batched(const Arguments& arg)
 {
 
     auto rocblas_nrm2_batched_fn
-        = arg.api == FORTRAN ? rocblas_nrm2_batched<T, true> : rocblas_nrm2_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched<T, true> : rocblas_nrm2_batched<T, false>;
 
-    auto rocblas_nrm2_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_nrm2_batched_64<T, true>
-                                                            : rocblas_nrm2_batched_64<T, false>;
+    auto rocblas_nrm2_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_64<T, true>
+                                                              : rocblas_nrm2_batched_64<T, false>;
 
     int64_t N           = arg.N;
     int64_t incx        = arg.incx;
     int64_t batch_count = arg.batch_count;
 
-    double rocblas_error_1;
-    double rocblas_error_2;
+    double error_host_ptr;
+    double error_device_ptr;
 
     rocblas_local_handle handle{arg};
 
@@ -120,8 +120,7 @@ void testing_nrm2_batched(const Arguments& arg)
     // Naming: `h` is in CPU (host) memory(eg hx), `d` is in GPU (device) memory (eg dx).
     // Allocate host memory
     host_batch_vector<T>   hx(N, incx, batch_count);
-    host_vector<real_t<T>> rocblas_result_1(batch_count);
-    host_vector<real_t<T>> rocblas_result_2(batch_count);
+    host_vector<real_t<T>> rocblas_result(batch_count);
     host_vector<real_t<T>> cpu_result(batch_count);
 
     // Check host memory allocation
@@ -129,10 +128,10 @@ void testing_nrm2_batched(const Arguments& arg)
 
     // Allocate device memory
     device_batch_vector<T>   dx(N, incx, batch_count);
-    device_vector<real_t<T>> d_rocblas_result_2(batch_count);
+    device_vector<real_t<T>> d_rocblas_result(batch_count);
 
     // Check device memory allocation
-    CHECK_DEVICE_ALLOCATION(d_rocblas_result_2.memcheck());
+    CHECK_DEVICE_ALLOCATION(d_rocblas_result.memcheck());
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
     // Initialize memory on host.
@@ -148,7 +147,7 @@ void testing_nrm2_batched(const Arguments& arg)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
             DAPI_CHECK(rocblas_nrm2_batched_fn,
-                       (handle, N, dx.ptr_on_device(), incx, batch_count, rocblas_result_1));
+                       (handle, N, dx.ptr_on_device(), incx, batch_count, rocblas_result));
         }
 
         if(arg.pointer_mode_device)
@@ -156,74 +155,98 @@ void testing_nrm2_batched(const Arguments& arg)
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
             handle.pre_test(arg);
             DAPI_CHECK(rocblas_nrm2_batched_fn,
-                       (handle, N, dx.ptr_on_device(), incx, batch_count, d_rocblas_result_2));
+                       (handle, N, dx.ptr_on_device(), incx, batch_count, d_rocblas_result));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<real_t<T>> rocblas_result_copy(batch_count);
+                CHECK_HIP_ERROR(rocblas_result.transfer_from(d_rocblas_result));
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory in new device
+                    device_batch_vector<T>   dx_copy(N, incx, batch_count);
+                    device_vector<real_t<T>> d_rocblas_result_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_nrm2_batched_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy.ptr_on_device(),
+                                    incx,
+                                    batch_count,
+                                    d_rocblas_result_copy));
+                        CHECK_HIP_ERROR(rocblas_result_copy.transfer_from(d_rocblas_result_copy));
+                        unit_check_general<real_t<T>, real_t<T>>(
+                            batch_count, 1, 1, rocblas_result, rocblas_result_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        for(int b = 0; b < batch_count; b++)
+        for(int64_t b = 0; b < batch_count; b++)
         {
-            cblas_nrm2<T>(N, hx[b], incx, cpu_result + b);
+            ref_nrm2<T>(N, hx[b], incx, cpu_result + b);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        real_t<T> abs_result = cpu_result[0] > 0 ? cpu_result[0] : -cpu_result[0];
-        real_t<T> abs_error;
-        if(abs_result > 0)
-        {
-            abs_error = std::numeric_limits<real_t<T>>::epsilon() * N * abs_result;
-        }
-        else
-        {
-            abs_error = std::numeric_limits<real_t<T>>::epsilon() * N;
-        }
-        real_t<T> tolerance = 2.0; //  accounts for rounding in reduction sum. depends on n.
-            //  If test fails, try decreasing n or increasing tolerance.
-        abs_error *= tolerance;
-
-        if(arg.pointer_mode_host)
-        {
+        auto compare_to_gold = [&] {
             if(!rocblas_isnan(arg.alpha))
             {
                 if(arg.unit_check)
                 {
-                    near_check_general<real_t<T>, real_t<T>>(
-                        batch_count, 1, 1, cpu_result, rocblas_result_1, abs_error);
+                    for(int64_t b = 0; b < batch_count; ++b)
+                    {
+                        double abs_error = sum_near_tolerance<T>(N, cpu_result[b]);
+                        near_check_general<real_t<T>, real_t<T>>(
+                            1, 1, 1, cpu_result + b, rocblas_result + b, abs_error);
+                    }
                 }
             }
 
+            double error = 0.0;
             if(arg.norm_check)
             {
-                for(int b = 0; b < batch_count; ++b)
+                for(int64_t b = 0; b < batch_count; ++b)
                 {
-                    rocblas_error_1
-                        += rocblas_abs((cpu_result[b] - rocblas_result_1[b]) / cpu_result[b]);
+                    error += rocblas_abs((cpu_result[b] - rocblas_result[b]) / cpu_result[b]);
                 }
             }
+            return error;
+        };
+
+        if(arg.pointer_mode_host)
+        {
+            error_host_ptr = compare_to_gold();
         }
 
         if(arg.pointer_mode_device)
         {
-            CHECK_HIP_ERROR(rocblas_result_2.transfer_from(d_rocblas_result_2));
-
-            if(!rocblas_isnan(arg.alpha))
-            {
-                if(arg.unit_check)
-                {
-                    near_check_general<real_t<T>, real_t<T>>(
-                        batch_count, 1, 1, cpu_result, rocblas_result_2, abs_error);
-                }
-            }
-
-            if(arg.norm_check)
-            {
-                for(int b = 0; b < batch_count; ++b)
-                {
-                    rocblas_error_2
-                        += rocblas_abs((cpu_result[b] - rocblas_result_2[b]) / cpu_result[b]);
-                }
-            }
+            CHECK_HIP_ERROR(rocblas_result.transfer_from(d_rocblas_result));
+            error_device_ptr = compare_to_gold();
         }
     }
 
@@ -244,7 +267,7 @@ void testing_nrm2_batched(const Arguments& arg)
                 gpu_time_used = get_time_us_sync(stream); // in microseconds
 
             DAPI_DISPATCH(rocblas_nrm2_batched_fn,
-                          (handle, N, dx.ptr_on_device(), incx, batch_count, d_rocblas_result_2));
+                          (handle, N, dx.ptr_on_device(), incx, batch_count, d_rocblas_result));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
@@ -255,7 +278,7 @@ void testing_nrm2_batched(const Arguments& arg)
                                                                 nrm2_gflop_count<T>(N),
                                                                 nrm2_gbyte_count<T>(N),
                                                                 cpu_time_used,
-                                                                rocblas_error_1,
-                                                                rocblas_error_2);
+                                                                error_host_ptr,
+                                                                error_device_ptr);
     }
 }
