@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,10 +28,10 @@ template <typename T, typename U = T, typename V = T>
 void testing_rot_bad_arg(const Arguments& arg)
 {
     auto rocblas_rot_fn
-        = arg.api == FORTRAN ? rocblas_rot<T, U, V, true> : rocblas_rot<T, U, V, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rot<T, U, V, true> : rocblas_rot<T, U, V, false>;
 
     auto rocblas_rot_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rot_64<T, U, V, true> : rocblas_rot_64<T, U, V, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rot_64<T, U, V, true> : rocblas_rot_64<T, U, V, false>;
 
     int64_t              N    = 100;
     int64_t              incx = 1;
@@ -71,10 +71,10 @@ void testing_rot(const Arguments& arg)
 {
 
     auto rocblas_rot_fn
-        = arg.api == FORTRAN ? rocblas_rot<T, U, V, true> : rocblas_rot<T, U, V, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rot<T, U, V, true> : rocblas_rot<T, U, V, false>;
 
     auto rocblas_rot_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rot_64<T, U, V, true> : rocblas_rot_64<T, U, V, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rot_64<T, U, V, true> : rocblas_rot_64<T, U, V, false>;
 
     int64_t N    = arg.N;
     int64_t incx = arg.incx;
@@ -148,10 +148,64 @@ void testing_rot(const Arguments& arg)
             handle.pre_test(arg);
             DAPI_CHECK(rocblas_rot_fn, (handle, N, dx, incx, dy, incy, dc, ds));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<T> hx_copy(N, incx);
+                host_vector<T> hy_copy(N, incy);
+                CHECK_HIP_ERROR(hx.transfer_from(dx));
+                CHECK_HIP_ERROR(hy.transfer_from(dy));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory
+                    // Allocate device memory
+                    device_vector<T> dx_copy(N, incx);
+                    device_vector<T> dy_copy(N, incy);
+                    device_vector<U> dc_copy(1, 1);
+                    device_vector<V> ds_copy(1, 1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dc_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(ds_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dc_copy.transfer_from(hc));
+                    CHECK_HIP_ERROR(ds_copy.transfer_from(hs));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dx_copy.transfer_from(hx_gold));
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                        DAPI_CHECK(
+                            rocblas_rot_fn,
+                            (handle_copy, N, dx_copy, incx, dy_copy, incy, dc_copy, ds_copy));
+                        CHECK_HIP_ERROR(hx_copy.transfer_from(dx_copy));
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                        unit_check_general<T>(1, N, incx, hx, hx_copy);
+                        unit_check_general<T>(1, N, incy, hy, hy_copy);
+                    }
+                }
+                return;
+            }
         }
 
         cpu_time_used = get_time_us_no_sync();
-        cblas_rot<T, T, U, V>(N, hx_gold, incx, hy_gold, incy, hc, hs);
+        ref_rot<T, T, U, V>(N, hx_gold, incx, hy_gold, incy, hc, hs);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         if(arg.pointer_mode_host)
