@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,11 +27,12 @@
 template <typename T, typename U = T, typename V = T>
 void testing_rot_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_rot_batched_fn = arg.api == FORTRAN ? rocblas_rot_batched<T, U, V, true>
-                                                     : rocblas_rot_batched<T, U, V, false>;
+    auto rocblas_rot_batched_fn = arg.api & c_API_FORTRAN ? rocblas_rot_batched<T, U, V, true>
+                                                          : rocblas_rot_batched<T, U, V, false>;
 
-    auto rocblas_rot_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_rot_batched_64<T, U, V, true>
-                                                           : rocblas_rot_batched_64<T, U, V, false>;
+    auto rocblas_rot_batched_fn_64 = arg.api & c_API_FORTRAN
+                                         ? rocblas_rot_batched_64<T, U, V, true>
+                                         : rocblas_rot_batched_64<T, U, V, false>;
 
     int64_t N           = 100;
     int64_t incx        = 1;
@@ -76,11 +77,12 @@ void testing_rot_batched_bad_arg(const Arguments& arg)
 template <typename T, typename U = T, typename V = T>
 void testing_rot_batched(const Arguments& arg)
 {
-    auto rocblas_rot_batched_fn = arg.api == FORTRAN ? rocblas_rot_batched<T, U, V, true>
-                                                     : rocblas_rot_batched<T, U, V, false>;
+    auto rocblas_rot_batched_fn = arg.api & c_API_FORTRAN ? rocblas_rot_batched<T, U, V, true>
+                                                          : rocblas_rot_batched<T, U, V, false>;
 
-    auto rocblas_rot_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_rot_batched_64<T, U, V, true>
-                                                           : rocblas_rot_batched_64<T, U, V, false>;
+    auto rocblas_rot_batched_fn_64 = arg.api & c_API_FORTRAN
+                                         ? rocblas_rot_batched_64<T, U, V, true>
+                                         : rocblas_rot_batched_64<T, U, V, false>;
 
     int64_t N           = arg.N;
     int64_t incx        = arg.incx;
@@ -180,12 +182,75 @@ void testing_rot_batched(const Arguments& arg)
                         ds,
                         batch_count));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_batch_vector<T> hx_copy(N, incx, batch_count);
+                host_batch_vector<T> hy_copy(N, incy, batch_count);
+
+                CHECK_HIP_ERROR(hx.transfer_from(dx));
+                CHECK_HIP_ERROR(hy.transfer_from(dy));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory
+                    device_batch_vector<T> dx_copy(N, incx, batch_count);
+                    device_batch_vector<T> dy_copy(N, incy, batch_count);
+                    device_vector<U>       dc_copy(1, 1);
+                    device_vector<V>       ds_copy(1, 1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dc_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(ds_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dc_copy.transfer_from(hc));
+                    CHECK_HIP_ERROR(ds_copy.transfer_from(hs));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dx_copy.transfer_from(hx_gold));
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+
+                        DAPI_CHECK(rocblas_rot_batched_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy.ptr_on_device(),
+                                    incx,
+                                    dy_copy.ptr_on_device(),
+                                    incy,
+                                    dc_copy,
+                                    ds_copy,
+                                    batch_count));
+                        CHECK_HIP_ERROR(hx_copy.transfer_from(dx_copy));
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+
+                        unit_check_general<T>(1, N, incx, hx, hx_copy, batch_count);
+                        unit_check_general<T>(1, N, incy, hy, hy_copy, batch_count);
+                    }
+                }
+                return;
+            }
         }
 
         cpu_time_used = get_time_us_no_sync();
         for(size_t b = 0; b < batch_count; b++)
         {
-            cblas_rot<T, T, U, V>(N, hx_gold[b], incx, hy_gold[b], incy, hc, hs);
+            ref_rot<T, T, U, V>(N, hx_gold[b], incx, hy_gold[b], incy, hc, hs);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
