@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,9 +27,9 @@
 template <typename T>
 void testing_rotm_bad_arg(const Arguments& arg)
 {
-    auto rocblas_rotm_fn = arg.api == FORTRAN ? rocblas_rotm<T, true> : rocblas_rotm<T, false>;
+    auto rocblas_rotm_fn = arg.api & c_API_FORTRAN ? rocblas_rotm<T, true> : rocblas_rotm<T, false>;
     auto rocblas_rotm_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rotm_64<T, true> : rocblas_rotm_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rotm_64<T, true> : rocblas_rotm_64<T, false>;
 
     int64_t             N         = 100;
     int64_t             incx      = 1;
@@ -65,9 +65,9 @@ void testing_rotm_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_rotm(const Arguments& arg)
 {
-    auto rocblas_rotm_fn = arg.api == FORTRAN ? rocblas_rotm<T, true> : rocblas_rotm<T, false>;
+    auto rocblas_rotm_fn = arg.api & c_API_FORTRAN ? rocblas_rotm<T, true> : rocblas_rotm<T, false>;
     auto rocblas_rotm_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rotm_64<T, true> : rocblas_rotm_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rotm_64<T, true> : rocblas_rotm_64<T, false>;
 
     int64_t N    = arg.N;
     int64_t incx = arg.incx;
@@ -118,7 +118,7 @@ void testing_rotm(const Arguments& arg)
     // that it zeros out the second element of the rotm vector parameter
     memset(hparam.data(), 0, 5 * sizeof(T));
 
-    cblas_rotmg<T>(&hdata[0], &hdata[1], &hdata[2], &hdata[3], hparam);
+    ref_rotmg<T>(&hdata[0], &hdata[1], &hdata[2], &hdata[3], hparam);
 
     // CPU BLAS reference data
     host_vector<T> hx_gold(N, incx);
@@ -164,10 +164,61 @@ void testing_rotm(const Arguments& arg)
                 handle.pre_test(arg);
                 DAPI_CHECK(rocblas_rotm_fn, (handle, N, dx, incx, dy, incy, dparam));
                 handle.post_test(arg);
+
+                if(arg.repeatability_check)
+                {
+                    host_vector<T> hx_copy(N, incx);
+                    host_vector<T> hy_copy(N, incy);
+                    CHECK_HIP_ERROR(hx.transfer_from(dx));
+                    CHECK_HIP_ERROR(hy.transfer_from(dy));
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        // Allocate device memory
+                        device_vector<T> dx_copy(N, incx);
+                        device_vector<T> dy_copy(N, incy);
+                        device_vector<T> dparam_copy(5, 1);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dparam_copy.memcheck());
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            CHECK_HIP_ERROR(dx_copy.transfer_from(hx_gold));
+                            CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                            CHECK_HIP_ERROR(dparam_copy.transfer_from(hparam));
+
+                            DAPI_CHECK(rocblas_rotm_fn,
+                                       (handle_copy, N, dx_copy, incx, dy_copy, incy, dparam_copy));
+
+                            CHECK_HIP_ERROR(hx_copy.transfer_from(dx_copy));
+                            CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+
+                            unit_check_general<T>(1, N, incx, hx, hx_copy);
+                            unit_check_general<T>(1, N, incy, hy, hy_copy);
+                        }
+                    }
+                    return;
+                }
             }
 
             cpu_time_used = get_time_us_no_sync();
-            cblas_rotm<T>(N, hx_gold, incx, hy_gold, incy, hparam);
+            ref_rotm<T>(N, hx_gold, incx, hy_gold, incy, hparam);
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
             if(arg.pointer_mode_host)
