@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,10 +27,10 @@
 template <typename T>
 void testing_rotm_strided_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_rotm_strided_batched_fn    = arg.api == FORTRAN
+    auto rocblas_rotm_strided_batched_fn    = arg.api & c_API_FORTRAN
                                                   ? rocblas_rotm_strided_batched<T, true>
                                                   : rocblas_rotm_strided_batched<T, false>;
-    auto rocblas_rotm_strided_batched_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_rotm_strided_batched_fn_64 = arg.api & c_API_FORTRAN
                                                   ? rocblas_rotm_strided_batched_64<T, true>
                                                   : rocblas_rotm_strided_batched_64<T, false>;
 
@@ -97,10 +97,10 @@ template <typename T>
 void testing_rotm_strided_batched(const Arguments& arg)
 {
 
-    auto rocblas_rotm_strided_batched_fn    = arg.api == FORTRAN
+    auto rocblas_rotm_strided_batched_fn    = arg.api & c_API_FORTRAN
                                                   ? rocblas_rotm_strided_batched<T, true>
                                                   : rocblas_rotm_strided_batched<T, false>;
-    auto rocblas_rotm_strided_batched_fn_64 = arg.api == FORTRAN_64
+    auto rocblas_rotm_strided_batched_fn_64 = arg.api & c_API_FORTRAN
                                                   ? rocblas_rotm_strided_batched_64<T, true>
                                                   : rocblas_rotm_strided_batched_64<T, false>;
 
@@ -172,7 +172,7 @@ void testing_rotm_strided_batched(const Arguments& arg)
         // that it zeros out the second element of the rotm vector parameter
         memset(hparam_ptr, 0, 5 * sizeof(T));
 
-        cblas_rotmg<T>(
+        ref_rotmg<T>(
             hdata + b * 4, hdata + b * 4 + 1, hdata + b * 4 + 2, hdata + b * 4 + 3, hparam_ptr);
     }
 
@@ -195,13 +195,6 @@ void testing_rotm_strided_batched(const Arguments& arg)
             hx_gold.copy_from(hx);
             hy_gold.copy_from(hy);
 
-            cpu_time_used = get_time_us_no_sync();
-            for(size_t b = 0; b < batch_count; b++)
-            {
-                cblas_rotm<T>(N, hx_gold[b], incx, hy_gold[b], incy, hparam[b]);
-            }
-            cpu_time_used = get_time_us_no_sync() - cpu_time_used;
-
             // Test rocblas_pointer_mode_host
             //if(arg.pointer_mode_device)
             //{
@@ -258,6 +251,69 @@ void testing_rotm_strided_batched(const Arguments& arg)
                 CHECK_HIP_ERROR(hx.transfer_from(dx));
                 CHECK_HIP_ERROR(hy.transfer_from(dy));
 
+                if(arg.repeatability_check)
+                {
+                    host_strided_batch_vector<T> hx_copy(N, incx, stride_x, batch_count);
+                    host_strided_batch_vector<T> hy_copy(N, incy, stride_y, batch_count);
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        // Allocate device memory
+                        device_strided_batch_vector<T> dx_copy(N, incx, stride_x, batch_count);
+                        device_strided_batch_vector<T> dy_copy(N, incy, stride_y, batch_count);
+                        device_strided_batch_vector<T> dparam_copy(5, 1, stride_param, batch_count);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dparam_copy.memcheck());
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            CHECK_HIP_ERROR(dx_copy.transfer_from(hx_gold));
+                            CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                            CHECK_HIP_ERROR(dparam_copy.transfer_from(hparam));
+                            DAPI_CHECK(rocblas_rotm_strided_batched_fn,
+                                       (handle_copy,
+                                        N,
+                                        dx_copy,
+                                        incx,
+                                        stride_x,
+                                        dy_copy,
+                                        incy,
+                                        stride_y,
+                                        dparam_copy,
+                                        stride_param,
+                                        batch_count));
+                            CHECK_HIP_ERROR(hx_copy.transfer_from(dx_copy));
+                            CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                            unit_check_general<T>(1, N, incx, stride_x, hx, hx_copy, batch_count);
+                            unit_check_general<T>(1, N, incy, stride_y, hy, hy_copy, batch_count);
+                        }
+                    }
+                    return;
+                }
+
+                cpu_time_used = get_time_us_no_sync();
+                for(size_t b = 0; b < batch_count; b++)
+                {
+                    ref_rotm<T>(N, hx_gold[b], incx, hy_gold[b], incy, hparam[b]);
+                }
+                cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+
                 if(arg.unit_check)
                 {
                     near_check_general<T>(
