@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,13 @@
 
 #include "testing_common.hpp"
 
-template <typename T, typename U = T>
+template <typename T>
 void testing_rotmg_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_rotmg_batched_fn
-        = arg.api == FORTRAN ? rocblas_rotmg_batched<T, true> : rocblas_rotmg_batched<T, false>;
-    auto rocblas_rotmg_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_rotmg_batched_64<T, true>
-                                                             : rocblas_rotmg_batched_64<T, false>;
+    auto rocblas_rotmg_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_rotmg_batched<T, true>
+                                                               : rocblas_rotmg_batched<T, false>;
+    auto rocblas_rotmg_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_rotmg_batched_64<T, true>
+                                                               : rocblas_rotmg_batched_64<T, false>;
 
     int64_t batch_count = 5;
 
@@ -110,10 +110,10 @@ template <typename T>
 void testing_rotmg_batched(const Arguments& arg)
 {
 
-    auto rocblas_rotmg_batched_fn
-        = arg.api == FORTRAN ? rocblas_rotmg_batched<T, true> : rocblas_rotmg_batched<T, false>;
-    auto rocblas_rotmg_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_rotmg_batched_64<T, true>
-                                                             : rocblas_rotmg_batched_64<T, false>;
+    auto rocblas_rotmg_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_rotmg_batched<T, true>
+                                                               : rocblas_rotmg_batched<T, false>;
+    auto rocblas_rotmg_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_rotmg_batched_64<T, true>
+                                                               : rocblas_rotmg_batched_64<T, false>;
 
     int64_t batch_count = arg.batch_count;
 
@@ -189,7 +189,7 @@ void testing_rotmg_batched(const Arguments& arg)
             cpu_time_used = get_time_us_no_sync();
             for(size_t b = 0; b < batch_count; b++)
             {
-                cblas_rotmg<T>(hd1_gold[b], hd2_gold[b], hx_gold[b], hy_gold[b], hparams_gold[b]);
+                ref_rotmg<T>(hd1_gold[b], hd2_gold[b], hx_gold[b], hy_gold[b], hparams_gold[b]);
             }
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -257,6 +257,75 @@ void testing_rotmg_batched(const Arguments& arg)
                 CHECK_HIP_ERROR(ry.transfer_from(dy));
                 CHECK_HIP_ERROR(rparams.transfer_from(dparams));
 
+                if(arg.repeatability_check)
+                {
+                    host_batch_vector<T> rd1_copy(1, 1, batch_count);
+                    host_batch_vector<T> rd2_copy(1, 1, batch_count);
+                    host_batch_vector<T> rx_copy(1, 1, batch_count);
+                    host_batch_vector<T> ry_copy(1, 1, batch_count);
+                    host_batch_vector<T> rparams_copy(5, 1, batch_count);
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        // Allocate device memory in new device
+                        device_batch_vector<T> dd1_copy(1, 1, batch_count);
+                        device_batch_vector<T> dd2_copy(1, 1, batch_count);
+                        device_batch_vector<T> dx_copy(1, 1, batch_count);
+                        device_batch_vector<T> dy_copy(1, 1, batch_count);
+                        device_batch_vector<T> dparams_copy(5, 1, batch_count);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dd1_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dd2_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dparams_copy.memcheck());
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            CHECK_HIP_ERROR(dd1_copy.transfer_from(hd1));
+                            CHECK_HIP_ERROR(dd2_copy.transfer_from(hd2));
+                            CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                            CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+                            CHECK_HIP_ERROR(dparams_copy.transfer_from(hparams));
+
+                            DAPI_CHECK(rocblas_rotmg_batched_fn,
+                                       (handle_copy,
+                                        dd1_copy.ptr_on_device(),
+                                        dd2_copy.ptr_on_device(),
+                                        dx_copy.ptr_on_device(),
+                                        dy_copy.ptr_on_device(),
+                                        dparams_copy.ptr_on_device(),
+                                        batch_count));
+
+                            CHECK_HIP_ERROR(rd1_copy.transfer_from(dd1_copy));
+                            CHECK_HIP_ERROR(rd2_copy.transfer_from(dd2_copy));
+                            CHECK_HIP_ERROR(rx_copy.transfer_from(dx_copy));
+                            CHECK_HIP_ERROR(ry_copy.transfer_from(dy_copy));
+                            CHECK_HIP_ERROR(rparams_copy.transfer_from(dparams_copy));
+                            unit_check_general<T>(1, 1, 1, rd1, rd1_copy, batch_count);
+                            unit_check_general<T>(1, 1, 1, rd2, rd2_copy, batch_count);
+                            unit_check_general<T>(1, 1, 1, rx, rx_copy, batch_count);
+                            unit_check_general<T>(1, 1, 1, ry, ry_copy, batch_count);
+                            unit_check_general<T>(1, 5, 1, rparams, rparams_copy, batch_count);
+                        }
+                    }
+                    return;
+                }
+
                 if(arg.unit_check)
                 {
                     near_check_general<T>(1, 1, 1, rd1, hd1_gold, batch_count, rel_error);
