@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,9 +27,9 @@
 template <typename T>
 void testing_swap_bad_arg(const Arguments& arg)
 {
-    auto rocblas_swap_fn = arg.api == FORTRAN ? rocblas_swap<T, true> : rocblas_swap<T, false>;
+    auto rocblas_swap_fn = arg.api & c_API_FORTRAN ? rocblas_swap<T, true> : rocblas_swap<T, false>;
     auto rocblas_swap_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_swap_64<T, true> : rocblas_swap_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_swap_64<T, true> : rocblas_swap_64<T, false>;
 
     int64_t N    = 100;
     int64_t incx = 1;
@@ -55,9 +55,9 @@ void testing_swap_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_swap(const Arguments& arg)
 {
-    auto rocblas_swap_fn = arg.api == FORTRAN ? rocblas_swap<T, true> : rocblas_swap<T, false>;
+    auto rocblas_swap_fn = arg.api & c_API_FORTRAN ? rocblas_swap<T, true> : rocblas_swap<T, false>;
     auto rocblas_swap_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_swap_64<T, true> : rocblas_swap_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_swap_64<T, true> : rocblas_swap_64<T, false>;
 
     int64_t              N    = arg.N;
     int64_t              incx = arg.incx;
@@ -113,7 +113,7 @@ void testing_swap(const Arguments& arg)
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        cblas_swap<T>(N, hx_gold, incx, hy_gold, incy);
+        ref_swap<T>(N, hx_gold, incx, hy_gold, incy);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         if(arg.unit_check)
