@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,10 +28,10 @@ template <typename T>
 void testing_swap_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_swap_batched_fn
-        = arg.api == FORTRAN ? rocblas_swap_batched<T, true> : rocblas_swap_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_swap_batched<T, true> : rocblas_swap_batched<T, false>;
 
-    auto rocblas_swap_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_swap_batched_64<T, true>
-                                                            : rocblas_swap_batched_64<T, false>;
+    auto rocblas_swap_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_swap_batched_64<T, true>
+                                                              : rocblas_swap_batched_64<T, false>;
 
     int64_t N           = 100;
     int64_t incx        = 1;
@@ -63,10 +63,10 @@ template <typename T>
 void testing_swap_batched(const Arguments& arg)
 {
     auto rocblas_swap_batched_fn
-        = arg.api == FORTRAN ? rocblas_swap_batched<T, true> : rocblas_swap_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_swap_batched<T, true> : rocblas_swap_batched<T, false>;
 
-    auto rocblas_swap_batched_fn_64 = arg.api == FORTRAN_64 ? rocblas_swap_batched_64<T, true>
-                                                            : rocblas_swap_batched_64<T, false>;
+    auto rocblas_swap_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_swap_batched_64<T, true>
+                                                              : rocblas_swap_batched_64<T, false>;
 
     int64_t N           = arg.N;
     int64_t incx        = arg.incx;
@@ -101,7 +101,7 @@ void testing_swap_batched(const Arguments& arg)
     rocblas_init_vector(hx, arg, rocblas_client_alpha_sets_nan, true);
     rocblas_init_vector(hy, arg, rocblas_client_alpha_sets_nan, false);
 
-    hx_gold.copy_from(hx); // swapped later by cblas_swap
+    hx_gold.copy_from(hx); // swapped later by ref_swap
     hy_gold.copy_from(hy);
 
     // Transfer data from CPU to device
@@ -127,7 +127,7 @@ void testing_swap_batched(const Arguments& arg)
         cpu_time_used = get_time_us_no_sync();
         for(size_t b = 0; b < batch_count; b++)
         {
-            cblas_swap<T>(N, hx_gold[b], incx, hy_gold[b], incy);
+            ref_swap<T>(N, hx_gold[b], incx, hy_gold[b], incy);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
