@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,25 +22,14 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_symv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_symv_fn = arg.api == FORTRAN ? rocblas_symv<T, true> : rocblas_symv<T, false>;
+    auto rocblas_symv_fn = arg.api & c_API_FORTRAN ? rocblas_symv<T, true> : rocblas_symv<T, false>;
+    auto rocblas_symv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_symv_64<T, true> : rocblas_symv_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -48,10 +37,10 @@ void testing_symv_bad_arg(const Arguments& arg)
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, pointer_mode));
 
         const rocblas_fill uplo = rocblas_fill_upper;
-        const rocblas_int  N    = 100;
-        const rocblas_int  incx = 1;
-        const rocblas_int  incy = 1;
-        const rocblas_int  lda  = 100;
+        const int64_t      N    = 100;
+        const int64_t      incx = 1;
+        const int64_t      incy = 1;
+        const int64_t      lda  = 100;
 
         device_vector<T> alpha_d(1), beta_d(1), one_d(1), zero_d(1);
 
@@ -84,64 +73,74 @@ void testing_symv_bad_arg(const Arguments& arg)
         CHECK_DEVICE_ALLOCATION(dx.memcheck());
         CHECK_DEVICE_ALLOCATION(dy.memcheck());
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(nullptr, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy),
-            rocblas_status_invalid_handle);
+        DAPI_EXPECT(rocblas_status_invalid_handle,
+                    rocblas_symv_fn,
+                    (nullptr, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(handle, rocblas_fill_full, N, alpha, dA, lda, dx, incx, beta, dy, incy),
-            rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_symv_fn,
+                    (handle, rocblas_fill_full, N, alpha, dA, lda, dx, incx, beta, dy, incy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(handle, uplo, N, nullptr, dA, lda, dx, incx, beta, dy, incy),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_symv_fn,
+                    (handle, uplo, N, nullptr, dA, lda, dx, incx, beta, dy, incy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, dx, incx, nullptr, dy, incy),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_symv_fn,
+                    (handle, uplo, N, alpha, dA, lda, dx, incx, nullptr, dy, incy));
 
         if(pointer_mode == rocblas_pointer_mode_host)
         {
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_symv_fn(handle, uplo, N, alpha, nullptr, lda, dx, incx, beta, dy, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_symv_fn,
+                        (handle, uplo, N, alpha, nullptr, lda, dx, incx, beta, dy, incy));
 
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, nullptr, incx, beta, dy, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_symv_fn,
+                        (handle, uplo, N, alpha, dA, lda, nullptr, incx, beta, dy, incy));
 
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, dx, incx, beta, nullptr, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_symv_fn,
+                        (handle, uplo, N, alpha, dA, lda, dx, incx, beta, nullptr, incy));
         }
 
         // If N==0, all pointers can be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(
-                handle, uplo, 0, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy),
-            rocblas_status_success);
+        DAPI_EXPECT(
+            rocblas_status_success,
+            rocblas_symv_fn,
+            (handle, uplo, 0, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy));
 
         // If alpha==0, then A and x may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(handle, uplo, N, zero, nullptr, lda, nullptr, incx, beta, dy, incy),
-            rocblas_status_success);
+        DAPI_EXPECT(rocblas_status_success,
+                    rocblas_symv_fn,
+                    (handle, uplo, N, zero, nullptr, lda, nullptr, incx, beta, dy, incy));
 
         // If alpha==0 && beta==1, then A, x and y may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(handle, uplo, N, zero, nullptr, lda, nullptr, incx, one, nullptr, incy),
-            rocblas_status_success);
+        DAPI_EXPECT(rocblas_status_success,
+                    rocblas_symv_fn,
+                    (handle, uplo, N, zero, nullptr, lda, nullptr, incx, one, nullptr, incy));
+
+        if(arg.api & c_API_64)
+        {
+            int64_t n_large = 2147483649;
+            DAPI_EXPECT(rocblas_status_invalid_size,
+                        rocblas_symv_fn,
+                        (handle, uplo, n_large, alpha, dA, lda, dx, incx, beta, dy, incy));
+        }
     }
 }
 
 template <typename T>
 void testing_symv(const Arguments& arg)
 {
-    auto rocblas_symv_fn = arg.api == FORTRAN ? rocblas_symv<T, true> : rocblas_symv<T, false>;
+    auto rocblas_symv_fn = arg.api & c_API_FORTRAN ? rocblas_symv<T, true> : rocblas_symv<T, false>;
+    auto rocblas_symv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_symv_64<T, true> : rocblas_symv_64<T, false>;
 
-    rocblas_int N    = arg.N;
-    rocblas_int lda  = arg.lda;
-    rocblas_int incx = arg.incx;
-    rocblas_int incy = arg.incy;
+    int64_t N    = arg.N;
+    int64_t lda  = arg.lda;
+    int64_t incx = arg.incx;
+    int64_t incy = arg.incy;
 
     host_vector<T> alpha(1);
     host_vector<T> beta(1);
@@ -155,10 +154,10 @@ void testing_symv(const Arguments& arg)
     // argument sanity check before allocating invalid memory
     if(N < 0 || lda < 1 || lda < N || !incx || !incy)
     {
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_symv_fn(
-                handle, uplo, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(
+            rocblas_status_invalid_size,
+            rocblas_symv_fn,
+            (handle, uplo, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy));
         return;
     }
 
@@ -197,7 +196,7 @@ void testing_symv(const Arguments& arg)
     CHECK_HIP_ERROR(dy.transfer_from(hy));
     CHECK_HIP_ERROR(dA.transfer_from(hA));
 
-    double gpu_time_used, cpu_time_used;
+    double cpu_time_used;
     double h_error, d_error;
 
     if(arg.unit_check || arg.norm_check)
@@ -208,8 +207,8 @@ void testing_symv(const Arguments& arg)
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
             handle.pre_test(arg);
-            CHECK_ROCBLAS_ERROR(
-                rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
+            DAPI_CHECK(rocblas_symv_fn,
+                       (handle, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
             handle.post_test(arg);
 
             // copy output from device to CPU
@@ -226,15 +225,76 @@ void testing_symv(const Arguments& arg)
             dy.transfer_from(hy_gold);
 
             handle.pre_test(arg);
-            CHECK_ROCBLAS_ERROR(
-                rocblas_symv_fn(handle, uplo, N, d_alpha, dA, lda, dx, incx, d_beta, dy, incy));
+            DAPI_CHECK(rocblas_symv_fn,
+                       (handle, uplo, N, d_alpha, dA, lda, dx, incx, d_beta, dy, incy));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                // copy output from device to CPU
+                CHECK_HIP_ERROR(hy.transfer_from(dy));
+                host_vector<T> hy_copy(N, incy);
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory
+                    device_matrix<T> dA_copy(N, N, lda);
+                    device_vector<T> dx_copy(N, incx);
+                    device_vector<T> dy_copy(N, incy);
+                    device_vector<T> d_alpha_copy(1);
+                    device_vector<T> d_beta_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_alpha_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_beta_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                    CHECK_HIP_ERROR(d_alpha_copy.transfer_from(alpha));
+                    CHECK_HIP_ERROR(d_beta_copy.transfer_from(beta));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                        CHECK_ROCBLAS_ERROR(rocblas_symv_fn(handle_copy,
+                                                            uplo,
+                                                            N,
+                                                            d_alpha_copy,
+                                                            dA_copy,
+                                                            lda,
+                                                            dx_copy,
+                                                            incx,
+                                                            d_beta_copy,
+                                                            dy_copy,
+                                                            incy));
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                        unit_check_general<T>(1, N, incy, hy, hy_copy);
+                    }
+                }
+                return;
+            }
         }
 
         cpu_time_used = get_time_us_no_sync();
 
         // cpu reference
-        cblas_symv<T>(uplo, N, alpha[0], hA, lda, hx, incx, beta[0], hy_gold, incy);
+        ref_symv<T>(uplo, N, alpha[0], hA, lda, hx, incx, beta[0], hy_gold, incy);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -288,23 +348,20 @@ void testing_symv(const Arguments& arg)
     {
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
-
-        for(int iter = 0; iter < number_cold_calls; iter++)
-        {
-            CHECK_ROCBLAS_ERROR(
-                rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
-        }
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int iter = 0; iter < number_hot_calls; iter++)
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            CHECK_ROCBLAS_ERROR(
-                rocblas_symv_fn(handle, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
+
+            DAPI_DISPATCH(rocblas_symv_fn,
+                          (handle, uplo, N, alpha, dA, lda, dx, incx, beta, dy, incy));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
