@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,32 +23,20 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tbmv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_tbmv_fn = arg.api == FORTRAN ? rocblas_tbmv<T, true> : rocblas_tbmv<T, false>;
+    auto rocblas_tbmv_fn = arg.api & c_API_FORTRAN ? rocblas_tbmv<T, true> : rocblas_tbmv<T, false>;
+    auto rocblas_tbmv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tbmv_64<T, true> : rocblas_tbmv_64<T, false>;
 
-    const rocblas_int       N                 = 100;
-    const rocblas_int       K                 = 5;
-    const rocblas_int       lda               = 100;
-    const rocblas_int       incx              = 1;
-    const rocblas_int       banded_matrix_row = K + 1;
+    const int64_t           N                 = 100;
+    const int64_t           K                 = 5;
+    const int64_t           lda               = 100;
+    const int64_t           incx              = 1;
+    const int64_t           banded_matrix_row = K + 1;
     const rocblas_fill      uplo              = rocblas_fill_upper;
     const rocblas_operation transA            = rocblas_operation_none;
     const rocblas_diagonal  diag              = rocblas_diagonal_non_unit;
@@ -63,46 +51,50 @@ void testing_tbmv_bad_arg(const Arguments& arg)
     CHECK_DEVICE_ALLOCATION(dAb.memcheck());
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbmv_fn(handle, rocblas_fill_full, transA, diag, N, K, dAb, lda, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tbmv_fn,
+                (handle, rocblas_fill_full, transA, diag, N, K, dAb, lda, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbmv_fn(
-            handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, K, dAb, lda, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(
+        rocblas_status_invalid_value,
+        rocblas_tbmv_fn,
+        (handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, K, dAb, lda, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbmv_fn(
-            handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, K, dAb, lda, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(
+        rocblas_status_invalid_value,
+        rocblas_tbmv_fn,
+        (handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, K, dAb, lda, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, nullptr, lda, dx, incx),
-                          rocblas_status_invalid_pointer);
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tbmv_fn,
+                (handle, uplo, transA, diag, N, K, nullptr, lda, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, dAb, lda, nullptr, incx),
-        rocblas_status_invalid_pointer);
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tbmv_fn,
+                (handle, uplo, transA, diag, N, K, dAb, lda, nullptr, incx));
 
-    EXPECT_ROCBLAS_STATUS(rocblas_tbmv_fn(nullptr, uplo, transA, diag, N, K, dAb, lda, dx, incx),
-                          rocblas_status_invalid_handle);
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tbmv_fn,
+                (nullptr, uplo, transA, diag, N, K, dAb, lda, dx, incx));
 }
 
 template <typename T>
 void testing_tbmv(const Arguments& arg)
 {
-    auto rocblas_tbmv_fn = arg.api == FORTRAN ? rocblas_tbmv<T, true> : rocblas_tbmv<T, false>;
+    auto rocblas_tbmv_fn = arg.api & c_API_FORTRAN ? rocblas_tbmv<T, true> : rocblas_tbmv<T, false>;
+    auto rocblas_tbmv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tbmv_64<T, true> : rocblas_tbmv_64<T, false>;
 
-    rocblas_int       N                 = arg.N;
-    rocblas_int       K                 = arg.K;
-    rocblas_int       lda               = arg.lda;
-    rocblas_int       incx              = arg.incx;
+    int64_t           N                 = arg.N;
+    int64_t           K                 = arg.K;
+    int64_t           lda               = arg.lda;
+    int64_t           incx              = arg.incx;
     char              char_uplo         = arg.uplo;
     char              char_diag         = arg.diag;
     rocblas_fill      uplo              = char2rocblas_fill(char_uplo);
     rocblas_operation transA            = char2rocblas_operation(arg.transA);
     rocblas_diagonal  diag              = char2rocblas_diagonal(char_diag);
-    const rocblas_int banded_matrix_row = K + 1;
+    const int64_t     banded_matrix_row = K + 1;
 
     rocblas_local_handle handle{arg};
 
@@ -110,9 +102,9 @@ void testing_tbmv(const Arguments& arg)
     bool invalid_size = N < 0 || K < 0 || lda < banded_matrix_row || !incx;
     if(invalid_size)
     {
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, nullptr, lda, nullptr, incx),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_tbmv_fn,
+                    (handle, uplo, transA, diag, N, K, nullptr, lda, nullptr, incx));
 
         return;
     }
@@ -143,7 +135,7 @@ void testing_tbmv(const Arguments& arg)
     CHECK_HIP_ERROR(dAb.transfer_from(hAb));
     CHECK_HIP_ERROR(dx.transfer_from(hx));
 
-    double gpu_time_used, cpu_time_used;
+    double cpu_time_used;
     double rocblas_error = 0.0;
 
     /* =====================================================================
@@ -154,12 +146,56 @@ void testing_tbmv(const Arguments& arg)
     {
         // pointer mode shouldn't matter here
         handle.pre_test(arg);
-        CHECK_ROCBLAS_ERROR(rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx, incx));
+        DAPI_CHECK(rocblas_tbmv_fn, (handle, uplo, transA, diag, N, K, dAb, lda, dx, incx));
         handle.post_test(arg);
 
+        if(arg.repeatability_check)
+        {
+            host_vector<T> hx_copy(N, incx);
+            CHECK_HIP_ERROR(hx_copy.memcheck());
+            CHECK_HIP_ERROR(hx.transfer_from(dx));
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_matrix<T> dAb_copy(banded_matrix_row, N, lda);
+                device_vector<T> dx_copy(N, incx);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dAb_copy.memcheck());
+
+                // copy data from CPU to device
+                CHECK_HIP_ERROR(dAb_copy.transfer_from(hAb));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx_gold));
+                    DAPI_CHECK(
+                        rocblas_tbmv_fn,
+                        (handle_copy, uplo, transA, diag, N, K, dAb_copy, lda, dx_copy, incx));
+                    CHECK_HIP_ERROR(hx_copy.transfer_from(dx_copy));
+                    unit_check_general<T>(1, N, incx, hx, hx_copy);
+                }
+            }
+            return;
+        }
+
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        cblas_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hx_gold, incx);
+        ref_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hx_gold, incx);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         // copy output from device to CPU
@@ -178,21 +214,19 @@ void testing_tbmv(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
-
-        for(int iter = 0; iter < number_cold_calls; iter++)
-        {
-            rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx, incx);
-        }
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int iter = 0; iter < number_hot_calls; iter++)
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            rocblas_tbmv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx, incx);
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream); // in microseconds
+
+            DAPI_DISPATCH(rocblas_tbmv_fn, (handle, uplo, transA, diag, N, K, dAb, lda, dx, incx));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
