@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,33 +22,24 @@
 
 #pragma once
 
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tbsv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_tbsv_fn = arg.api == FORTRAN ? rocblas_tbsv<T, true> : rocblas_tbsv<T, false>;
+    auto rocblas_tbsv_fn = arg.api & c_API_FORTRAN ? rocblas_tbsv<T, true> : rocblas_tbsv<T, false>;
 
-    const rocblas_int       N                 = 100;
-    const rocblas_int       K                 = 5;
-    const rocblas_int       lda               = 100;
-    const rocblas_int       incx              = 1;
+    auto rocblas_tbsv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tbsv_64<T, true> : rocblas_tbsv_64<T, false>;
+
+    const int64_t           N                 = 100;
+    const int64_t           K                 = 5;
+    const int64_t           lda               = 100;
+    const int64_t           incx              = 1;
+    const int64_t           banded_matrix_row = K + 1;
     const rocblas_operation transA            = rocblas_operation_none;
     const rocblas_fill      uplo              = rocblas_fill_lower;
     const rocblas_diagonal  diag              = rocblas_diagonal_non_unit;
-    const rocblas_int       banded_matrix_row = K + 1;
 
     rocblas_local_handle handle{arg};
 
@@ -63,39 +54,49 @@ void testing_tbsv_bad_arg(const Arguments& arg)
     //
     // Checks.
     //
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbsv_fn(handle, rocblas_fill_full, transA, diag, N, K, dA, lda, dx, incx),
-        rocblas_status_invalid_value);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbsv_fn(
-            handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, K, dA, lda, dx, incx),
-        rocblas_status_invalid_value);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tbsv_fn(
-            handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, K, dA, lda, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tbsv_fn,
+                (nullptr, uplo, transA, diag, N, K, dA, lda, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, nullptr, lda, dx, incx),
-                          rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, dA, lda, nullptr, incx),
-                          rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(rocblas_tbsv_fn(nullptr, uplo, transA, diag, N, K, dA, lda, dx, incx),
-                          rocblas_status_invalid_handle);
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tbsv_fn,
+                (handle, rocblas_fill_full, transA, diag, N, K, dA, lda, dx, incx));
+
+    DAPI_EXPECT(
+        rocblas_status_invalid_value,
+        rocblas_tbsv_fn,
+        (handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, K, dA, lda, dx, incx));
+
+    DAPI_EXPECT(
+        rocblas_status_invalid_value,
+        rocblas_tbsv_fn,
+        (handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, K, dA, lda, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tbsv_fn,
+                (handle, uplo, transA, diag, N, K, nullptr, lda, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tbsv_fn,
+                (handle, uplo, transA, diag, N, K, dA, lda, nullptr, incx));
 }
 
 template <typename T>
 void testing_tbsv(const Arguments& arg)
 {
-    auto rocblas_tbsv_fn = arg.api == FORTRAN ? rocblas_tbsv<T, true> : rocblas_tbsv<T, false>;
+    auto rocblas_tbsv_fn = arg.api & c_API_FORTRAN ? rocblas_tbsv<T, true> : rocblas_tbsv<T, false>;
 
-    rocblas_int       N                 = arg.N;
-    rocblas_int       K                 = arg.K;
-    rocblas_int       lda               = arg.lda;
-    rocblas_int       incx              = arg.incx;
+    auto rocblas_tbsv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tbsv_64<T, true> : rocblas_tbsv_64<T, false>;
+
+    int64_t           N                 = arg.N;
+    int64_t           K                 = arg.K;
+    int64_t           lda               = arg.lda;
+    int64_t           incx              = arg.incx;
+    const int64_t     banded_matrix_row = K + 1;
     char              char_uplo         = arg.uplo;
     char              char_transA       = arg.transA;
     char              char_diag         = arg.diag;
-    const rocblas_int banded_matrix_row = K + 1;
     rocblas_fill      uplo              = char2rocblas_fill(char_uplo);
     rocblas_operation transA            = char2rocblas_operation(char_transA);
     rocblas_diagonal  diag              = char2rocblas_diagonal(char_diag);
@@ -107,10 +108,9 @@ void testing_tbsv(const Arguments& arg)
     bool invalid_size = N < 0 || K < 0 || lda < banded_matrix_row || !incx;
     if(invalid_size || !N)
     {
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, nullptr, lda, nullptr, incx),
-            invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_tbsv_fn,
+                    (handle, uplo, transA, diag, N, K, nullptr, lda, nullptr, incx));
         return;
     }
 
@@ -156,14 +156,13 @@ void testing_tbsv(const Arguments& arg)
     // initialize "exact" answer hx
     hb = hx;
 
-    cblas_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hb, incx);
+    ref_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hb, incx);
     cpu_x_or_b = hb;
     hx_or_b    = hb;
 
     double max_err     = 0.0;
     double max_err_res = 0.0;
 
-    double gpu_time_used, cpu_time_used;
     double error_eps_multiplier    = 40.0;
     double residual_eps_multiplier = 40.0;
     double eps                     = std::numeric_limits<real_t<T>>::epsilon();
@@ -174,12 +173,53 @@ void testing_tbsv(const Arguments& arg)
         CHECK_HIP_ERROR(dx_or_b.transfer_from(hx_or_b));
 
         handle.pre_test(arg);
-        CHECK_ROCBLAS_ERROR(
-            rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx_or_b, incx));
+        DAPI_CHECK(rocblas_tbsv_fn, (handle, uplo, transA, diag, N, K, dAb, lda, dx_or_b, incx));
         handle.post_test(arg);
 
         CHECK_HIP_ERROR(hx_or_b.transfer_from(dx_or_b));
 
+        if(arg.repeatability_check)
+        {
+            host_vector<T> hx_or_b_copy(N, incx);
+            CHECK_HIP_ERROR(hx_or_b_copy.memcheck());
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_matrix<T> dAb_copy(banded_matrix_row, N, lda);
+                device_vector<T> dx_or_b_copy(N, incx);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dAb_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dx_or_b_copy.memcheck());
+
+                CHECK_HIP_ERROR(dAb_copy.transfer_from(hAb));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_or_b_copy.transfer_from(hb));
+                    DAPI_CHECK(
+                        rocblas_tbsv_fn,
+                        (handle_copy, uplo, transA, diag, N, K, dAb_copy, lda, dx_or_b_copy, incx));
+                    CHECK_HIP_ERROR(hx_or_b_copy.transfer_from(dx_or_b_copy));
+                    unit_check_general<T>(1, N, incx, hx_or_b, hx_or_b_copy);
+                }
+            }
+            return;
+        }
+
         // computed result is in hx_or_b, so forward error is E = hx - hx_or_b
         // calculate norm 1 of vector E
         max_err = rocblas_abs(vector_norm_1<T>(N, incx, hx, hx_or_b));
@@ -188,7 +228,7 @@ void testing_tbsv(const Arguments& arg)
             trsm_err_res_check<T>(max_err, N, error_eps_multiplier, eps);
 
         // hx_or_b contains A * (calculated X), so res = A * (calculated x) - b = hx_or_b - hb
-        cblas_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hx_or_b, incx);
+        ref_tbmv<T>(uplo, transA, diag, N, K, hAb, lda, hx_or_b, incx);
 
         // Calculate norm 1 of vector res
         max_err_res = rocblas_abs(vector_norm_1<T>(N, incx, hx_or_b, hb));
@@ -199,33 +239,34 @@ void testing_tbsv(const Arguments& arg)
 
     if(arg.timing)
     {
-        // GPU rocBLAS
-        CHECK_HIP_ERROR(dx_or_b.transfer_from(hx_or_b));
-
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
-
-        for(int i = 0; i < number_cold_calls; i++)
-            rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx_or_b, incx);
+        double gpu_time_used, cpu_time_used = 0.0;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int i = 0; i < number_hot_calls; i++)
-            rocblas_tbsv_fn(handle, uplo, transA, diag, N, K, dAb, lda, dx_or_b, incx);
+        for(int iter = 0; iter < total_calls; iter++)
+        {
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
 
-        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+            DAPI_DISPATCH(rocblas_tbsv_fn,
+                          (handle, uplo, transA, diag, N, K, dAb, lda, dx_or_b, incx));
+        }
 
-        // CPU cblas
-        cpu_time_used = get_time_us_no_sync();
+        gpu_time_used = get_time_us_sync(stream) - gpu_time_used; // in microseconds
 
-        if(arg.norm_check)
-            cblas_tbsv<T>(uplo, transA, diag, N, K, hAb, lda, cpu_x_or_b, incx);
+        if(arg.unit_check || arg.norm_check)
+        {
+            // CPU cblas
+            cpu_time_used = get_time_us_no_sync();
 
-        cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+            if(arg.norm_check)
+                ref_tbsv<T>(uplo, transA, diag, N, K, hAb, lda, cpu_x_or_b, incx);
+
+            cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+        }
 
         ArgumentModel<e_uplo, e_transA, e_diag, e_N, e_K, e_lda, e_incx>{}.log_args<T>(
             rocblas_cout,
