@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,29 +23,17 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tpmv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_tpmv_fn = arg.api == FORTRAN ? rocblas_tpmv<T, true> : rocblas_tpmv<T, false>;
+    auto rocblas_tpmv_fn = arg.api & c_API_FORTRAN ? rocblas_tpmv<T, true> : rocblas_tpmv<T, false>;
+    auto rocblas_tpmv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tpmv_64<T, true> : rocblas_tpmv_64<T, false>;
 
-    const rocblas_int       N      = 100;
-    const rocblas_int       incx   = 1;
+    const int64_t           N      = 100;
+    const int64_t           incx   = 1;
     const rocblas_operation transA = rocblas_operation_none;
     const rocblas_fill      uplo   = rocblas_fill_lower;
     const rocblas_diagonal  diag   = rocblas_diagonal_non_unit;
@@ -63,36 +51,45 @@ void testing_tpmv_bad_arg(const Arguments& arg)
     //
     // Checks.
     //
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_fn(handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tpmv_fn,
+                (nullptr, uplo, transA, diag, N, dAp, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpmv_fn,
+                (handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx));
+
     // arg_checks code shared so transA, diag tested only in non-batched
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpmv_fn,
+                (handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, dAp, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_fn(handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpmv_fn,
+                (handle, uplo, transA, diag, N, nullptr, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_fn(
-            handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
-
-    EXPECT_ROCBLAS_STATUS(rocblas_tpmv_fn(handle, uplo, transA, diag, N, nullptr, dx, incx),
-                          rocblas_status_invalid_pointer);
-
-    EXPECT_ROCBLAS_STATUS(rocblas_tpmv_fn(handle, uplo, transA, diag, N, dAp, nullptr, incx),
-                          rocblas_status_invalid_pointer);
-
-    EXPECT_ROCBLAS_STATUS(rocblas_tpmv_fn(nullptr, uplo, transA, diag, N, dAp, dx, incx),
-                          rocblas_status_invalid_handle);
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpmv_fn,
+                (handle, uplo, transA, diag, N, dAp, nullptr, incx));
+    // If N is 64 bit
+    if(arg.api & c_API_64)
+    {
+        int64_t n_over_int32 = 2147483649;
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_tpmv_fn,
+                    (nullptr, uplo, transA, diag, n_over_int32, dAp, dx, incx));
+    }
 }
 
 template <typename T>
 void testing_tpmv(const Arguments& arg)
 {
-    auto rocblas_tpmv_fn = arg.api == FORTRAN ? rocblas_tpmv<T, true> : rocblas_tpmv<T, false>;
+    auto rocblas_tpmv_fn = arg.api & c_API_FORTRAN ? rocblas_tpmv<T, true> : rocblas_tpmv<T, false>;
 
-    rocblas_int N = arg.N, incx = arg.incx;
+    auto rocblas_tpmv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tpmv_64<T, true> : rocblas_tpmv_64<T, false>;
+
+    int64_t N = arg.N, incx = arg.incx;
 
     char char_uplo = arg.uplo, char_transA = arg.transA, char_diag = arg.diag;
 
@@ -104,9 +101,9 @@ void testing_tpmv(const Arguments& arg)
     bool invalid_size = N < 0 || !incx;
     if(invalid_size || !N)
     {
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_tpmv_fn(handle, uplo, transA, diag, N, nullptr, nullptr, incx),
-            invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_tpmv_fn,
+                    (handle, uplo, transA, diag, N, nullptr, nullptr, incx));
 
         return;
     }
@@ -138,7 +135,7 @@ void testing_tpmv(const Arguments& arg)
     CHECK_HIP_ERROR(dAp.transfer_from(hAp));
     CHECK_HIP_ERROR(dx.transfer_from(hx));
 
-    double gpu_time_used, cpu_time_used;
+    double cpu_time_used;
     double rocblas_error;
 
     /* =====================================================================
@@ -149,19 +146,60 @@ void testing_tpmv(const Arguments& arg)
 
         handle.pre_test(arg);
         // ROCBLAS
-        CHECK_ROCBLAS_ERROR(rocblas_tpmv_fn(handle, uplo, transA, diag, N, dAp, dx, incx));
+        DAPI_CHECK(rocblas_tpmv_fn, (handle, uplo, transA, diag, N, dAp, dx, incx));
         handle.post_test(arg);
 
+        // fetch GPU
+        CHECK_HIP_ERROR(hres.transfer_from(dx));
+        if(arg.repeatability_check)
+        {
+            host_vector<T> hres_copy(N, incx);
+            CHECK_HIP_ERROR(hres_copy.memcheck());
+
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_matrix<T> dAp_copy(1, rocblas_packed_matrix_size(N), 1);
+                device_vector<T> dx_copy(N, incx);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dAp_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+
+                CHECK_HIP_ERROR(dAp_copy.transfer_from(hAp));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    DAPI_CHECK(rocblas_tpmv_fn,
+                               (handle_copy, uplo, transA, diag, N, dAp_copy, dx_copy, incx));
+                    CHECK_HIP_ERROR(hres_copy.transfer_from(dx_copy));
+                    unit_check_general<T>(1, N, incx, hres, hres_copy);
+                }
+            }
+            return;
+        }
+
         // CPU BLAS
         {
             cpu_time_used = get_time_us_no_sync();
-            cblas_tpmv<T>(uplo, transA, diag, N, hAp, hx, incx);
+            ref_tpmv<T>(uplo, transA, diag, N, hAp, hx, incx);
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
         }
 
-        // fetch GPU
-        CHECK_HIP_ERROR(hres.transfer_from(dx));
-
         // Unit check.
         if(arg.unit_check)
         {
@@ -177,27 +215,22 @@ void testing_tpmv(const Arguments& arg)
 
     if(arg.timing)
     {
-        // Warmup
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
+
+        hipStream_t stream;
+        CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
+
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            int number_cold_calls = arg.cold_iters;
-            for(int iter = 0; iter < number_cold_calls; iter++)
-            {
-                rocblas_tpmv_fn(handle, uplo, transA, diag, N, dAp, dx, incx);
-            }
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
+
+            DAPI_DISPATCH(rocblas_tpmv_fn, (handle, uplo, transA, diag, N, dAp, dx, incx));
         }
 
-        // Go !
-        {
-            hipStream_t stream;
-            CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-            gpu_time_used        = get_time_us_sync(stream); // in microseconds
-            int number_hot_calls = arg.iters;
-            for(int iter = 0; iter < number_hot_calls; iter++)
-            {
-                rocblas_tpmv_fn(handle, uplo, transA, diag, N, dAp, dx, incx);
-            }
-            gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
-        }
+        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
 
         // Log performance.
         ArgumentModel<e_uplo, e_transA, e_diag, e_N, e_incx>{}.log_args<T>(rocblas_cout,
