@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,32 +22,22 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tpmv_strided_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_tpmv_strided_batched_fn = arg.api == FORTRAN
+    auto rocblas_tpmv_strided_batched_fn = arg.api & c_API_FORTRAN
                                                ? rocblas_tpmv_strided_batched<T, true>
                                                : rocblas_tpmv_strided_batched<T, false>;
 
-    const rocblas_int       N           = 100;
-    const rocblas_int       incx        = 1;
-    const rocblas_int       batch_count = 1;
+    auto rocblas_tpmv_strided_batched_fn_64 = arg.api & c_API_FORTRAN
+                                                  ? rocblas_tpmv_strided_batched_64<T, true>
+                                                  : rocblas_tpmv_strided_batched_64<T, false>;
+
+    const int64_t           N           = 100;
+    const int64_t           incx        = 1;
+    const int64_t           batch_count = 1;
     const rocblas_stride    stride_a    = (N * (N + 1)) / 2;
     const rocblas_stride    stride_x    = N;
     const rocblas_operation transA      = rocblas_operation_none;
@@ -65,44 +55,48 @@ void testing_tpmv_strided_batched_bad_arg(const Arguments& arg)
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
     // Checks.
-    EXPECT_ROCBLAS_STATUS(rocblas_tpmv_strided_batched_fn(handle,
-                                                          rocblas_fill_full,
-                                                          transA,
-                                                          diag,
-                                                          N,
-                                                          dAp,
-                                                          stride_a,
-                                                          dx,
-                                                          incx,
-                                                          stride_x,
-                                                          batch_count),
-                          rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tpmv_strided_batched_fn,
+                (nullptr, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count));
+
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpmv_strided_batched_fn,
+                (handle,
+                 rocblas_fill_full,
+                 transA,
+                 diag,
+                 N,
+                 dAp,
+                 stride_a,
+                 dx,
+                 incx,
+                 stride_x,
+                 batch_count));
+
     // arg_checks code shared so transA, diag tested only in non-batched
+    DAPI_EXPECT(
+        rocblas_status_invalid_pointer,
+        rocblas_tpmv_strided_batched_fn,
+        (handle, uplo, transA, diag, N, nullptr, stride_a, dx, incx, stride_x, batch_count));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_strided_batched_fn(
-            handle, uplo, transA, diag, N, nullptr, stride_a, dx, incx, stride_x, batch_count),
-        rocblas_status_invalid_pointer);
-
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_strided_batched_fn(
-            handle, uplo, transA, diag, N, dAp, stride_a, nullptr, incx, stride_x, batch_count),
-        rocblas_status_invalid_pointer);
-
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpmv_strided_batched_fn(
-            nullptr, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count),
-        rocblas_status_invalid_handle);
+    DAPI_EXPECT(
+        rocblas_status_invalid_pointer,
+        rocblas_tpmv_strided_batched_fn,
+        (handle, uplo, transA, diag, N, dAp, stride_a, nullptr, incx, stride_x, batch_count));
 }
 
 template <typename T>
 void testing_tpmv_strided_batched(const Arguments& arg)
 {
-    auto rocblas_tpmv_strided_batched_fn = arg.api == FORTRAN
+    auto rocblas_tpmv_strided_batched_fn = arg.api & c_API_FORTRAN
                                                ? rocblas_tpmv_strided_batched<T, true>
                                                : rocblas_tpmv_strided_batched<T, false>;
 
-    rocblas_int    N = arg.N, incx = arg.incx, batch_count = arg.batch_count;
+    auto rocblas_tpmv_strided_batched_fn_64 = arg.api & c_API_FORTRAN
+                                                  ? rocblas_tpmv_strided_batched_64<T, true>
+                                                  : rocblas_tpmv_strided_batched_64<T, false>;
+
+    int64_t        N = arg.N, incx = arg.incx, batch_count = arg.batch_count;
     rocblas_stride stride_a = arg.stride_a, stride_x = arg.stride_x;
 
     char char_uplo = arg.uplo, char_transA = arg.transA, char_diag = arg.diag;
@@ -117,18 +111,19 @@ void testing_tpmv_strided_batched(const Arguments& arg)
     bool invalid_size = N < 0 || !incx || batch_count < 0;
     if(invalid_size || !N || !batch_count)
     {
-        EXPECT_ROCBLAS_STATUS(rocblas_tpmv_strided_batched_fn(handle,
-                                                              uplo,
-                                                              transA,
-                                                              diag,
-                                                              N,
-                                                              nullptr,
-                                                              stride_a,
-                                                              nullptr,
-                                                              incx,
-                                                              stride_x,
-                                                              batch_count),
-                              invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_tpmv_strided_batched_fn,
+                    (handle,
+                     uplo,
+                     transA,
+                     diag,
+                     N,
+                     nullptr,
+                     stride_a,
+                     nullptr,
+                     incx,
+                     stride_x,
+                     batch_count));
 
         return;
     }
@@ -166,7 +161,7 @@ void testing_tpmv_strided_batched(const Arguments& arg)
     CHECK_HIP_ERROR(dAp.transfer_from(hAp));
     CHECK_HIP_ERROR(dx.transfer_from(hx));
 
-    double gpu_time_used, cpu_time_used, rocblas_error;
+    double cpu_time_used, rocblas_error;
 
     /* =====================================================================
      ROCBLAS
@@ -175,23 +170,76 @@ void testing_tpmv_strided_batched(const Arguments& arg)
     {
         handle.pre_test(arg);
         // GPU BLAS
-        CHECK_ROCBLAS_ERROR(rocblas_tpmv_strided_batched_fn(
-            handle, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count));
+        DAPI_CHECK(rocblas_tpmv_strided_batched_fn,
+                   (handle, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count));
         handle.post_test(arg);
 
+        // fetch GPU
+        CHECK_HIP_ERROR(hres.transfer_from(dx));
+
+        if(arg.repeatability_check)
+        {
+            host_strided_batch_vector<T> hres_copy(N, incx, stride_x, batch_count);
+            CHECK_HIP_ERROR(hres_copy.memcheck());
+
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_strided_batch_matrix<T> dAp_copy(
+                    1, rocblas_packed_matrix_size(N), 1, stride_a, batch_count);
+                device_strided_batch_vector<T> dx_copy(N, incx, stride_x, batch_count);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dAp_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+
+                CHECK_HIP_ERROR(dAp_copy.transfer_from(hAp));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    DAPI_CHECK(rocblas_tpmv_strided_batched_fn,
+                               (handle_copy,
+                                uplo,
+                                transA,
+                                diag,
+                                N,
+                                dAp_copy,
+                                stride_a,
+                                dx_copy,
+                                incx,
+                                stride_x,
+                                batch_count));
+                    CHECK_HIP_ERROR(hres_copy.transfer_from(dx_copy));
+                    unit_check_general<T>(1, N, incx, stride_x, hres, hres_copy, batch_count);
+                }
+            }
+            return;
+        }
+
         // CPU BLAS
         {
             cpu_time_used = get_time_us_no_sync();
-            for(rocblas_int b = 0; b < batch_count; ++b)
+            for(size_t b = 0; b < batch_count; ++b)
             {
-                cblas_tpmv<T>(uplo, transA, diag, N, hAp[b], hx[b], incx);
+                ref_tpmv<T>(uplo, transA, diag, N, hAp[b], hx[b], incx);
             }
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
         }
 
-        // fetch GPU
-        CHECK_HIP_ERROR(hres.transfer_from(dx));
-
         // Unit check.
         if(arg.unit_check)
         {
@@ -207,30 +255,23 @@ void testing_tpmv_strided_batched(const Arguments& arg)
 
     if(arg.timing)
     {
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
-        // Warmup
-        {
-            int number_cold_calls = arg.cold_iters;
-            for(int iter = 0; iter < number_cold_calls; iter++)
-            {
-                rocblas_tpmv_strided_batched_fn(
-                    handle, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count);
-            }
-        }
+        hipStream_t stream;
+        CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
 
-        // Go !
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            hipStream_t stream;
-            CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-            gpu_time_used        = get_time_us_sync(stream); // in microseconds
-            int number_hot_calls = arg.iters;
-            for(int iter = 0; iter < number_hot_calls; iter++)
-            {
-                rocblas_tpmv_strided_batched_fn(
-                    handle, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count);
-            }
-            gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
+
+            DAPI_DISPATCH(
+                rocblas_tpmv_strided_batched_fn,
+                (handle, uplo, transA, diag, N, dAp, stride_a, dx, incx, stride_x, batch_count));
         }
+        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
 
         // Log performance.
         ArgumentModel<e_uplo,
