@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,28 +22,18 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tpsv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_tpsv_fn = arg.api == FORTRAN ? rocblas_tpsv<T, true> : rocblas_tpsv<T, false>;
+    auto rocblas_tpsv_fn = arg.api & c_API_FORTRAN ? rocblas_tpsv<T, true> : rocblas_tpsv<T, false>;
 
-    const rocblas_int       N      = 100;
-    const rocblas_int       incx   = 1;
+    auto rocblas_tpsv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tpsv_64<T, true> : rocblas_tpsv_64<T, false>;
+
+    const int64_t           N      = 100;
+    const int64_t           incx   = 1;
     const rocblas_operation transA = rocblas_operation_none;
     const rocblas_fill      uplo   = rocblas_fill_lower;
     const rocblas_diagonal  diag   = rocblas_diagonal_non_unit;
@@ -59,35 +49,44 @@ void testing_tpsv_bad_arg(const Arguments& arg)
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
     // Checks
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_fn(handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_fn(handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_fn(
-            handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, dAp, dx, incx),
-        rocblas_status_invalid_value);
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tpsv_fn,
+                (nullptr, uplo, transA, diag, N, dAp, dx, incx));
 
-    EXPECT_ROCBLAS_STATUS(rocblas_tpsv_fn(handle, uplo, transA, diag, N, nullptr, dx, incx),
-                          rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(rocblas_tpsv_fn(handle, uplo, transA, diag, N, dAp, nullptr, incx),
-                          rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(rocblas_tpsv_fn(nullptr, uplo, transA, diag, N, dAp, dx, incx),
-                          rocblas_status_invalid_handle);
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpsv_fn,
+                (handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpsv_fn,
+                (handle, uplo, (rocblas_operation)rocblas_fill_full, diag, N, dAp, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpsv_fn,
+                (handle, uplo, transA, (rocblas_diagonal)rocblas_fill_full, N, dAp, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpsv_fn,
+                (handle, uplo, transA, diag, N, nullptr, dx, incx));
+
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpsv_fn,
+                (handle, uplo, transA, diag, N, dAp, nullptr, incx));
 }
 
 template <typename T>
 void testing_tpsv(const Arguments& arg)
 {
-    auto rocblas_tpsv_fn = arg.api == FORTRAN ? rocblas_tpsv<T, true> : rocblas_tpsv<T, false>;
+    auto rocblas_tpsv_fn = arg.api & c_API_FORTRAN ? rocblas_tpsv<T, true> : rocblas_tpsv<T, false>;
 
-    rocblas_int N           = arg.N;
-    rocblas_int incx        = arg.incx;
-    char        char_uplo   = arg.uplo;
-    char        char_transA = arg.transA;
-    char        char_diag   = arg.diag;
+    auto rocblas_tpsv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_tpsv_64<T, true> : rocblas_tpsv_64<T, false>;
+
+    int64_t N           = arg.N;
+    int64_t incx        = arg.incx;
+    char    char_uplo   = arg.uplo;
+    char    char_transA = arg.transA;
+    char    char_diag   = arg.diag;
 
     rocblas_fill      uplo   = char2rocblas_fill(char_uplo);
     rocblas_operation transA = char2rocblas_operation(char_transA);
@@ -100,10 +99,9 @@ void testing_tpsv(const Arguments& arg)
     bool invalid_size = N < 0 || !incx;
     if(invalid_size || !N)
     {
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_tpsv_fn(handle, uplo, transA, diag, N, nullptr, nullptr, incx),
-            invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_tpsv_fn,
+                    (handle, uplo, transA, diag, N, nullptr, nullptr, incx));
         return;
     }
 
@@ -140,7 +138,7 @@ void testing_tpsv(const Arguments& arg)
     hb = hx;
 
     // Calculate hb = hA*hx;
-    cblas_trmv<T>(uplo, transA, diag, N, hA, N, hb, incx);
+    ref_trmv<T>(uplo, transA, diag, N, hA, N, hb, incx);
     cpu_x_or_b = hb; // cpuXorB <- B
     hx_or_b    = hb;
 
@@ -151,9 +149,8 @@ void testing_tpsv(const Arguments& arg)
     CHECK_HIP_ERROR(dAp.transfer_from(hAp));
     CHECK_HIP_ERROR(dx_or_b.transfer_from(hx_or_b));
 
-    double max_err = 0.0;
-    double max_res = 0.0;
-    double gpu_time_used, cpu_time_used;
+    double max_err                 = 0.0;
+    double max_res                 = 0.0;
     double error_eps_multiplier    = 40.0;
     double residual_eps_multiplier = 20.0;
     double eps                     = std::numeric_limits<real_t<T>>::epsilon();
@@ -161,16 +158,57 @@ void testing_tpsv(const Arguments& arg)
     {
         // calculate dxorb <- A^(-1) b
         handle.pre_test(arg);
-        CHECK_ROCBLAS_ERROR(rocblas_tpsv_fn(handle, uplo, transA, diag, N, dAp, dx_or_b, incx));
+        DAPI_CHECK(rocblas_tpsv_fn, (handle, uplo, transA, diag, N, dAp, dx_or_b, incx));
         handle.post_test(arg);
         CHECK_HIP_ERROR(hx_or_b.transfer_from(dx_or_b));
 
+        if(arg.repeatability_check)
+        {
+            host_vector<T> hx_or_b_copy(N, incx);
+            CHECK_HIP_ERROR(hx_or_b_copy.memcheck());
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_matrix<T> dAp_copy(1, rocblas_packed_matrix_size(N), 1);
+                device_vector<T> dx_or_b_copy(N, incx);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dAp_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dx_or_b_copy.memcheck());
+
+                CHECK_HIP_ERROR(dAp_copy.transfer_from(hAp));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_or_b_copy.transfer_from(hb));
+                    DAPI_CHECK(rocblas_tpsv_fn,
+                               (handle_copy, uplo, transA, diag, N, dAp_copy, dx_or_b_copy, incx));
+                    CHECK_HIP_ERROR(hx_or_b_copy.transfer_from(dx_or_b_copy));
+                    unit_check_general<T>(1, N, incx, hx_or_b, hx_or_b_copy);
+                }
+            }
+            return;
+        }
+
         max_err = rocblas_abs(vector_norm_1<T>(N, incx, hx, hx_or_b));
 
         if(arg.unit_check)
             trsm_err_res_check(max_err, N, error_eps_multiplier, eps);
 
-        cblas_trmv<T>(uplo, transA, diag, N, hA, N, hx_or_b, incx);
+        ref_trmv<T>(uplo, transA, diag, N, hA, N, hx_or_b, incx);
         // hx_or_b contains A * (calculated X), so residual = A * (calculated x) - b
         //                                                  = hx_or_b - hb
         // res is the one norm of the scaled residual for each column
@@ -183,31 +221,32 @@ void testing_tpsv(const Arguments& arg)
 
     if(arg.timing)
     {
-        // GPU rocBLAS
-        CHECK_HIP_ERROR(dx_or_b.transfer_from(cpu_x_or_b));
-
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
-
-        for(int i = 0; i < number_cold_calls; i++)
-            rocblas_tpsv_fn(handle, uplo, transA, diag, N, dAp, dx_or_b, incx);
+        double gpu_time_used, cpu_time_used = 0.0;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int i = 0; i < number_hot_calls; i++)
-            rocblas_tpsv_fn(handle, uplo, transA, diag, N, dAp, dx_or_b, incx);
+        for(int iter = 0; iter < total_calls; iter++)
+        {
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
 
-        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+            DAPI_DISPATCH(rocblas_tpsv_fn, (handle, uplo, transA, diag, N, dAp, dx_or_b, incx));
+        }
 
-        // CPU cblas
-        cpu_time_used = get_time_us_no_sync();
+        gpu_time_used = get_time_us_sync(stream) - gpu_time_used; // in microseconds
 
-        if(arg.norm_check)
-            cblas_tpsv<T>(uplo, transA, diag, N, hAp, cpu_x_or_b, incx);
+        if(arg.unit_check || arg.norm_check)
+        {
+            // CPU cblas
+            cpu_time_used = get_time_us_no_sync();
 
-        cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+            ref_tpsv<T>(uplo, transA, diag, N, hAp, cpu_x_or_b, incx);
+
+            cpu_time_used = get_time_us_no_sync() - cpu_time_used; // in microseconds
+        }
 
         ArgumentModel<e_uplo, e_transA, e_diag, e_N, e_incx>{}.log_args<T>(
             rocblas_cout,
