@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,30 +22,20 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_tpsv_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_tpsv_batched_fn
-        = arg.api == FORTRAN ? rocblas_tpsv_batched<T, true> : rocblas_tpsv_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_tpsv_batched<T, true> : rocblas_tpsv_batched<T, false>;
 
-    const rocblas_int       N           = 100;
-    const rocblas_int       incx        = 1;
-    const rocblas_int       batch_count = 3;
+    auto rocblas_tpsv_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_tpsv_batched_64<T, true>
+                                                              : rocblas_tpsv_batched_64<T, false>;
+
+    const int64_t           N           = 100;
+    const int64_t           incx        = 1;
+    const int64_t           batch_count = 3;
     const rocblas_operation transA      = rocblas_operation_none;
     const rocblas_fill      uplo        = rocblas_fill_lower;
     const rocblas_diagonal  diag        = rocblas_diagonal_non_unit;
@@ -61,35 +51,39 @@ void testing_tpsv_batched_bad_arg(const Arguments& arg)
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
 
     // Checks.
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_batched_fn(
-            handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx, batch_count),
-        rocblas_status_invalid_value);
-    // arg_checks code shared so transA, diag tested only in non-batched
+    DAPI_EXPECT(rocblas_status_invalid_handle,
+                rocblas_tpsv_batched_fn,
+                (nullptr, uplo, transA, diag, N, dAp, dx, incx, batch_count));
 
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_batched_fn(handle, uplo, transA, diag, N, nullptr, dx, incx, batch_count),
-        rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_batched_fn(handle, uplo, transA, diag, N, dAp, nullptr, incx, batch_count),
-        rocblas_status_invalid_pointer);
-    EXPECT_ROCBLAS_STATUS(
-        rocblas_tpsv_batched_fn(nullptr, uplo, transA, diag, N, dAp, dx, incx, batch_count),
-        rocblas_status_invalid_handle);
+    DAPI_EXPECT(rocblas_status_invalid_value,
+                rocblas_tpsv_batched_fn,
+                (handle, rocblas_fill_full, transA, diag, N, dAp, dx, incx, batch_count));
+
+    // arg_checks code shared so transA, diag tested only in non-batched
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpsv_batched_fn,
+                (handle, uplo, transA, diag, N, nullptr, dx, incx, batch_count));
+
+    DAPI_EXPECT(rocblas_status_invalid_pointer,
+                rocblas_tpsv_batched_fn,
+                (handle, uplo, transA, diag, N, dAp, nullptr, incx, batch_count));
 }
 
 template <typename T>
 void testing_tpsv_batched(const Arguments& arg)
 {
     auto rocblas_tpsv_batched_fn
-        = arg.api == FORTRAN ? rocblas_tpsv_batched<T, true> : rocblas_tpsv_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_tpsv_batched<T, true> : rocblas_tpsv_batched<T, false>;
 
-    rocblas_int N           = arg.N;
-    rocblas_int incx        = arg.incx;
-    char        char_uplo   = arg.uplo;
-    char        char_transA = arg.transA;
-    char        char_diag   = arg.diag;
-    rocblas_int batch_count = arg.batch_count;
+    auto rocblas_tpsv_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_tpsv_batched_64<T, true>
+                                                              : rocblas_tpsv_batched_64<T, false>;
+
+    int64_t N           = arg.N;
+    int64_t incx        = arg.incx;
+    int64_t batch_count = arg.batch_count;
+    char    char_uplo   = arg.uplo;
+    char    char_transA = arg.transA;
+    char    char_diag   = arg.diag;
 
     rocblas_fill      uplo   = char2rocblas_fill(char_uplo);
     rocblas_operation transA = char2rocblas_operation(char_transA);
@@ -102,15 +96,12 @@ void testing_tpsv_batched(const Arguments& arg)
     bool invalid_size = N < 0 || !incx || batch_count < 0;
     if(invalid_size || !N || !batch_count)
     {
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_tpsv_batched_fn(
-                handle, uplo, transA, diag, N, nullptr, nullptr, incx, batch_count),
-            invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_tpsv_batched_fn,
+                    (handle, uplo, transA, diag, N, nullptr, nullptr, incx, batch_count));
         return;
     }
 
-    double gpu_time_used, cpu_time_used;
     double error_eps_multiplier    = 40.0;
     double residual_eps_multiplier = 20.0;
     double eps                     = std::numeric_limits<real_t<T>>::epsilon();
@@ -155,10 +146,10 @@ void testing_tpsv_batched(const Arguments& arg)
     }
 
     hb.copy_from(hx);
-    for(int b = 0; b < batch_count; b++)
+    for(size_t b = 0; b < batch_count; b++)
     {
         // Calculate hb = hA*hx;
-        cblas_trmv<T>(uplo, transA, diag, N, hA[b], N, hb[b], incx);
+        ref_trmv<T>(uplo, transA, diag, N, hA[b], N, hb[b], incx);
     }
 
     // helper function to convert Regular matrix `hA` to packed matrix `hAp`
@@ -177,27 +168,77 @@ void testing_tpsv_batched(const Arguments& arg)
     {
         // calculate dxorb <- A^(-1) b
         handle.pre_test(arg);
-        CHECK_ROCBLAS_ERROR(rocblas_tpsv_batched_fn(handle,
-                                                    uplo,
-                                                    transA,
-                                                    diag,
-                                                    N,
-                                                    dAp.ptr_on_device(),
-                                                    dx_or_b.ptr_on_device(),
-                                                    incx,
-                                                    batch_count));
+        DAPI_CHECK(rocblas_tpsv_batched_fn,
+                   (handle,
+                    uplo,
+                    transA,
+                    diag,
+                    N,
+                    dAp.ptr_on_device(),
+                    dx_or_b.ptr_on_device(),
+                    incx,
+                    batch_count));
         handle.post_test(arg);
         CHECK_HIP_ERROR(hx_or_b.transfer_from(dx_or_b));
 
+        if(arg.repeatability_check)
+        {
+            host_batch_vector<T> hx_or_b_copy(N, incx, batch_count);
+            CHECK_HIP_ERROR(hx_or_b_copy.memcheck());
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                // Allocate device memory
+                device_batch_matrix<T> dAp_copy(1, rocblas_packed_matrix_size(N), 1, batch_count);
+                device_batch_vector<T> dx_or_b_copy(N, incx, batch_count);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dAp_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dx_or_b_copy.memcheck());
+
+                CHECK_HIP_ERROR(dAp_copy.transfer_from(hAp));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dx_or_b_copy.transfer_from(hb));
+                    DAPI_CHECK(rocblas_tpsv_batched_fn,
+                               (handle_copy,
+                                uplo,
+                                transA,
+                                diag,
+                                N,
+                                dAp_copy.ptr_on_device(),
+                                dx_or_b_copy.ptr_on_device(),
+                                incx,
+                                batch_count));
+                    CHECK_HIP_ERROR(hx_or_b_copy.transfer_from(dx_or_b_copy));
+                    unit_check_general<T>(1, N, incx, hx_or_b, hx_or_b_copy, batch_count);
+                }
+            }
+            return;
+        }
+
         max_err = vector_norm_1(N, incx, hx, hx_or_b);
 
         if(arg.unit_check)
             trsm_err_res_check<T>(max_err, N, error_eps_multiplier, eps);
 
         // hx_or_b contains A * (calculated X), so res = A * (calculated x) - b = hx_or_b - hb
-        for(int b = 0; b < batch_count; b++)
+        for(size_t b = 0; b < batch_count; b++)
         {
-            cblas_trmv<T>(uplo, transA, diag, N, hA[b], N, hx_or_b[b], incx);
+            ref_trmv<T>(uplo, transA, diag, N, hA[b], N, hx_or_b[b], incx);
         }
 
         max_res = vector_norm_1(N, incx, hx_or_b, hb);
@@ -208,48 +249,42 @@ void testing_tpsv_batched(const Arguments& arg)
 
     if(arg.timing)
     {
-        // GPU rocBLAS
-        CHECK_HIP_ERROR(dx_or_b.transfer_from(cpu_x_or_b));
-
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
-
-        for(int i = 0; i < number_cold_calls; i++)
-            rocblas_tpsv_batched_fn(handle,
-                                    uplo,
-                                    transA,
-                                    diag,
-                                    N,
-                                    dAp.ptr_on_device(),
-                                    dx_or_b.ptr_on_device(),
-                                    incx,
-                                    batch_count);
+        double gpu_time_used, cpu_time_used = 0.0;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
         gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int i = 0; i < number_hot_calls; i++)
-            rocblas_tpsv_batched_fn(handle,
-                                    uplo,
-                                    transA,
-                                    diag,
-                                    N,
-                                    dAp.ptr_on_device(),
-                                    dx_or_b.ptr_on_device(),
-                                    incx,
-                                    batch_count);
+        for(int iter = 0; iter < total_calls; iter++)
+        {
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
 
-        gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+            DAPI_DISPATCH(rocblas_tpsv_batched_fn,
+                          (handle,
+                           uplo,
+                           transA,
+                           diag,
+                           N,
+                           dAp.ptr_on_device(),
+                           dx_or_b.ptr_on_device(),
+                           incx,
+                           batch_count));
+        }
+        gpu_time_used = get_time_us_sync(stream) - gpu_time_used; // in microseconds
 
         // CPU cblas
-        cpu_time_used = get_time_us_no_sync();
+        if(arg.unit_check || arg.norm_check)
+        {
+            cpu_time_used = get_time_us_no_sync();
 
-        if(arg.norm_check)
-            for(int b = 0; b < batch_count; b++)
-                cblas_tpsv<T>(uplo, transA, diag, N, hAp[b], cpu_x_or_b[b], incx);
+            for(size_t b = 0; b < batch_count; b++)
+                ref_tpsv<T>(uplo, transA, diag, N, hAp[b], cpu_x_or_b[b], incx);
 
-        cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+            cpu_time_used = get_time_us_no_sync() - cpu_time_used;
+        }
 
         ArgumentModel<e_uplo, e_transA, e_diag, e_N, e_incx, e_batch_count>{}.log_args<T>(
             rocblas_cout,
