@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,38 +22,28 @@
 
 #pragma once
 
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
-
-#include "blas3/Tensile/gemm.hpp"
+#include "blas3/rocblas_gemm.hpp"
+#include "frequency_monitor.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_gemm_batched_bad_arg(const Arguments& arg)
 {
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
-        auto rocblas_gemm_batched_fn
-            = arg.api == FORTRAN ? rocblas_gemm_batched<T, true> : rocblas_gemm_batched<T, false>;
+        auto rocblas_gemm_batched_fn    = arg.api & c_API_FORTRAN ? rocblas_gemm_batched<T, true>
+                                                                  : rocblas_gemm_batched<T, false>;
+        auto rocblas_gemm_batched_fn_64 = arg.api & c_API_FORTRAN
+                                              ? rocblas_gemm_batched_64<T, true>
+                                              : rocblas_gemm_batched_64<T, false>;
 
-        const rocblas_int M = 100;
-        const rocblas_int N = 100;
-        const rocblas_int K = 100;
+        const int64_t M = 100;
+        const int64_t N = 100;
+        const int64_t K = 100;
 
-        const rocblas_int lda = 100;
-        const rocblas_int ldb = 100;
-        const rocblas_int ldc = 100;
+        const int64_t lda = 100;
+        const int64_t ldb = 100;
+        const int64_t ldc = 100;
 
         const size_t safe_size = N * ldc;
 
@@ -84,7 +74,7 @@ void testing_gemm_batched_bad_arg(const Arguments& arg)
             zero = zero_d;
         }
 
-        rocblas_int batch_count = 2;
+        int64_t batch_count = 2;
 
         // allocate memory on device
         device_batch_vector<T> dA(safe_size, 1, batch_count);
@@ -97,85 +87,85 @@ void testing_gemm_batched_bad_arg(const Arguments& arg)
         // clang-format off
 
 // check for valid enum
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, (rocblas_operation) rocblas_side_both, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_value);
+DAPI_EXPECT( rocblas_status_invalid_value,rocblas_gemm_batched_fn,(handle, (rocblas_operation) rocblas_side_both, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, (rocblas_operation) rocblas_side_both, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_value);
+DAPI_EXPECT( rocblas_status_invalid_value,rocblas_gemm_batched_fn,(handle, transA, (rocblas_operation) rocblas_side_both, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
 // check for invalid size
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, -1, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, transA, transB, -1, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, -1, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, transA, transB, M, -1, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, -1, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, -1, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, -1), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, -1) );
 
 // check for invalid leading dimension
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
-dA.ptr_on_device(), M-1, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
+dA.ptr_on_device(), M-1, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), K-1, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), K-1, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, rocblas_operation_transpose, rocblas_operation_transpose, M, N, K, alpha,
-dA.ptr_on_device(), K-1, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, rocblas_operation_transpose, rocblas_operation_transpose, M, N, K, alpha,
+dA.ptr_on_device(), K-1, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, rocblas_operation_transpose, rocblas_operation_transpose, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), N-1, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, rocblas_operation_transpose, rocblas_operation_transpose, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), N-1, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), M-1, batch_count), rocblas_status_invalid_size);
+DAPI_EXPECT( rocblas_status_invalid_size,rocblas_gemm_batched_fn,(handle, rocblas_operation_none, rocblas_operation_none, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), M-1, batch_count) );
 
 // check that nullptr gives rocblas_status_invalid_handle or rocblas_status_invalid_pointer
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(nullptr, transA, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_handle);
+DAPI_EXPECT( rocblas_status_invalid_handle,rocblas_gemm_batched_fn,(nullptr, transA, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, nullptr,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_pointer);
+DAPI_EXPECT( rocblas_status_invalid_pointer,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, nullptr,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, alpha,
-nullptr, lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_pointer);
+DAPI_EXPECT( rocblas_status_invalid_pointer,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, alpha,
+nullptr, lda, dB.ptr_on_device(), ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_pointer);
+DAPI_EXPECT( rocblas_status_invalid_pointer,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, nullptr, dC.ptr_on_device(), ldc, batch_count), rocblas_status_invalid_pointer);
+DAPI_EXPECT( rocblas_status_invalid_pointer,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, nullptr, dC.ptr_on_device(), ldc, batch_count) );
 
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, alpha,
-dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, nullptr, ldc, batch_count), rocblas_status_invalid_pointer);
+DAPI_EXPECT( rocblas_status_invalid_pointer,rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, alpha,
+dA.ptr_on_device(), lda, dB.ptr_on_device(), ldb, beta, nullptr, ldc, batch_count) );
 
 // If batch_count==0, then all pointers can be nullptr without issue.
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, nullptr, nullptr, lda,
-nullptr, ldb, nullptr, nullptr, ldc, 0), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, nullptr, nullptr, lda,
+nullptr, ldb, nullptr, nullptr, ldc, 0));
 
 // If M==0, then all pointers can be nullptr without issue.
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, 0, N, K, nullptr, nullptr, lda,
-nullptr, ldb, nullptr, nullptr, ldc, batch_count), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, 0, N, K, nullptr, nullptr, lda,
+nullptr, ldb, nullptr, nullptr, ldc, batch_count));
 
 // If N==0, then A and B can be nullptr without issue.
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, 0, K, nullptr, nullptr, lda,
-nullptr, ldb, nullptr, nullptr, ldc, batch_count), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, M, 0, K, nullptr, nullptr, lda,
+nullptr, ldb, nullptr, nullptr, ldc, batch_count));
 
 // the following tests still output to C
 
 // If K==0, then alpha, A and B can be nullptr without issue.
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, 0,
-nullptr, nullptr, lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, M, N, 0,
+nullptr, nullptr, lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count));
 
 // If alpha==0, then A and B can be nullptr without issue
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, zero,
-nullptr, lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, zero,
+nullptr, lda, nullptr, ldb, beta, dC.ptr_on_device(), ldc, batch_count));
 
 // If alpha==0 and beta==1, then A, B and C can be nullptr without issue
-EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle, transA, transB, M, N, K, zero,
-nullptr, lda, nullptr, ldb, one, nullptr, ldc, batch_count), rocblas_status_success);
+DAPI_CHECK(rocblas_gemm_batched_fn,(handle, transA, transB, M, N, K, zero,
+nullptr, lda, nullptr, ldb, one, nullptr, ldc, batch_count));
 
         // clang-format on
     }
@@ -185,24 +175,28 @@ template <typename T>
 void testing_gemm_batched(const Arguments& arg)
 {
     auto rocblas_gemm_batched_fn
-        = arg.api == FORTRAN ? rocblas_gemm_batched<T, true> : rocblas_gemm_batched<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_gemm_batched<T, true> : rocblas_gemm_batched<T, false>;
+    auto rocblas_gemm_batched_fn_64 = arg.api & c_API_FORTRAN ? rocblas_gemm_batched_64<T, true>
+                                                              : rocblas_gemm_batched_64<T, false>;
 
     rocblas_local_handle handle{arg};
-    rocblas_int          M           = arg.M;
-    rocblas_int          N           = arg.N;
-    rocblas_int          K           = arg.K;
-    T                    h_alpha     = arg.get_alpha<T>();
-    T                    h_beta      = arg.get_beta<T>();
-    rocblas_int          lda         = arg.lda;
-    rocblas_int          ldb         = arg.ldb;
-    rocblas_int          ldc         = arg.ldc;
-    rocblas_int          batch_count = arg.batch_count;
-    rocblas_operation    transA      = char2rocblas_operation(arg.transA);
-    rocblas_operation    transB      = char2rocblas_operation(arg.transB);
-    rocblas_int          A_row       = transA == rocblas_operation_none ? M : std::max(K, 1);
-    rocblas_int          A_col       = transA == rocblas_operation_none ? std::max(K, 1) : M;
-    rocblas_int          B_row       = transB == rocblas_operation_none ? std::max(K, 1) : N;
-    rocblas_int          B_col       = transB == rocblas_operation_none ? N : std::max(K, 1);
+
+    int64_t           M           = arg.M;
+    int64_t           N           = arg.N;
+    int64_t           K           = arg.K;
+    T                 h_alpha     = arg.get_alpha<T>();
+    T                 h_beta      = arg.get_beta<T>();
+    int64_t           lda         = arg.lda;
+    int64_t           ldb         = arg.ldb;
+    int64_t           ldc         = arg.ldc;
+    int64_t           batch_count = arg.batch_count;
+    rocblas_operation transA      = char2rocblas_operation(arg.transA);
+    rocblas_operation transB      = char2rocblas_operation(arg.transB);
+
+    int64_t A_row = transA == rocblas_operation_none ? M : std::max(K, int64_t(1));
+    int64_t A_col = transA == rocblas_operation_none ? std::max(K, int64_t(1)) : M;
+    int64_t B_row = transB == rocblas_operation_none ? std::max(K, int64_t(1)) : N;
+    int64_t B_col = transB == rocblas_operation_none ? N : std::max(K, int64_t(1));
 
     rocblas_math_mode math_mode = rocblas_math_mode(arg.math_mode);
     CHECK_ROCBLAS_ERROR(rocblas_set_math_mode(handle, math_mode));
@@ -220,28 +214,27 @@ void testing_gemm_batched(const Arguments& arg)
         = M < 0 || N < 0 || K < 0 || lda < A_row || ldb < B_row || ldc < M || batch_count < 0;
     if(invalid_size || !M || !N || !batch_count)
     {
-        EXPECT_ROCBLAS_STATUS(rocblas_gemm_batched_fn(handle,
-                                                      transA,
-                                                      transB,
-                                                      M,
-                                                      N,
-                                                      K,
-                                                      nullptr,
-                                                      nullptr,
-                                                      lda,
-                                                      nullptr,
-                                                      ldb,
-                                                      nullptr,
-                                                      nullptr,
-                                                      ldc,
-                                                      batch_count),
-                              invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
-
+        DAPI_EXPECT(invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+                    rocblas_gemm_batched_fn,
+                    (handle,
+                     transA,
+                     transB,
+                     M,
+                     N,
+                     K,
+                     nullptr,
+                     nullptr,
+                     lda,
+                     nullptr,
+                     ldb,
+                     nullptr,
+                     nullptr,
+                     ldc,
+                     batch_count));
         return;
     }
 
     double cpu_time_used = 0.0;
-
     double rocblas_error = 0.0, error_hst_ptr = 0.0, error_dev_ptr = 0.0;
 
 #ifdef ROCBLAS_BENCH
@@ -250,21 +243,22 @@ void testing_gemm_batched(const Arguments& arg)
         device_batch_vector<T> dA(1, 1, batch_count);
         device_batch_vector<T> dB(1, 1, batch_count);
         device_batch_vector<T> dC(1, 1, batch_count);
-        CHECK_ROCBLAS_ERROR((rocblas_gemm_batched_fn(handle,
-                                                     transA,
-                                                     transB,
-                                                     M,
-                                                     N,
-                                                     K,
-                                                     &h_alpha,
-                                                     dA.ptr_on_device(),
-                                                     lda,
-                                                     dB.ptr_on_device(),
-                                                     ldb,
-                                                     &h_beta,
-                                                     dC.ptr_on_device(),
-                                                     ldc,
-                                                     batch_count)));
+        DAPI_CHECK(rocblas_gemm_batched_fn,
+                   (handle,
+                    transA,
+                    transB,
+                    M,
+                    N,
+                    K,
+                    &h_alpha,
+                    dA.ptr_on_device(),
+                    lda,
+                    dB.ptr_on_device(),
+                    ldb,
+                    &h_beta,
+                    dC.ptr_on_device(),
+                    ldc,
+                    batch_count));
         return;
     }
 #endif
@@ -322,21 +316,22 @@ void testing_gemm_batched(const Arguments& arg)
             handle.pre_test(arg);
             if(arg.api != INTERNAL)
             {
-                CHECK_ROCBLAS_ERROR((rocblas_gemm_batched_fn(handle,
-                                                             transA,
-                                                             transB,
-                                                             M,
-                                                             N,
-                                                             K,
-                                                             &h_alpha,
-                                                             dA.ptr_on_device(),
-                                                             lda,
-                                                             dB.ptr_on_device(),
-                                                             ldb,
-                                                             &h_beta,
-                                                             dC.ptr_on_device(),
-                                                             ldc,
-                                                             batch_count)));
+                DAPI_CHECK(rocblas_gemm_batched_fn,
+                           (handle,
+                            transA,
+                            transB,
+                            M,
+                            N,
+                            K,
+                            &h_alpha,
+                            dA.ptr_on_device(),
+                            lda,
+                            dB.ptr_on_device(),
+                            ldb,
+                            &h_beta,
+                            dC.ptr_on_device(),
+                            ldc,
+                            batch_count));
             }
             else
             {
@@ -379,21 +374,92 @@ void testing_gemm_batched(const Arguments& arg)
 
             if(arg.api != INTERNAL)
             {
-                CHECK_ROCBLAS_ERROR((rocblas_gemm_batched_fn(handle,
-                                                             transA,
-                                                             transB,
-                                                             M,
-                                                             N,
-                                                             K,
-                                                             d_alpha,
-                                                             dA.ptr_on_device(),
-                                                             lda,
-                                                             dB.ptr_on_device(),
-                                                             ldb,
-                                                             d_beta,
-                                                             dC.ptr_on_device(),
-                                                             ldc,
-                                                             batch_count)));
+                DAPI_CHECK(rocblas_gemm_batched_fn,
+                           (handle,
+                            transA,
+                            transB,
+                            M,
+                            N,
+                            K,
+                            d_alpha,
+                            dA.ptr_on_device(),
+                            lda,
+                            dB.ptr_on_device(),
+                            ldb,
+                            d_beta,
+                            dC.ptr_on_device(),
+                            ldc,
+                            batch_count));
+
+                if(arg.repeatability_check)
+                {
+                    host_batch_matrix<T> hC_copy(M, N, ldc, batch_count);
+                    CHECK_HIP_ERROR(hC.transfer_from(dC));
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        //Allocate device memory in new device
+                        device_batch_matrix<T> dA_copy(
+                            A_row, A_col, lda, batch_count, false, offsetA);
+                        device_batch_matrix<T> dB_copy(
+                            B_row, B_col, ldb, batch_count, false, offsetB);
+                        device_batch_matrix<T> dC_copy(M, N, ldc, batch_count, false, offsetC);
+                        device_vector<T>       d_alpha_copy(1);
+                        device_vector<T>       d_beta_copy(1);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dB_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(dC_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(d_alpha_copy.memcheck());
+                        CHECK_DEVICE_ALLOCATION(d_beta_copy.memcheck());
+
+                        // copy data from CPU to device
+                        CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                        CHECK_HIP_ERROR(dB_copy.transfer_from(hB));
+                        CHECK_HIP_ERROR(d_alpha_copy.transfer_from(halpha));
+                        CHECK_HIP_ERROR(d_beta_copy.transfer_from(hbeta));
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            CHECK_HIP_ERROR(dC_copy.transfer_from(hC_gold));
+
+                            DAPI_CHECK(rocblas_gemm_batched_fn,
+                                       (handle_copy,
+                                        transA,
+                                        transB,
+                                        M,
+                                        N,
+                                        K,
+                                        d_alpha_copy,
+                                        dA_copy.ptr_on_device(),
+                                        lda,
+                                        dB_copy.ptr_on_device(),
+                                        ldb,
+                                        d_beta_copy,
+                                        dC_copy.ptr_on_device(),
+                                        ldc,
+                                        batch_count));
+
+                            CHECK_HIP_ERROR(hC_copy.transfer_from(dC_copy));
+                            unit_check_general<T>(M, N, ldc, hC, hC_copy, batch_count);
+                        }
+                    }
+                    return;
+                }
             }
             else
             {
@@ -427,7 +493,7 @@ void testing_gemm_batched(const Arguments& arg)
         // For the xf32 xdl math op, cast type of A/B from float to xfloat32 .
         if(std::is_same<T, float>{} && math_mode == rocblas_xf32_xdl_math_op)
         {
-            for(int b = 0; b < batch_count; b++)
+            for(int64_t b = 0; b < batch_count; b++)
             {
                 type_to_xdl_math_op_type<rocblas_xfloat32, float>(hA[b], hA.nmemb());
                 type_to_xdl_math_op_type<rocblas_xfloat32, float>(hB[b], hB.nmemb());
@@ -436,16 +502,14 @@ void testing_gemm_batched(const Arguments& arg)
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        for(rocblas_int b = 0; b < batch_count; b++)
+        for(int64_t b = 0; b < batch_count; b++)
         {
-            cblas_gemm<T>(
+            ref_gemm<T>(
                 transA, transB, M, N, K, h_alpha, hA[b], lda, hB[b], ldb, h_beta, hC_gold[b], ldc);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        // GPU fetch
-        if(arg.pointer_mode_host)
-        {
+        auto compare_to_gold = [&] {
             if(arg.unit_check)
             {
                 if(std::is_same_v<T,
@@ -454,10 +518,8 @@ void testing_gemm_batched(const Arguments& arg)
                     const double tol = K * sum_error_tolerance_for_gfx11<T, T, T>;
                     near_check_general<T>(M, N, ldc, hC_gold, hC, batch_count, tol);
                 }
-                else if(std::is_same_v<T, rocblas_half> && K > 10000)
+                else if(reduction_requires_near<T>(arg, K))
                 {
-                    // For large K, rocblas_half tends to diverge proportional to K
-                    // Tolerance is slightly greater than 1 / 1024.0
                     const double tol = K * sum_error_tolerance<T>;
                     near_check_general<T>(M, N, ldc, hC_gold, hC, batch_count, tol);
                 }
@@ -466,97 +528,69 @@ void testing_gemm_batched(const Arguments& arg)
                     unit_check_general<T>(M, N, ldc, hC_gold, hC, batch_count);
                 }
             }
+
+            double error = 0;
             if(arg.norm_check)
             {
-                error_hst_ptr
-                    = std::abs(norm_check_general<T>('F', M, N, ldc, hC_gold, hC, batch_count));
+                error = std::abs(norm_check_general<T>('F', M, N, ldc, hC_gold, hC, batch_count));
             }
+            return error;
+        };
+
+        // check error and norm
+        if(arg.pointer_mode_host)
+        {
+            error_hst_ptr = compare_to_gold();
         }
         if(arg.pointer_mode_device)
         {
             CHECK_HIP_ERROR(hC.transfer_from(dC));
-
-            if(arg.unit_check)
-            {
-                if(std::is_same_v<T,
-                                  rocblas_half> && (rocblas_handle(handle)->getArchMajor() == 11))
-                {
-                    const double tol = K * sum_error_tolerance_for_gfx11<T, T, T>;
-                    near_check_general<T>(M, N, ldc, hC_gold, hC, batch_count, tol);
-                }
-                else if(std::is_same_v<T, rocblas_half> && K > 10000)
-                {
-                    // For large K, rocblas_half tends to diverge proportional to K
-                    // Tolerance is slightly greater than 1 / 1024.0
-                    const double tol = K * sum_error_tolerance<T>;
-                    near_check_general<T>(M, N, ldc, hC_gold, hC, batch_count, tol);
-                }
-                else
-                {
-                    unit_check_general<T>(M, N, ldc, hC_gold, hC, batch_count);
-                }
-            }
-            if(arg.norm_check)
-            {
-                error_dev_ptr
-                    = std::abs(norm_check_general<T>('F', M, N, ldc, hC_gold, hC, batch_count));
-            }
+            error_dev_ptr = compare_to_gold();
         }
-
         rocblas_error = error_dev_ptr > error_hst_ptr ? error_dev_ptr : error_hst_ptr;
     }
 
     if(arg.timing && arg.api != INTERNAL)
     {
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
+        double gpu_time_used     = 0.0;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        for(int i = 0; i < number_cold_calls; i++)
-        {
-            CHECK_ROCBLAS_ERROR((rocblas_gemm_batched_fn(handle,
-                                                         transA,
-                                                         transB,
-                                                         M,
-                                                         N,
-                                                         K,
-                                                         &h_alpha,
-                                                         dA.ptr_on_device(),
-                                                         lda,
-                                                         dB.ptr_on_device(),
-                                                         ldb,
-                                                         &h_beta,
-                                                         dC.ptr_on_device(),
-                                                         ldc,
-                                                         batch_count)));
-        }
-
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        double gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int i = 0; i < number_hot_calls; i++)
+        FrequencyMonitor& freq_monitor = getFrequencyMonitor();
+        freq_monitor.start();
+
+        for(int i = 0; i < total_calls; i++)
         {
-            rocblas_gemm_batched_fn(handle,
-                                    transA,
-                                    transB,
-                                    M,
-                                    N,
-                                    K,
-                                    &h_alpha,
-                                    dA.ptr_on_device(),
-                                    lda,
-                                    dB.ptr_on_device(),
-                                    ldb,
-                                    &h_beta,
-                                    dC.ptr_on_device(),
-                                    ldc,
-                                    batch_count);
+            if(i == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream); // in microseconds
+
+            DAPI_DISPATCH(rocblas_gemm_batched_fn,
+                          (handle,
+                           transA,
+                           transB,
+                           M,
+                           N,
+                           K,
+                           &h_alpha,
+                           dA.ptr_on_device(),
+                           lda,
+                           dB.ptr_on_device(),
+                           ldb,
+                           &h_beta,
+                           dC.ptr_on_device(),
+                           ldc,
+                           batch_count));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
 
+        freq_monitor.stop();
+
         ArgumentModel<e_transA,
                       e_transB,
                       e_M,
@@ -574,6 +608,9 @@ void testing_gemm_batched(const Arguments& arg)
                          gemm_gflop_count<T>(M, N, K),
                          ArgumentLogging::NA_value,
                          cpu_time_used,
-                         rocblas_error);
+                         rocblas_error,
+                         ArgumentLogging::NA_value,
+                         ArgumentLogging::NA_value,
+                         ArgumentLogging::NA_value);
     }
 }
