@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "near.hpp"
 #include "norm.hpp"
@@ -34,14 +35,14 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 #include "blas3/rocblas_trtri.hpp"
 
 template <typename T>
 void testing_trtri_bad_arg(const Arguments& arg)
 {
-    auto rocblas_trtri_fn = arg.api == FORTRAN ? rocblas_trtri<T, true> : rocblas_trtri<T, false>;
+    auto rocblas_trtri_fn
+        = arg.api & c_API_FORTRAN ? rocblas_trtri<T, true> : rocblas_trtri<T, false>;
 
     rocblas_local_handle handle{arg};
 
@@ -94,7 +95,8 @@ void testing_trtri_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_trtri(const Arguments& arg)
 {
-    auto rocblas_trtri_fn = arg.api == FORTRAN ? rocblas_trtri<T, true> : rocblas_trtri<T, false>;
+    auto rocblas_trtri_fn
+        = arg.api & c_API_FORTRAN ? rocblas_trtri<T, true> : rocblas_trtri<T, false>;
 
     rocblas_int N = arg.N;
     rocblas_int lda;
@@ -192,6 +194,42 @@ void testing_trtri(const Arguments& arg)
         if(arg.api != INTERNAL)
         {
             CHECK_ROCBLAS_ERROR(rocblas_trtri_fn(handle, uplo, diag, N, dA, lda, dinvA, ldinvA));
+            if(arg.repeatability_check)
+            {
+                host_matrix<T> hA_copy(N, N, lda);
+                CHECK_HIP_ERROR(hA.transfer_from(dinvA));
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_matrix<T> dA_copy(N, N, lda);
+                    device_matrix<T> dinvA_copy(N, N, lda);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dinvA_copy.memcheck());
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dA_copy.transfer_from(hB));
+                        CHECK_HIP_ERROR(dinvA_copy.transfer_from(hB));
+                        CHECK_ROCBLAS_ERROR(rocblas_trtri_fn(
+                            handle_copy, uplo, diag, N, dA_copy, lda, dinvA_copy, ldinvA));
+                        CHECK_HIP_ERROR(hA_copy.transfer_from(dinvA_copy));
+                        unit_check_general<T>(N, N, lda, hA, hA_copy);
+                    }
+                }
+                return;
+            }
         }
         else
         {
