@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "near.hpp"
 #include "norm.hpp"
@@ -34,15 +35,14 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 #include "blas3/rocblas_trtri.hpp"
 
 template <typename T>
 void testing_trtri_batched_bad_arg(const Arguments& arg)
 {
-    auto rocblas_trtri_batched_fn
-        = arg.api == FORTRAN ? rocblas_trtri_batched<T, true> : rocblas_trtri_batched<T, false>;
+    auto rocblas_trtri_batched_fn = arg.api & c_API_FORTRAN ? rocblas_trtri_batched<T, true>
+                                                            : rocblas_trtri_batched<T, false>;
 
     rocblas_local_handle handle{arg};
 
@@ -158,8 +158,8 @@ void testing_trtri_batched_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_trtri_batched(const Arguments& arg)
 {
-    auto rocblas_trtri_batched_fn
-        = arg.api == FORTRAN ? rocblas_trtri_batched<T, true> : rocblas_trtri_batched<T, false>;
+    auto rocblas_trtri_batched_fn = arg.api & c_API_FORTRAN ? rocblas_trtri_batched<T, true>
+                                                            : rocblas_trtri_batched<T, false>;
 
     rocblas_int N           = arg.N;
     rocblas_int lda         = arg.lda;
@@ -290,6 +290,50 @@ void testing_trtri_batched(const Arguments& arg)
                                                          dinvA.ptr_on_device(),
                                                          lda,
                                                          batch_count));
+            if(arg.repeatability_check)
+            {
+                host_batch_matrix<T> hA_copy(N, N, lda, batch_count);
+                CHECK_HIP_ERROR(hA.transfer_from(dinvA));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_matrix<T> dA_copy(N, N, lda, batch_count, false, offsetA);
+                    device_batch_matrix<T> dinvA_copy(N, N, lda, batch_count, false, offsetinvA);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dinvA_copy.memcheck());
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dA_copy.transfer_from(hB));
+                        CHECK_HIP_ERROR(dinvA_copy.transfer_from(hB));
+                        CHECK_ROCBLAS_ERROR(rocblas_trtri_batched_fn(handle_copy,
+                                                                     uplo,
+                                                                     diag,
+                                                                     N,
+                                                                     dA_copy.ptr_on_device(),
+                                                                     lda,
+                                                                     dinvA_copy.ptr_on_device(),
+                                                                     lda,
+                                                                     batch_count));
+                        CHECK_HIP_ERROR(hA_copy.transfer_from(dinvA_copy));
+                        unit_check_general<T>(N, N, lda, hA, hA_copy, batch_count);
+                    }
+                }
+                return;
+            }
         }
         else
         {
