@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,9 +29,9 @@ template <typename Ta, typename Tx = Ta, typename Ty = Tx, typename Tex = Ty>
 void testing_axpy_batched_ex_bad_arg(const Arguments& arg)
 {
     auto rocblas_axpy_batched_ex_fn
-        = arg.api == FORTRAN ? rocblas_axpy_batched_ex_fortran : rocblas_axpy_batched_ex;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_ex_fortran : rocblas_axpy_batched_ex;
     auto rocblas_axpy_batched_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_axpy_batched_ex_64_fortran : rocblas_axpy_batched_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_ex_64_fortran : rocblas_axpy_batched_ex_64;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -185,9 +185,9 @@ template <typename Ta, typename Tx = Ta, typename Ty = Tx, typename Tex = Ty>
 void testing_axpy_batched_ex(const Arguments& arg)
 {
     auto rocblas_axpy_batched_ex_fn
-        = arg.api == FORTRAN ? rocblas_axpy_batched_ex_fortran : rocblas_axpy_batched_ex;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_ex_fortran : rocblas_axpy_batched_ex;
     auto rocblas_axpy_batched_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_axpy_batched_ex_64_fortran : rocblas_axpy_batched_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_batched_ex_64_fortran : rocblas_axpy_batched_ex_64;
 
     rocblas_datatype alpha_type     = arg.a_type;
     rocblas_datatype x_type         = arg.b_type;
@@ -280,51 +280,117 @@ void testing_axpy_batched_ex(const Arguments& arg)
         // Call routine with pointer mode on host.
 
         // Pointer mode.
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
+        if(arg.pointer_mode_host)
+        {
+            CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        // Call routine.
-        CHECK_HIP_ERROR(dy.transfer_from(hy));
-        handle.pre_test(arg);
-        DAPI_CHECK(rocblas_axpy_batched_ex_fn,
-                   (handle,
-                    N,
-                    halpha,
-                    alpha_type,
-                    dx.ptr_on_device(),
-                    x_type,
-                    incx,
-                    dy.ptr_on_device(),
-                    y_type,
-                    incy,
-                    batch_count,
-                    execution_type));
-        handle.post_test(arg);
-        // Transfer from device to host.
-        CHECK_HIP_ERROR(hy1.transfer_from(dy));
+            // Call routine.
+            CHECK_HIP_ERROR(dy.transfer_from(hy));
+            handle.pre_test(arg);
+            DAPI_CHECK(rocblas_axpy_batched_ex_fn,
+                       (handle,
+                        N,
+                        halpha,
+                        alpha_type,
+                        dx.ptr_on_device(),
+                        x_type,
+                        incx,
+                        dy.ptr_on_device(),
+                        y_type,
+                        incy,
+                        batch_count,
+                        execution_type));
+            handle.post_test(arg);
+            // Transfer from device to host.
+            CHECK_HIP_ERROR(hy1.transfer_from(dy));
+        }
 
-        // Pointer mode.
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
+        if(arg.pointer_mode_device)
+        {
+            CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
 
-        // Call routine.
-        CHECK_HIP_ERROR(dalpha.transfer_from(halpha));
-        CHECK_HIP_ERROR(dy.transfer_from(hy));
-        handle.pre_test(arg);
-        DAPI_CHECK(rocblas_axpy_batched_ex_fn,
-                   (handle,
-                    N,
-                    dalpha,
-                    alpha_type,
-                    dx.ptr_on_device(),
-                    x_type,
-                    incx,
-                    dy.ptr_on_device(),
-                    y_type,
-                    incy,
-                    batch_count,
-                    execution_type));
-        handle.post_test(arg);
-        // Transfer from device to host.
-        CHECK_HIP_ERROR(hy2.transfer_from(dy));
+            // Call routine.
+            CHECK_HIP_ERROR(dalpha.transfer_from(halpha));
+            CHECK_HIP_ERROR(dy.transfer_from(hy));
+            handle.pre_test(arg);
+            DAPI_CHECK(rocblas_axpy_batched_ex_fn,
+                       (handle,
+                        N,
+                        dalpha,
+                        alpha_type,
+                        dx.ptr_on_device(),
+                        x_type,
+                        incx,
+                        dy.ptr_on_device(),
+                        y_type,
+                        incy,
+                        batch_count,
+                        execution_type));
+            handle.post_test(arg);
+
+            // Transfer from device to host.
+            CHECK_HIP_ERROR(hy2.transfer_from(dy));
+
+            if(arg.repeatability_check)
+            {
+                host_batch_vector<Ty> hy_copy(N, incy, batch_count);
+                CHECK_HIP_ERROR(hy_copy.memcheck());
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_vector<Tx> dx_copy(N, incx, batch_count);
+                    device_batch_vector<Ty> dy_copy(N, incy, batch_count);
+                    device_vector<Ta>       dalpha_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dalpha_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dalpha_copy.transfer_from(halpha));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                        DAPI_CHECK(rocblas_axpy_batched_ex_fn,
+                                   (handle_copy,
+                                    N,
+                                    dalpha_copy,
+                                    alpha_type,
+                                    dx_copy.ptr_on_device(),
+                                    x_type,
+                                    incx,
+                                    dy_copy.ptr_on_device(),
+                                    y_type,
+                                    incy,
+                                    batch_count,
+                                    execution_type));
+
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+
+                        unit_check_general<Ty>(1, N, incy, hy2, hy_copy, batch_count);
+                    }
+                }
+                return;
+            }
+        }
 
         // CPU BLAS
         {
@@ -333,7 +399,7 @@ void testing_axpy_batched_ex(const Arguments& arg)
             // Compute the host solution.
             for(int64_t b = 0; b < batch_count; ++b)
             {
-                cblas_axpy<Tex>(N, h_alpha_ex, hx_ex[b], incx, hy_ex[b], incy);
+                ref_axpy<Tex>(N, h_alpha_ex, hx_ex[b], incx, hy_ex[b], incy);
             }
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -344,18 +410,32 @@ void testing_axpy_batched_ex(const Arguments& arg)
             }
         }
 
-        // Compare with with hsolution.
-        if(arg.unit_check)
+        if(arg.pointer_mode_host)
         {
-            unit_check_general<Ty>(1, N, incy, hy, hy1, batch_count);
+            // Compare with with hsolution.
+            if(arg.unit_check)
+            {
+                unit_check_general<Ty>(1, N, incy, hy, hy1, batch_count);
+            }
 
-            unit_check_general<Ty>(1, N, incy, hy, hy2, batch_count);
+            if(arg.norm_check)
+            {
+                rocblas_error_1 = norm_check_general<Ty>('I', 1, N, incy, hy, hy1, batch_count);
+            }
         }
-
-        if(arg.norm_check)
+        if(arg.pointer_mode_device)
         {
-            rocblas_error_1 = norm_check_general<Ty>('I', 1, N, incy, hy, hy1, batch_count);
-            rocblas_error_2 = norm_check_general<Ty>('I', 1, N, incy, hy, hy2, batch_count);
+            // Compare with with hsolution.
+            if(arg.unit_check)
+            {
+
+                unit_check_general<Ty>(1, N, incy, hy, hy2, batch_count);
+            }
+
+            if(arg.norm_check)
+            {
+                rocblas_error_2 = norm_check_general<Ty>('I', 1, N, incy, hy, hy2, batch_count);
+            }
         }
     }
 
