@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,9 +28,9 @@
 template <typename Ta, typename Tx = Ta, typename Ty = Tx, typename Tex = Ty>
 void testing_axpy_ex_bad_arg(const Arguments& arg)
 {
-    auto rocblas_axpy_ex_fn = arg.api == FORTRAN ? rocblas_axpy_ex_fortran : rocblas_axpy_ex;
+    auto rocblas_axpy_ex_fn = arg.api & c_API_FORTRAN ? rocblas_axpy_ex_fortran : rocblas_axpy_ex;
     auto rocblas_axpy_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_axpy_ex_64_fortran : rocblas_axpy_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_ex_64_fortran : rocblas_axpy_ex_64;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -142,9 +142,9 @@ void testing_axpy_ex_bad_arg(const Arguments& arg)
 template <typename Ta, typename Tx = Ta, typename Ty = Tx, typename Tex = Ty>
 void testing_axpy_ex(const Arguments& arg)
 {
-    auto rocblas_axpy_ex_fn = arg.api == FORTRAN ? rocblas_axpy_ex_fortran : rocblas_axpy_ex;
+    auto rocblas_axpy_ex_fn = arg.api & c_API_FORTRAN ? rocblas_axpy_ex_fortran : rocblas_axpy_ex;
     auto rocblas_axpy_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_axpy_ex_64_fortran : rocblas_axpy_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_axpy_ex_64_fortran : rocblas_axpy_ex_64;
 
     rocblas_datatype alpha_type     = arg.a_type;
     rocblas_datatype x_type         = arg.b_type;
@@ -281,13 +281,71 @@ void testing_axpy_ex(const Arguments& arg)
                         incy,
                         execution_type));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<Ty> hy_copy(N, incy);
+                CHECK_HIP_ERROR(hy_copy.memcheck());
+                CHECK_HIP_ERROR(hy.transfer_from(dy));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_vector<Tx> dx_copy(N, incx);
+                    device_vector<Ty> dy_copy(N, incy);
+                    device_vector<Ta> dalpha_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dalpha_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(
+                        hipMemcpy(dalpha_copy, &h_alpha, sizeof(Ta), hipMemcpyHostToDevice));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                        DAPI_CHECK(rocblas_axpy_ex_fn,
+                                   (handle_copy,
+                                    N,
+                                    dalpha_copy,
+                                    alpha_type,
+                                    dx_copy,
+                                    x_type,
+                                    incx,
+                                    dy_copy,
+                                    y_type,
+                                    incy,
+                                    execution_type));
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                        unit_check_general<Ty>(1, N, incy, hy, hy_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
 
-        // cblas_axpy<Ta>(N, h_alpha, hx, incx, hy_gold, incy);
-        cblas_axpy<Tex>(N, h_alpha_ex, hx_ex, incx, hy_gold_ex, incy);
+        // ref_axpy<Ta>(N, h_alpha, hx, incx, hy_gold, incy);
+        ref_axpy<Tex>(N, h_alpha_ex, hx_ex, incx, hy_gold_ex, incy);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
