@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,12 +27,12 @@
 template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr, bool CONJ = false>
 void testing_dot_ex_bad_arg(const Arguments& arg)
 {
-    auto rocblas_dot_ex_fn = arg.api == FORTRAN
+    auto rocblas_dot_ex_fn = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc_ex_fortran : rocblas_dot_ex_fortran)
                                  : (CONJ ? rocblas_dotc_ex : rocblas_dot_ex);
     auto rocblas_dot_ex_fn_64
-        = arg.api == FORTRAN_64 ? (CONJ ? rocblas_dotc_ex_64_fortran : rocblas_dot_ex_64_fortran)
-                                : (CONJ ? rocblas_dotc_ex_64 : rocblas_dot_ex_64);
+        = arg.api & c_API_FORTRAN ? (CONJ ? rocblas_dotc_ex_64_fortran : rocblas_dot_ex_64_fortran)
+                                  : (CONJ ? rocblas_dotc_ex_64 : rocblas_dot_ex_64);
 
     rocblas_datatype x_type         = rocblas_type2datatype<Tx>();
     rocblas_datatype y_type         = rocblas_type2datatype<Ty>();
@@ -103,21 +103,15 @@ void testing_dot_ex_bad_arg(const Arguments& arg)
         (handle, N, dx, x_type, incx, dy, y_type, incy, nullptr, result_type, execution_type));
 }
 
-template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr>
-void testing_dotc_ex_bad_arg(const Arguments& arg)
-{
-    testing_dot_ex_bad_arg<Tx, Ty, Tr, Tex, true>(arg);
-}
-
 template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr, bool CONJ = false>
 void testing_dot_ex(const Arguments& arg)
 {
-    auto rocblas_dot_ex_fn = arg.api == FORTRAN
+    auto rocblas_dot_ex_fn = arg.api & c_API_FORTRAN
                                  ? (CONJ ? rocblas_dotc_ex_fortran : rocblas_dot_ex_fortran)
                                  : (CONJ ? rocblas_dotc_ex : rocblas_dot_ex);
     auto rocblas_dot_ex_fn_64
-        = arg.api == FORTRAN_64 ? (CONJ ? rocblas_dotc_ex_64_fortran : rocblas_dot_ex_64_fortran)
-                                : (CONJ ? rocblas_dotc_ex_64 : rocblas_dot_ex_64);
+        = arg.api & c_API_FORTRAN ? (CONJ ? rocblas_dotc_ex_64_fortran : rocblas_dot_ex_64_fortran)
+                                  : (CONJ ? rocblas_dotc_ex_64 : rocblas_dot_ex_64);
 
     rocblas_datatype x_type         = arg.a_type;
     rocblas_datatype y_type         = arg.b_type;
@@ -249,11 +243,71 @@ void testing_dot_ex(const Arguments& arg)
                         result_type,
                         execution_type));
             handle.post_test(arg);
+            if(arg.repeatability_check)
+            {
+                host_vector<Tr> rocblas_result_device_copy(1, 1);
+                CHECK_HIP_ERROR(rocblas_result_device_copy.memcheck());
+
+                CHECK_HIP_ERROR(rocblas_result_device.transfer_from(d_rocblas_result_device));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_vector<Tx> dx_copy(N, incx);
+                    device_vector<Ty> dy_copy(N, incy);
+                    device_vector<Tr> d_rocblas_result_device_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_device_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                    auto dy_ptr_copy = (arg.algo) ? (Tx*)(dx_copy) : (Ty*)(dy_copy);
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_dot_ex_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy,
+                                    x_type,
+                                    incx,
+                                    dy_ptr_copy,
+                                    y_type,
+                                    incy,
+                                    d_rocblas_result_device_copy,
+                                    result_type,
+                                    execution_type));
+                        CHECK_HIP_ERROR(
+                            rocblas_result_device_copy.transfer_from(d_rocblas_result_device_copy));
+                        unit_check_general<Tr>(
+                            1, 1, 1, rocblas_result_device, rocblas_result_device_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        (CONJ ? cblas_dotc<Tx> : cblas_dot<Tx>)(N, hx, incx, hy_ptr, incy, cpu_result);
+        (CONJ ? ref_dotc<Tx, Tr> : ref_dot<Tx, Tr>)(N, hx, incx, hy_ptr, incy, cpu_result);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
         // For large N, rocblas_half tends to diverge proportional to N
@@ -348,9 +402,3 @@ void testing_dot_ex(const Arguments& arg)
                                                                   rocblas_error_device);
     }
 }
-
-template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr>
-void testing_dotc_ex(const Arguments& arg)
-{
-    testing_dot_ex<Tx, Ty, Tr, Tex, true>(arg);
-}
