@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "norm.hpp"
 #include "rocblas.hpp"
@@ -34,14 +35,13 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 /* ============================================================================================ */
 
 template <typename T>
 void testing_geam_ex_bad_arg(const Arguments& arg)
 {
-    auto rocblas_geam_ex_fn = arg.api == FORTRAN ? rocblas_geam_ex : rocblas_geam_ex_fortran;
+    auto rocblas_geam_ex_fn = arg.api & c_API_FORTRAN ? rocblas_geam_ex : rocblas_geam_ex_fortran;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -511,7 +511,7 @@ void testing_geam_ex_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_geam_ex(const Arguments& arg)
 {
-    auto rocblas_geam_ex_fn = arg.api == FORTRAN ? rocblas_geam_ex : rocblas_geam_ex_fortran;
+    auto rocblas_geam_ex_fn = arg.api & c_API_FORTRAN ? rocblas_geam_ex : rocblas_geam_ex_fortran;
 
     rocblas_operation transA = char2rocblas_operation(arg.transA);
     rocblas_operation transB = char2rocblas_operation(arg.transB);
@@ -647,95 +647,190 @@ void testing_geam_ex(const Arguments& arg)
     if(arg.unit_check || arg.norm_check)
     {
         // ROCBLAS
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-        handle.pre_test(arg);
-        CHECK_ROCBLAS_ERROR(rocblas_geam_ex_fn(handle,
-                                               transA,
-                                               transB,
-                                               M,
-                                               N,
-                                               K,
-                                               &alpha,
-                                               dA,
-                                               a_type,
-                                               lda,
-                                               dB,
-                                               b_type,
-                                               ldb,
-                                               &beta,
-                                               dC,
-                                               c_type,
-                                               ldc,
-                                               dD,
-                                               d_type,
-                                               ldd,
-                                               compute_type,
-                                               geam_ex_op));
-        handle.post_test(arg);
-        CHECK_HIP_ERROR(hD_1.transfer_from(dD));
-        CHECK_HIP_ERROR(dD.transfer_from(hD_2));
+        if(arg.pointer_mode_host)
+        {
+            CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
+            handle.pre_test(arg);
+            CHECK_ROCBLAS_ERROR(rocblas_geam_ex_fn(handle,
+                                                   transA,
+                                                   transB,
+                                                   M,
+                                                   N,
+                                                   K,
+                                                   &alpha,
+                                                   dA,
+                                                   a_type,
+                                                   lda,
+                                                   dB,
+                                                   b_type,
+                                                   ldb,
+                                                   &beta,
+                                                   dC,
+                                                   c_type,
+                                                   ldc,
+                                                   dD,
+                                                   d_type,
+                                                   ldd,
+                                                   compute_type,
+                                                   geam_ex_op));
+            handle.post_test(arg);
+            CHECK_HIP_ERROR(hD_1.transfer_from(dD));
+            CHECK_HIP_ERROR(dD.transfer_from(hD_2));
+        }
 
-        CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
-        CHECK_ROCBLAS_ERROR(rocblas_geam_ex_fn(handle,
-                                               transA,
-                                               transB,
-                                               M,
-                                               N,
-                                               K,
-                                               d_alpha,
-                                               dA,
-                                               a_type,
-                                               lda,
-                                               dB,
-                                               b_type,
-                                               ldb,
-                                               d_beta,
-                                               dC,
-                                               c_type,
-                                               ldc,
-                                               dD,
-                                               d_type,
-                                               ldd,
-                                               compute_type,
-                                               geam_ex_op));
+        if(arg.pointer_mode_device)
+        {
+            CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
+            CHECK_ROCBLAS_ERROR(rocblas_geam_ex_fn(handle,
+                                                   transA,
+                                                   transB,
+                                                   M,
+                                                   N,
+                                                   K,
+                                                   d_alpha,
+                                                   dA,
+                                                   a_type,
+                                                   lda,
+                                                   dB,
+                                                   b_type,
+                                                   ldb,
+                                                   d_beta,
+                                                   dC,
+                                                   c_type,
+                                                   ldc,
+                                                   dD,
+                                                   d_type,
+                                                   ldd,
+                                                   compute_type,
+                                                   geam_ex_op));
 
-        CHECK_HIP_ERROR(hD_2.transfer_from(dD));
+            CHECK_HIP_ERROR(hD_2.transfer_from(dD));
+
+            if(arg.repeatability_check)
+            {
+                host_matrix<T> hD_copy(M, N, ldd);
+                CHECK_HIP_ERROR(hD_copy.memcheck());
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_matrix<T> dA_copy(A_row, A_col, lda);
+                    device_matrix<T> dB_copy(B_row, B_col, ldb);
+                    device_matrix<T> dC_copy(M, N, ldc);
+                    device_matrix<T> dD_copy(M, N, ldd);
+                    device_vector<T> d_alpha_copy(1);
+                    device_vector<T> d_beta_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dB_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dC_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dD_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_alpha_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_beta_copy.memcheck());
+
+                    CHECK_HIP_ERROR(d_alpha_copy.transfer_from(h_alpha));
+                    CHECK_HIP_ERROR(d_beta_copy.transfer_from(h_beta));
+                    CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                    CHECK_HIP_ERROR(dB_copy.transfer_from(hB));
+                    CHECK_HIP_ERROR(dC_copy.transfer_from(hC));
+                    CHECK_HIP_ERROR(dD_copy.transfer_from(hD_1));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+
+                        CHECK_ROCBLAS_ERROR(rocblas_geam_ex_fn(handle_copy,
+                                                               transA,
+                                                               transB,
+                                                               M,
+                                                               N,
+                                                               K,
+                                                               d_alpha_copy,
+                                                               dA_copy,
+                                                               a_type,
+                                                               lda,
+                                                               dB_copy,
+                                                               b_type,
+                                                               ldb,
+                                                               d_beta_copy,
+                                                               dC_copy,
+                                                               c_type,
+                                                               ldc,
+                                                               dD_copy,
+                                                               d_type,
+                                                               ldd,
+                                                               compute_type,
+                                                               geam_ex_op));
+                        CHECK_HIP_ERROR(hD_copy.transfer_from(dD_copy));
+                        unit_check_general<T>(M, N, ldd, hD_2, hD_copy);
+                    }
+                }
+                return;
+            }
+        }
 
         // reference calculation for golden result
         cpu_time_used = get_time_us_no_sync();
 
-        auto cblas_geam_ex_fn = geam_ex_op == rocblas_geam_ex_operation_min_plus
-                                    ? cblas_geam_min_plus<T>
-                                    : cblas_geam_plus_min<T>;
+        auto ref_geam_ex_fn = geam_ex_op == rocblas_geam_ex_operation_min_plus
+                                  ? ref_geam_min_plus<T>
+                                  : ref_geam_plus_min<T>;
 
-        cblas_geam_ex_fn(transA,
-                         transB,
-                         M,
-                         N,
-                         K,
-                         h_alpha[0],
-                         (T*)hA,
-                         lda,
-                         (T*)hB,
-                         ldb,
-                         h_beta[0],
-                         (T*)hC,
-                         ldc,
-                         (T*)hD_gold,
-                         ldd);
+        ref_geam_ex_fn(transA,
+                       transB,
+                       M,
+                       N,
+                       K,
+                       h_alpha[0],
+                       (T*)hA,
+                       lda,
+                       (T*)hB,
+                       ldb,
+                       h_beta[0],
+                       (T*)hC,
+                       ldc,
+                       (T*)hD_gold,
+                       ldd);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        if(arg.unit_check)
+        if(arg.pointer_mode_host)
         {
-            unit_check_general<T>(M, N, ldd, hD_gold, hD_1);
-            unit_check_general<T>(M, N, ldd, hD_gold, hD_2);
+            if(arg.unit_check)
+            {
+                unit_check_general<T>(M, N, ldd, hD_gold, hD_1);
+            }
+
+            if(arg.norm_check)
+            {
+                rocblas_error_1 = norm_check_general<T>('F', M, N, ldd, (T*)hD_gold, (T*)hD_1);
+            }
         }
 
-        if(arg.norm_check)
+        if(arg.pointer_mode_device)
         {
-            rocblas_error_1 = norm_check_general<T>('F', M, N, ldd, (T*)hD_gold, (T*)hD_1);
-            rocblas_error_2 = norm_check_general<T>('F', M, N, ldd, (T*)hD_gold, (T*)hD_2);
+            if(arg.unit_check)
+            {
+                unit_check_general<T>(M, N, ldd, hD_gold, hD_2);
+            }
+
+            if(arg.norm_check)
+            {
+                rocblas_error_2 = norm_check_general<T>('F', M, N, ldd, (T*)hD_gold, (T*)hD_2);
+            }
         }
 
         // inplace check for dD == dC
@@ -778,21 +873,21 @@ void testing_geam_ex(const Arguments& arg)
                     hipMemcpy(hD_1, dD_in_place, sizeof(T) * size_D, hipMemcpyDeviceToHost));
 
                 // reference calculation
-                cblas_geam_ex_fn(transA,
-                                 transB,
-                                 M,
-                                 N,
-                                 K,
-                                 h_alpha[0],
-                                 (T*)hA_copy,
-                                 lda,
-                                 (T*)hB_copy,
-                                 ldb,
-                                 h_beta[0],
-                                 (T*)hC,
-                                 ldc,
-                                 (T*)hD_gold,
-                                 ldd);
+                ref_geam_ex_fn(transA,
+                               transB,
+                               M,
+                               N,
+                               K,
+                               h_alpha[0],
+                               (T*)hA_copy,
+                               lda,
+                               (T*)hB_copy,
+                               ldb,
+                               h_beta[0],
+                               (T*)hC,
+                               ldc,
+                               (T*)hD_gold,
+                               ldd);
 
                 if(arg.unit_check)
                 {
