@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,9 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
+#include "frequency_monitor.hpp"
 #include "near.hpp"
 #include "norm.hpp"
 #include "rocblas.hpp"
@@ -37,7 +39,6 @@
 #include "testing_gemm_ex3.hpp"
 #include "type_dispatch.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 /* ============================================================================================ */
 template <typename TiA, typename TiB, typename To, typename Tc>
@@ -45,7 +46,7 @@ void testing_gemm_strided_batched_ex3_bad_arg(const Arguments& arg)
 {
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
-        auto rocblas_gemm_strided_batched_ex3_fn = arg.api == FORTRAN
+        auto rocblas_gemm_strided_batched_ex3_fn = arg.api & c_API_FORTRAN
                                                        ? rocblas_gemm_strided_batched_ex3_fortran
                                                        : rocblas_gemm_strided_batched_ex3;
 
@@ -921,7 +922,7 @@ void testing_gemm_strided_batched_ex3_bad_arg(const Arguments& arg)
 template <typename TiA, typename TiB, typename To, typename Tc>
 void testing_gemm_strided_batched_ex3(const Arguments& arg)
 {
-    auto rocblas_gemm_strided_batched_ex3_fn = arg.api == FORTRAN
+    auto rocblas_gemm_strided_batched_ex3_fn = arg.api & c_API_FORTRAN
                                                    ? rocblas_gemm_strided_batched_ex3_fortran
                                                    : rocblas_gemm_strided_batched_ex3;
 
@@ -1383,6 +1384,8 @@ void testing_gemm_strided_batched_ex3(const Arguments& arg)
         int         number_hot_calls = arg.iters;
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
+        FrequencyMonitor& freq_monitor = getFrequencyMonitor();
+        freq_monitor.start();
         gpu_time_used = get_time_us_sync(stream); // in microseconds
         for(int i = 0; i < number_hot_calls; i++)
         {
@@ -1417,6 +1420,7 @@ void testing_gemm_strided_batched_ex3(const Arguments& arg)
                                                 flags);
         }
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
+        freq_monitor.stop();
 
         ArgumentModel<e_transA,
                       e_transB,
