@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,11 +24,11 @@
 
 #define ROCBLAS_BETA_FEATURES_API
 #include "../../library/src/include/handle.hpp"
+#include "client_utility.hpp"
 #include "rocblas.hpp"
 #include "rocblas_matrix.hpp"
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
-#include "utility.hpp"
 
 template <typename Ti, typename To, typename Tc>
 void testing_gemm_strided_batched_ex_get_solutions(const Arguments& arg)
