@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "norm.hpp"
 #include "rocblas.hpp"
@@ -34,7 +35,6 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 #define ERROR_EPS_MULTIPLIER 40
 #define RESIDUAL_EPS_MULTIPLIER 40
@@ -56,7 +56,7 @@ void printMatrix(const char* name, T* A, rocblas_int m, rocblas_int n, rocblas_i
 template <typename T>
 void testing_trsm_ex_bad_arg(const Arguments& arg)
 {
-    auto rocblas_trsm_ex_fn = arg.api == FORTRAN ? rocblas_trsm_ex_fortran : rocblas_trsm_ex;
+    auto rocblas_trsm_ex_fn = arg.api & c_API_FORTRAN ? rocblas_trsm_ex_fortran : rocblas_trsm_ex;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -249,7 +249,7 @@ void testing_trsm_ex_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_trsm_ex(const Arguments& arg)
 {
-    auto rocblas_trsm_ex_fn = arg.api == FORTRAN ? rocblas_trsm_ex_fortran : rocblas_trsm_ex;
+    auto rocblas_trsm_ex_fn = arg.api & c_API_FORTRAN ? rocblas_trsm_ex_fortran : rocblas_trsm_ex;
 
     rocblas_int M   = arg.M;
     rocblas_int N   = arg.N;
@@ -307,7 +307,6 @@ void testing_trsm_ex(const Arguments& arg)
     host_matrix<T> hXorB_2(M, N, ldb);
     host_matrix<T> cpuXorB(M, N, ldb);
     host_matrix<T> invATemp1(TRSM_BLOCK, TRSM_BLOCK, K);
-    host_matrix<T> invATemp2(TRSM_BLOCK, TRSM_BLOCK, K);
     host_matrix<T> hinvAI(TRSM_BLOCK, TRSM_BLOCK, K);
 
     // Allocate device memory
@@ -339,7 +338,7 @@ void testing_trsm_ex(const Arguments& arg)
     }
 
     // Calculate hB = hA*hX;
-    cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hB, ldb);
+    ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hB, ldb);
 
     hXorB_1 = hB; // hXorB <- B
     hXorB_2 = hB; // hXorB <- B
@@ -426,8 +425,6 @@ void testing_trsm_ex(const Arguments& arg)
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
         CHECK_HIP_ERROR(dXorB.transfer_from(hXorB_1));
 
-        hipStream_t rocblas_stream;
-        CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &rocblas_stream));
         handle.pre_test(arg);
         if(blocks > 0)
             CHECK_ROCBLAS_ERROR(rocblas_trtri_strided_batched<T>(handle,
@@ -500,6 +497,71 @@ void testing_trsm_ex(const Arguments& arg)
 
         CHECK_HIP_ERROR(hXorB_2.transfer_from(dXorB));
 
+        if(arg.repeatability_check)
+        {
+            host_matrix<T> hXorB_copy(M, N, ldb);
+            CHECK_HIP_ERROR(hXorB_copy.memcheck());
+            CHECK_HIP_ERROR(invATemp1.transfer_from(dinvA));
+
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                //Allocate device memory in new device
+                device_matrix<T> dA_copy(K, K, lda);
+                device_matrix<T> dXorB_copy(M, N, ldb);
+                device_matrix<T> dinvA_copy(TRSM_BLOCK, TRSM_BLOCK, K);
+                device_vector<T> alpha_d_copy(1);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dXorB_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dinvA_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(alpha_d_copy.memcheck());
+
+                CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                CHECK_HIP_ERROR(dinvA_copy.transfer_from(invATemp1));
+
+                CHECK_HIP_ERROR(
+                    hipMemcpy(alpha_d_copy, &alpha_h, sizeof(T), hipMemcpyHostToDevice));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dXorB_copy.transfer_from(hB));
+                    CHECK_ROCBLAS_ERROR(rocblas_trsm_ex_fn(handle_copy,
+                                                           side,
+                                                           uplo,
+                                                           transA,
+                                                           diag,
+                                                           M,
+                                                           N,
+                                                           alpha_d_copy,
+                                                           dA_copy,
+                                                           lda,
+                                                           dXorB_copy,
+                                                           ldb,
+                                                           dinvA_copy,
+                                                           TRSM_BLOCK * K,
+                                                           arg.compute_type));
+
+                    CHECK_HIP_ERROR(hXorB_copy.transfer_from(dXorB_copy));
+                    unit_check_general<T>(M, N, ldb, hXorB_2, hXorB_copy);
+                }
+            }
+            return;
+        }
+
         //computed result is in hx_or_b, so forward error is E = hx - hx_or_b
         // calculate vector-induced-norm 1 of matrix E
         max_err_1 = rocblas_abs(matrix_norm_1<T>(M, N, ldb, hX, hXorB_1));
@@ -510,8 +572,8 @@ void testing_trsm_ex(const Arguments& arg)
         trsm_err_res_check<T>(max_err_2, M, error_eps_multiplier, eps);
 
         // hx_or_b contains A * (calculated X), so res = A * (calculated x) - b = hx_or_b - hb
-        cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
-        cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_2, ldb);
+        ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
+        ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_2, ldb);
 
         max_err_1 = rocblas_abs(matrix_norm_1<T>(M, N, ldb, hXorB_1, hB));
         max_err_2 = rocblas_abs(matrix_norm_1<T>(M, N, ldb, hXorB_2, hB));
@@ -540,7 +602,7 @@ void testing_trsm_ex(const Arguments& arg)
         // CPU cblas
         cpu_time_used = get_time_us_no_sync();
 
-        cblas_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA, lda, cpuXorB, ldb);
+        ref_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA, lda, cpuXorB, ldb);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
