@@ -558,9 +558,6 @@ inline void lapack_xsyr2(rocblas_fill uplo,
     }
 }
 
-template <typename T, typename U>
-void cblas_scal(int64_t n, T alpha, U x, int64_t incx);
-
 // cblas doesn't have trti2 implementation for now so using the lapack trti2 implementation below
 template <typename T>
 void lapack_xtrti2(char uplo, char diag, int64_t n, T* A, int64_t lda)
@@ -584,15 +581,15 @@ void lapack_xtrti2(char uplo, char diag, int64_t n, T* A, int64_t lda)
                 AJJ = T(-1.0);
             }
             //Compute elements 0:j-1 of j-th column.
-            cblas_trmv(rocblas_fill_upper,
-                       rocblas_operation_none,
-                       char2rocblas_diagonal(diag),
-                       j,
-                       A,
-                       lda,
-                       A + (j * lda),
-                       1);
-            cblas_scal(j, AJJ, A + (j * lda), 1);
+            ref_trmv(rocblas_fill_upper,
+                     rocblas_operation_none,
+                     char2rocblas_diagonal(diag),
+                     j,
+                     A,
+                     lda,
+                     A + (j * lda),
+                     1);
+            ref_scal(j, AJJ, A + (j * lda), 1);
         }
     }
     else
@@ -611,15 +608,15 @@ void lapack_xtrti2(char uplo, char diag, int64_t n, T* A, int64_t lda)
             if(j < n - 1)
             {
                 //Compute elements 0:j-1 of j-th column.
-                cblas_trmv(rocblas_fill_lower,
-                           rocblas_operation_none,
-                           char2rocblas_diagonal(diag),
-                           n - j - 1,
-                           A + ((j + 1) + (j + 1) * lda),
-                           lda,
-                           A + ((j + 1) + j * lda),
-                           1);
-                cblas_scal(n - j - 1, AJJ, A + ((j + 1) + j * lda), 1);
+                ref_trmv(rocblas_fill_lower,
+                         rocblas_operation_none,
+                         char2rocblas_diagonal(diag),
+                         n - j - 1,
+                         A + ((j + 1) + (j + 1) * lda),
+                         lda,
+                         A + ((j + 1) + j * lda),
+                         1);
+                ref_scal(n - j - 1, AJJ, A + ((j + 1) + j * lda), 1);
             }
         }
     }
@@ -648,28 +645,28 @@ void lapack_xtrtri(char uplo, char diag, int64_t n, T* A, int64_t lda)
                 JB = std::min(NB, n - j);
 
                 // Compute rows 0:j-1 of current block column
-                cblas_trmm(rocblas_side_left,
-                           rocblas_fill_upper,
-                           rocblas_operation_none,
-                           char2rocblas_diagonal(diag),
-                           j,
-                           JB,
-                           T(1.0),
-                           A,
-                           lda,
-                           A + (j * lda),
-                           lda);
-                cblas_trsm(rocblas_side_right,
-                           rocblas_fill_upper,
-                           rocblas_operation_none,
-                           char2rocblas_diagonal(diag),
-                           j,
-                           JB,
-                           T(-1.0),
-                           A + (j + j * lda),
-                           lda,
-                           A + j * lda,
-                           lda);
+                ref_trmm(rocblas_side_left,
+                         rocblas_fill_upper,
+                         rocblas_operation_none,
+                         char2rocblas_diagonal(diag),
+                         j,
+                         JB,
+                         T(1.0),
+                         A,
+                         lda,
+                         A + (j * lda),
+                         lda);
+                ref_trsm(rocblas_side_right,
+                         rocblas_fill_upper,
+                         rocblas_operation_none,
+                         char2rocblas_diagonal(diag),
+                         j,
+                         JB,
+                         T(-1.0),
+                         A + (j + j * lda),
+                         lda,
+                         A + j * lda,
+                         lda);
                 lapack_xtrti2(uplo, diag, JB, A + (j + j * lda), lda);
             }
         }
@@ -681,28 +678,28 @@ void lapack_xtrtri(char uplo, char diag, int64_t n, T* A, int64_t lda)
                 JB = std::min(NB, n - j);
                 if(j + JB <= n)
                 {
-                    cblas_trmm(rocblas_side_left,
-                               rocblas_fill_lower,
-                               rocblas_operation_none,
-                               char2rocblas_diagonal(diag),
-                               n - j - JB,
-                               JB,
-                               T(1.0),
-                               A + ((j + JB) + (j + JB) * lda),
-                               lda,
-                               A + ((j + JB) + j * lda),
-                               lda);
-                    cblas_trsm(rocblas_side_right,
-                               rocblas_fill_lower,
-                               rocblas_operation_none,
-                               char2rocblas_diagonal(diag),
-                               n - j - JB,
-                               JB,
-                               T(-1.0),
-                               A + (j + j * lda),
-                               lda,
-                               A + ((j + JB) + j * lda),
-                               lda);
+                    ref_trmm(rocblas_side_left,
+                             rocblas_fill_lower,
+                             rocblas_operation_none,
+                             char2rocblas_diagonal(diag),
+                             n - j - JB,
+                             JB,
+                             T(1.0),
+                             A + ((j + JB) + (j + JB) * lda),
+                             lda,
+                             A + ((j + JB) + j * lda),
+                             lda);
+                    ref_trsm(rocblas_side_right,
+                             rocblas_fill_lower,
+                             rocblas_operation_none,
+                             char2rocblas_diagonal(diag),
+                             n - j - JB,
+                             JB,
+                             T(-1.0),
+                             A + (j + j * lda),
+                             lda,
+                             A + ((j + JB) + j * lda),
+                             lda);
                 }
                 lapack_xtrti2(uplo, diag, JB, A + (j + j * lda), lda);
             }
