@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,6 +40,13 @@
 // sqrt(0.5) factor for complex cutoff calculations
 constexpr double sqrthalf = 0.7071067811865475244;
 
+template <class T>
+inline bool reduction_requires_near(const Arguments& arg, int64_t n)
+{
+    return arg.initialization == rocblas_initialization::hpl
+           || (std::is_same_v<T, rocblas_half> && n > 10000);
+}
+
 // Sum error tolerance for large sums. Multiplied by the number of items
 // in the sum to get an expected absolute error bound.
 
@@ -101,6 +108,18 @@ ROCBLAS_CLANG_STATIC constexpr double
                                   rocblas_double_complex,
                                   rocblas_double_complex> = 1 / 1000000.0;
 
+template <typename T>
+double sum_near_tolerance(int64_t n, real_t<T> sum)
+{
+    double count     = (n == 1 || n > 4) ? sqrt(n) : 2.0;
+    double tolerance = sum_error_tolerance<T> * 2.0 * count;
+    if(sum != 0)
+    {
+        tolerance *= std::abs(sum);
+    }
+    return tolerance;
+}
+
 #ifndef GOOGLE_TEST
 #define NEAR_CHECK(M, N, lda, strideA, hCPU, hGPU, batch_count, err, NEAR_ASSERT)
 #define NEAR_CHECK_B(M, N, lda, hCPU, hGPU, batch_count, err, NEAR_ASSERT)
