@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -136,8 +136,6 @@ struct Arguments
 
     rocblas_initialization initialization;
 
-    rocblas_arithmetic_check arithmetic_check;
-
     rocblas_atomics_mode atomics_mode;
 
     rocblas_client_os os_flags;
@@ -153,8 +151,9 @@ struct Arguments
 
     uint32_t math_mode;
 
-    // number of bytes of cache to flush for context sensitive benchmarking
-    uint64_t flush_malloc_size;
+    // number of copies of arrays to allocate for context sensitive benchmarking
+    uint64_t flush_batch_count;
+    uint64_t flush_memory_size;
 
     // 16 bit
     uint16_t threads;
@@ -181,6 +180,9 @@ struct Arguments
     bool outofplace;
     bool HMM; // xnack+
     bool graph_test;
+    bool repeatability_check;
+
+    int use_hipblaslt;
 
     /*************************************************************************
      *                     End Of Arguments                                  *
@@ -237,16 +239,16 @@ struct Arguments
     OPER(c_type) SEP                 \
     OPER(d_type) SEP                 \
     OPER(compute_type) SEP           \
-    OPER(composite_compute_type) SEP       \
+    OPER(composite_compute_type) SEP \
     OPER(initialization) SEP         \
-    OPER(arithmetic_check) SEP       \
     OPER(atomics_mode) SEP           \
     OPER(os_flags) SEP               \
     OPER(gpu_arch) SEP               \
     OPER(api) SEP                    \
     OPER(pad) SEP                    \
     OPER(math_mode) SEP              \
-    OPER(flush_malloc_size) SEP             \
+    OPER(flush_batch_count) SEP             \
+    OPER(flush_memory_size) SEP             \
     OPER(threads) SEP                \
     OPER(streams) SEP                \
     OPER(devices) SEP                \
@@ -265,8 +267,9 @@ struct Arguments
     OPER(c_noalias_d) SEP            \
     OPER(outofplace) SEP             \
     OPER(HMM) SEP                    \
-    OPER(graph_test)
-
+    OPER(graph_test) SEP             \
+    OPER(repeatability_check) SEP    \
+    OPER(use_hipblaslt)
     // clang-format on
 
     // Validate input format.
