@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -53,6 +53,8 @@ typedef long long ssize_t; /* x64 only supported */
 
 #ifdef GOOGLE_TEST
 
+#include "gtest_helpers.hpp"
+
 // Extra macro so that macro arguments get expanded before calling Google Test
 #define CHECK_HIP_ERROR2(ERROR) ASSERT_EQ(ERROR, hipSuccess)
 #define CHECK_HIP_ERROR(ERROR) CHECK_HIP_ERROR2(ERROR)
@@ -75,22 +77,22 @@ typedef long long ssize_t; /* x64 only supported */
 // This wraps the rocBLAS call with catch_signals_and_exceptions_as_failures().
 // By placing it at the rocBLAS call site, memory resources are less likely to
 // be leaked in the event of a caught signal.
-#define EXPECT_ROCBLAS_STATUS(STATUS, EXPECT)                 \
-    do                                                        \
-    {                                                         \
-        volatile bool signal_or_exception = true;             \
-        /* Use status__ in case STATUS contains "status" */   \
-        rocblas_status status__;                              \
-        catch_signals_and_exceptions_as_failures([&] {        \
-            status__            = (STATUS);                   \
-            signal_or_exception = false;                      \
-        });                                                   \
-        if(signal_or_exception)                               \
-            return;                                           \
-        { /* localize status for ASSERT_EQ message */         \
-            rocblas_status status_ = status__;                \
-            ASSERT_EQ(status_, EXPECT); /* prints "status" */ \
-        }                                                     \
+#define EXPECT_ROCBLAS_STATUS(STATUS, EXPECT)               \
+    do                                                      \
+    {                                                       \
+        volatile bool signal_or_exception = true;           \
+        /* Use status__ in case STATUS contains "status" */ \
+        rocblas_status status__;                            \
+        catch_signals_and_exceptions_as_failures([&] {      \
+            status__            = (STATUS);                 \
+            signal_or_exception = false;                    \
+        });                                                 \
+        if(signal_or_exception)                             \
+            return;                                         \
+        { /* localize status for ASSERT_EQ message */       \
+            rocblas_status status_ = status__;              \
+            ASSERT_TRUE(status_match(EXPECT, status_));     \
+        }                                                   \
     } while(0)
 
 #define CHECK_ALLOC_QUERY(STATUS)                                  \
@@ -192,9 +194,6 @@ bool match_test_category(const Arguments& arg, const char* category);
 // INSTANTIATE_TEST_CATEGORY(testclass, HMM)         \
 // INSTANTIATE_TEST_CATEGORY(testclass, known_bug)
 
-// Function to catch signals and exceptions as failures
-void catch_signals_and_exceptions_as_failures(std::function<void()> test, bool set_alarm = false);
-
 // Macro to call catch_signals_and_exceptions_as_failures() with a lambda expression
 #define CATCH_SIGNALS_AND_EXCEPTIONS_AS_FAILURES(test) \
     catch_signals_and_exceptions_as_failures([&] { test; }, true)
@@ -293,8 +292,6 @@ public:
 extern stream_pool g_stream_pool;
 extern thread_pool g_thread_pool;
 
-extern thread_local std::unique_ptr<std::function<void(rocblas_handle)>> t_set_stream_callback;
-
 /* ============================================================================================ */
 /*! \brief  Normalized test name to conform to Google Tests */
 // The template parameter is only used to generate multiple instantiations with distinct static local variables
