@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,27 +22,22 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_set_get_matrix_async(const Arguments& arg)
 {
-    rocblas_int          rows = arg.M;
-    rocblas_int          cols = arg.N;
-    rocblas_int          lda  = arg.lda;
-    rocblas_int          ldb  = arg.ldb;
-    rocblas_int          ldc  = arg.ldc;
+    auto rocblas_set_matrix_async_fn    = rocblas_set_matrix_async;
+    auto rocblas_set_matrix_async_fn_64 = rocblas_set_matrix_async_64;
+    auto rocblas_get_matrix_async_fn    = rocblas_get_matrix_async;
+    auto rocblas_get_matrix_async_fn_64 = rocblas_get_matrix_async_64;
+
+    int64_t rows = arg.M;
+    int64_t cols = arg.N;
+    int64_t lda  = arg.lda;
+    int64_t ldb  = arg.ldb;
+    int64_t ldd  = arg.ldd;
+
     rocblas_local_handle handle{arg};
 
     hipStream_t stream;
@@ -50,55 +45,57 @@ void testing_set_get_matrix_async(const Arguments& arg)
 
     // argument sanity check, quick return if input parameters are invalid before allocating invalid
     // memory
-    bool invalidGPUMatrix = rows < 0 || cols < 0 || ldc <= 0 || ldc < rows;
+    bool invalidGPUMatrix = rows < 0 || cols < 0 || ldd <= 0 || ldd < rows;
     bool invalidSet       = invalidGPUMatrix || lda <= 0 || lda < rows;
     bool invalidGet       = invalidGPUMatrix || ldb <= 0 || ldb < rows;
 
     if(invalidSet || invalidGet)
     {
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_set_matrix_async(rows, cols, sizeof(T), nullptr, lda, nullptr, ldc, stream),
-            invalidSet ? rocblas_status_invalid_size : rocblas_status_invalid_pointer);
+        DAPI_EXPECT(invalidSet ? rocblas_status_invalid_size : rocblas_status_invalid_pointer,
+                    rocblas_set_matrix_async,
+                    (rows, cols, sizeof(T), nullptr, lda, nullptr, ldd, stream));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_get_matrix_async(rows, cols, sizeof(T), nullptr, ldc, nullptr, ldb, stream),
-            invalidGet ? rocblas_status_invalid_size : rocblas_status_invalid_pointer);
+        DAPI_EXPECT(invalidGet ? rocblas_status_invalid_size : rocblas_status_invalid_pointer,
+                    rocblas_get_matrix_async,
+                    (rows, cols, sizeof(T), nullptr, ldd, nullptr, ldb, stream));
 
         return;
     }
 
     // Naming: dK is in GPU (device) memory. hK is in CPU (host) memory,
-    host_pinned_vector<T> ha(cols * size_t(lda)); // using vector layout to reuse pinned_vector
-    host_pinned_vector<T> hb(cols * size_t(ldb));
-    host_vector<T>        hb_gold(cols * size_t(ldb));
+    host_pinned_vector<T> hA(cols * size_t(lda)); // using vector layout to reuse pinned_vector
+    host_pinned_vector<T> hB(cols * size_t(ldb));
+    host_vector<T>        hB_gold(cols * size_t(ldb));
 
-    double gpu_time_used, cpu_time_used;
+    CHECK_HIP_ERROR(hA.memcheck());
+    CHECK_HIP_ERROR(hB.memcheck());
+    CHECK_HIP_ERROR(hB_gold.memcheck());
+
+    double cpu_time_used;
     double rocblas_error = 0.0;
 
     // allocate memory on device
-    device_vector<T> dc(cols * size_t(ldc)); // vector layout
-    CHECK_DEVICE_ALLOCATION(dc.memcheck());
+    device_vector<T> dD(cols * size_t(ldd)); // vector layout
+    CHECK_DEVICE_ALLOCATION(dD.memcheck());
 
     // Initial Data on CPU
     rocblas_seedrand();
-    rocblas_init<T>(ha, rows, cols, lda);
-    rocblas_init<T>(hb, rows, cols, ldb);
+    rocblas_init<T>(hA, rows, cols, lda);
+    rocblas_init<T>(hB, rows, cols, ldb);
 
     if(arg.unit_check || arg.norm_check)
     {
-        CHECK_HIP_ERROR(hipMemset(dc, 0, sizeof(T) * cols * ldc)); // vector layout
+        CHECK_HIP_ERROR(hipMemset(dD, 0, sizeof(T) * cols * ldd)); // vector layout
 
-        CHECK_ROCBLAS_ERROR(
-            rocblas_set_matrix_async(rows, cols, sizeof(T), ha, lda, dc, ldc, stream));
-        CHECK_ROCBLAS_ERROR(
-            rocblas_get_matrix_async(rows, cols, sizeof(T), dc, ldc, hb, ldb, stream));
+        DAPI_CHECK(rocblas_set_matrix_async_fn, (rows, cols, sizeof(T), hA, lda, dD, ldd, stream));
+        DAPI_CHECK(rocblas_get_matrix_async_fn, (rows, cols, sizeof(T), dD, ldd, hB, ldb, stream));
 
         // reference calculation
         cpu_time_used = get_time_us_no_sync();
 
         for(size_t i2 = 0; i2 < cols; i2++)
             for(size_t i1 = 0; i1 < rows; i1++)
-                hb_gold[i1 + i2 * ldb] = ha[i1 + i2 * lda];
+                hB_gold[i1 + i2 * ldb] = hA[i1 + i2 * lda];
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -106,40 +103,39 @@ void testing_set_get_matrix_async(const Arguments& arg)
 
         if(arg.unit_check)
         {
-            unit_check_general<T>(rows, cols, ldb, hb, hb_gold);
+            unit_check_general<T>(rows, cols, ldb, hB, hB_gold);
         }
 
         if(arg.norm_check)
         {
-            rocblas_error = norm_check_general<T>('F', rows, cols, ldb, hb, hb_gold);
+            rocblas_error = norm_check_general<T>('F', rows, cols, ldb, hB, hB_gold);
         }
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
 
-        for(int iter = 0; iter < number_cold_calls; iter++)
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            rocblas_set_matrix_async(rows, cols, sizeof(T), ha, lda, dc, ldc, stream);
-            rocblas_get_matrix_async(rows, cols, sizeof(T), dc, ldc, hb, ldb, stream);
-        }
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
 
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
-
-        for(int iter = 0; iter < number_hot_calls; iter++)
-        {
-            rocblas_set_matrix_async(rows, cols, sizeof(T), ha, lda, dc, ldc, stream);
-            rocblas_get_matrix_async(rows, cols, sizeof(T), dc, ldc, hb, ldb, stream);
+            DAPI_DISPATCH(rocblas_set_matrix_async,
+                          (rows, cols, sizeof(T), hA, lda, dD, ldd, stream));
+            DAPI_DISPATCH(rocblas_get_matrix_async,
+                          (rows, cols, sizeof(T), dD, ldd, hB, ldb, stream));
         }
 
+        hipStreamSynchronize(stream);
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
 
-        ArgumentModel<e_M, e_N, e_lda, e_ldb, e_ldc>{}.log_args<T>(
+        ArgumentModel<e_M, e_N, e_lda, e_ldb, e_ldd>{}.log_args<T>(
             rocblas_cout,
             arg,
             gpu_time_used,
