@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -56,7 +56,7 @@
 // uses recursive folding reduction
 #include "../blas1/reduction.hpp"
 
-template <rocblas_int NB, typename Tex, typename To>
+template <int NB, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemv_scal_kernel_calc(
     rocblas_int n, Tex beta, rocblas_stride stride_beta, To* y, rocblas_int incy)
 {
@@ -76,7 +76,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemv_scal_kernel_calc(
     }
 }
 
-template <rocblas_int NB, typename Tex, typename To>
+template <int NB, typename Tex, typename To>
 ROCBLAS_KERNEL(NB)
 rocblas_gemv_scal_kernel(rocblas_int    n,
                          Tex            beta_device_host,
@@ -93,6 +93,7 @@ rocblas_gemv_scal_kernel(rocblas_int    n,
     rocblas_gemv_scal_kernel_calc<NB>(n, beta, stride_beta, y, incy);
 }
 
+//
 template <int DIM_X, int DIM_Y, int elements_per_thread, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemvn_double_buffered_kernel_calc(rocblas_int rows,
                                                                   rocblas_int cols,
@@ -291,9 +292,9 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_double_buffered_kernel_calc(rocblas_int ro
     }
 }
 
-template <rocblas_int DIM_X,
-          rocblas_int DIM_Y,
-          typename T_index,
+template <int DIM_X,
+          int DIM_Y,
+          typename T_Index,
           typename Ti,
           typename Tex,
           typename To,
@@ -302,14 +303,14 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int m,
                                                   rocblas_int n,
                                                   Tex         alpha,
                                                   const Ti*   A,
-                                                  rocblas_int lda,
+                                                  T_Index     lda,
                                                   const Ti*   x,
-                                                  rocblas_int incx,
+                                                  T_Index     incx,
                                                   Tex         beta,
                                                   To*         y,
-                                                  rocblas_int incy)
+                                                  T_Index     incy)
 {
-    rocblas_int thread_id = threadIdx.x + threadIdx.y * blockDim.x;
+    rocblas_int thread_id = threadIdx.x + threadIdx.y * DIM_X;
 
     if(!alpha)
     {
@@ -317,7 +318,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int m,
         {
             int64_t ind = blockIdx.x * DIM_X * 4 + thread_id;
             if(ind < m)
-                y[ind * T_index(incy)] = beta ? (To)(beta * y[ind * T_index(incy)]) : (To)0;
+                y[ind * T_Index(incy)] = beta ? (To)(beta * y[ind * T_Index(incy)]) : (To)0;
         }
         return;
     }
@@ -342,38 +343,38 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int m,
 
     for(col = ty * 4; col < (n - n_tail); col += 4 * DIM_Y)
     {
-        res_x[0] = x[(col + 0) * T_index(incx)];
-        res_x[1] = x[(col + 1) * T_index(incx)];
-        res_x[2] = x[(col + 2) * T_index(incx)];
-        res_x[3] = x[(col + 3) * T_index(incx)];
+        res_x[0] = x[(col + 0) * T_Index(incx)];
+        res_x[1] = x[(col + 1) * T_Index(incx)];
+        res_x[2] = x[(col + 2) * T_Index(incx)];
+        res_x[3] = x[(col + 3) * T_Index(incx)];
 
         if(ind < m)
         {
-            res_A[0] += A[ind + (col + 0) * T_index(lda)] * res_x[0];
-            res_A[0] += A[ind + (col + 1) * T_index(lda)] * res_x[1];
-            res_A[0] += A[ind + (col + 2) * T_index(lda)] * res_x[2];
-            res_A[0] += A[ind + (col + 3) * T_index(lda)] * res_x[3];
+            res_A[0] += A[ind + (col + 0) * T_Index(lda)] * res_x[0];
+            res_A[0] += A[ind + (col + 1) * T_Index(lda)] * res_x[1];
+            res_A[0] += A[ind + (col + 2) * T_Index(lda)] * res_x[2];
+            res_A[0] += A[ind + (col + 3) * T_Index(lda)] * res_x[3];
 
             if(ind + DIM_X < m)
             {
-                res_A[1] += A[ind + DIM_X + (col + 0) * T_index(lda)] * res_x[0];
-                res_A[1] += A[ind + DIM_X + (col + 1) * T_index(lda)] * res_x[1];
-                res_A[1] += A[ind + DIM_X + (col + 2) * T_index(lda)] * res_x[2];
-                res_A[1] += A[ind + DIM_X + (col + 3) * T_index(lda)] * res_x[3];
+                res_A[1] += A[ind + DIM_X + (col + 0) * T_Index(lda)] * res_x[0];
+                res_A[1] += A[ind + DIM_X + (col + 1) * T_Index(lda)] * res_x[1];
+                res_A[1] += A[ind + DIM_X + (col + 2) * T_Index(lda)] * res_x[2];
+                res_A[1] += A[ind + DIM_X + (col + 3) * T_Index(lda)] * res_x[3];
 
                 if(ind + 2 * DIM_X < m)
                 {
-                    res_A[2] += A[ind + 2 * DIM_X + (col + 0) * T_index(lda)] * res_x[0];
-                    res_A[2] += A[ind + 2 * DIM_X + (col + 1) * T_index(lda)] * res_x[1];
-                    res_A[2] += A[ind + 2 * DIM_X + (col + 2) * T_index(lda)] * res_x[2];
-                    res_A[2] += A[ind + 2 * DIM_X + (col + 3) * T_index(lda)] * res_x[3];
+                    res_A[2] += A[ind + 2 * DIM_X + (col + 0) * T_Index(lda)] * res_x[0];
+                    res_A[2] += A[ind + 2 * DIM_X + (col + 1) * T_Index(lda)] * res_x[1];
+                    res_A[2] += A[ind + 2 * DIM_X + (col + 2) * T_Index(lda)] * res_x[2];
+                    res_A[2] += A[ind + 2 * DIM_X + (col + 3) * T_Index(lda)] * res_x[3];
 
                     if(ind + 3 * DIM_X < m)
                     {
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 0) * T_index(lda)] * res_x[0];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 1) * T_index(lda)] * res_x[1];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 2) * T_index(lda)] * res_x[2];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 3) * T_index(lda)] * res_x[3];
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 0) * T_Index(lda)] * res_x[0];
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 1) * T_Index(lda)] * res_x[1];
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 2) * T_Index(lda)] * res_x[2];
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 3) * T_Index(lda)] * res_x[3];
                     }
                 }
             }
@@ -387,56 +388,56 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int m,
 
         if(col + 0 < n)
         {
-            res_x[0] = x[(col + 0) * T_index(incx)];
+            res_x[0] = x[(col + 0) * T_Index(incx)];
 
             if(col + 1 < n)
             {
-                res_x[1] = x[(col + 1) * T_index(incx)];
+                res_x[1] = x[(col + 1) * T_Index(incx)];
 
                 if(col + 2 < n)
                 {
-                    res_x[2] = x[(col + 2) * T_index(incx)];
+                    res_x[2] = x[(col + 2) * T_Index(incx)];
 
                     if(col + 3 < n)
-                        res_x[3] = x[(col + 3) * T_index(incx)];
+                        res_x[3] = x[(col + 3) * T_Index(incx)];
                 }
             }
         }
 
         if(ind < m)
         {
-            res_A[0] += A[ind + (col + 0) * T_index(lda) * (col + 0 < n)] * res_x[0];
-            res_A[0] += A[ind + (col + 1) * T_index(lda) * (col + 1 < n)] * res_x[1];
-            res_A[0] += A[ind + (col + 2) * T_index(lda) * (col + 2 < n)] * res_x[2];
-            res_A[0] += A[ind + (col + 3) * T_index(lda) * (col + 3 < n)] * res_x[3];
+            res_A[0] += A[ind + (col + 0) * T_Index(lda) * (col + 0 < n)] * res_x[0];
+            res_A[0] += A[ind + (col + 1) * T_Index(lda) * (col + 1 < n)] * res_x[1];
+            res_A[0] += A[ind + (col + 2) * T_Index(lda) * (col + 2 < n)] * res_x[2];
+            res_A[0] += A[ind + (col + 3) * T_Index(lda) * (col + 3 < n)] * res_x[3];
 
             if(ind + DIM_X < m)
             {
-                res_A[1] += A[ind + DIM_X + (col + 0) * T_index(lda) * (col + 0 < n)] * res_x[0];
-                res_A[1] += A[ind + DIM_X + (col + 1) * T_index(lda) * (col + 1 < n)] * res_x[1];
-                res_A[1] += A[ind + DIM_X + (col + 2) * T_index(lda) * (col + 2 < n)] * res_x[2];
-                res_A[1] += A[ind + DIM_X + (col + 3) * T_index(lda) * (col + 3 < n)] * res_x[3];
+                res_A[1] += A[ind + DIM_X + (col + 0) * T_Index(lda) * (col + 0 < n)] * res_x[0];
+                res_A[1] += A[ind + DIM_X + (col + 1) * T_Index(lda) * (col + 1 < n)] * res_x[1];
+                res_A[1] += A[ind + DIM_X + (col + 2) * T_Index(lda) * (col + 2 < n)] * res_x[2];
+                res_A[1] += A[ind + DIM_X + (col + 3) * T_Index(lda) * (col + 3 < n)] * res_x[3];
 
                 if(ind + 2 * DIM_X < m)
                 {
                     res_A[2]
-                        += A[ind + 2 * DIM_X + (col + 0) * T_index(lda) * (col + 0 < n)] * res_x[0];
+                        += A[ind + 2 * DIM_X + (col + 0) * T_Index(lda) * (col + 0 < n)] * res_x[0];
                     res_A[2]
-                        += A[ind + 2 * DIM_X + (col + 1) * T_index(lda) * (col + 1 < n)] * res_x[1];
+                        += A[ind + 2 * DIM_X + (col + 1) * T_Index(lda) * (col + 1 < n)] * res_x[1];
                     res_A[2]
-                        += A[ind + 2 * DIM_X + (col + 2) * T_index(lda) * (col + 2 < n)] * res_x[2];
+                        += A[ind + 2 * DIM_X + (col + 2) * T_Index(lda) * (col + 2 < n)] * res_x[2];
                     res_A[2]
-                        += A[ind + 2 * DIM_X + (col + 3) * T_index(lda) * (col + 3 < n)] * res_x[3];
+                        += A[ind + 2 * DIM_X + (col + 3) * T_Index(lda) * (col + 3 < n)] * res_x[3];
 
                     if(ind + 3 * DIM_X < m)
                     {
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 0) * T_index(lda) * (col + 0 < n)]
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 0) * T_Index(lda) * (col + 0 < n)]
                                     * res_x[0];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 1) * T_index(lda) * (col + 1 < n)]
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 1) * T_Index(lda) * (col + 1 < n)]
                                     * res_x[1];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 2) * T_index(lda) * (col + 2 < n)]
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 2) * T_Index(lda) * (col + 2 < n)]
                                     * res_x[2];
-                        res_A[3] += A[ind + 3 * DIM_X + (col + 3) * T_index(lda) * (col + 3 < n)]
+                        res_A[3] += A[ind + 3 * DIM_X + (col + 3) * T_Index(lda) * (col + 3 < n)]
                                     * res_x[3];
                     }
                 }
@@ -459,25 +460,25 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int m,
         ind = blockIdx.x * DIM_X * 4 + thread_id;
 
         if(ind < m)
-            y[ind * T_index(incy)]
-                = beta ? (To)(alpha * sdata[thread_id] + beta * y[ind * T_index(incy)])
+            y[ind * T_Index(incy)]
+                = beta ? (To)(alpha * sdata[thread_id] + beta * y[ind * T_Index(incy)])
                        : (To)(alpha * sdata[thread_id]);
     }
 }
 
 // Overload for double precision complex numbers. We run out of registers
 // if we use the above algorithm.
-template <rocblas_int DIM_X, rocblas_int DIM_Y, typename T_index, typename U>
+template <int DIM_X, int DIM_Y, typename T_Index, typename U>
 ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int                   m,
                                                   rocblas_int                   n,
                                                   U                             alpha,
                                                   const rocblas_double_complex* A,
-                                                  rocblas_int                   lda,
+                                                  T_Index                       lda,
                                                   const rocblas_double_complex* x,
-                                                  rocblas_int                   incx,
+                                                  T_Index                       incx,
                                                   U                             beta,
                                                   rocblas_double_complex*       y,
-                                                  rocblas_int                   incy)
+                                                  T_Index                       incy)
 {
     rocblas_int thread_id = threadIdx.x + threadIdx.y * blockDim.x;
 
@@ -487,7 +488,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int
         {
             int64_t ind = blockIdx.x * DIM_X + thread_id;
             if(ind < m)
-                y[ind * T_index(incy)] = beta ? beta * y[ind * T_index(incy)] : 0;
+                y[ind * T_Index(incy)] = beta ? beta * y[ind * T_Index(incy)] : 0;
         }
         return;
     }
@@ -513,7 +514,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int
 
         if(ind < m)
         {
-            res_A += A[ind + col * T_index(lda)] * x[col * T_index(incx)];
+            res_A += A[ind + col * T_Index(lda)] * x[col * T_Index(incx)];
         }
     }
 
@@ -521,13 +522,13 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int
     if(n_tail > 0)
     {
         if(col + 0 < n)
-            res_x = x[col * T_index(incx)];
+            res_x = x[col * T_Index(incx)];
         else
             res_x = rocblas_double_complex{0, 0};
 
         if(ind < m)
         {
-            res_A += A[ind + col * T_index(lda) * (col < n)] * res_x;
+            res_A += A[ind + col * T_Index(lda) * (col < n)] * res_x;
         }
     }
 
@@ -545,13 +546,13 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvn_kernel_calc(rocblas_int
 
         if(ind < m)
         {
-            y[ind * T_index(incy)] = beta ? alpha * sdata[thread_id] + beta * y[ind * T_index(incy)]
+            y[ind * T_Index(incy)] = beta ? alpha * sdata[thread_id] + beta * y[ind * T_Index(incy)]
                                           : alpha * sdata[thread_id];
         }
     }
 }
 
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemvt_kernel_calc(rocblas_int m,
                                                   rocblas_int n,
                                                   Tex         alpha,
@@ -620,17 +621,17 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_kernel_calc(rocblas_int m,
 }
 
 //Optimized kernel for GEMV transpose case when m or n is less than 6000
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename T_Index, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemvt_warp_reduce_kernel_calc(rocblas_int m,
                                                               rocblas_int n,
                                                               Tex         alpha,
                                                               const Ti* __restrict__ A,
-                                                              rocblas_int lda,
+                                                              T_Index lda,
                                                               const Ti* __restrict__ x,
-                                                              rocblas_int incx,
-                                                              Tex         beta,
+                                                              T_Index incx,
+                                                              Tex     beta,
                                                               To* __restrict__ y,
-                                                              rocblas_int incy)
+                                                              T_Index incy)
 {
     rocblas_int tx  = threadIdx.x;
     rocblas_int col = blockIdx.x;
@@ -638,7 +639,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_warp_reduce_kernel_calc(rocblas_int m,
     if(!alpha)
     {
         if(tx == 0)
-            y[col * int64_t(incy)] = beta ? (To)(beta * y[col * int64_t(incy)]) : (To)0;
+            y[col * (incy)] = beta ? (To)(beta * y[col * (incy)]) : (To)0;
         return;
     }
 
@@ -646,7 +647,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_warp_reduce_kernel_calc(rocblas_int m,
         A += tx;
 
     //Each BlockIdx.x takes care of each column of matrix A
-    A += col * size_t(lda);
+    A += col * (lda);
 
     Tex res = 0;
 
@@ -656,10 +657,10 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_warp_reduce_kernel_calc(rocblas_int m,
     //Each column of Matrix A is multiplied with vector x and the resultant value is stored in res.
     //If m > NB_X, then the threads are reused and the multiplied values will be accumalated.
     for(rocblas_int i = 0; tx + i < m_full; i += NB_X)
-        res += (CONJ ? conj(A[i]) : A[i]) * x[(tx + i) * int64_t(incx)];
+        res += (CONJ ? conj(A[i]) : A[i]) * x[(tx + i) * (incx)];
 
     if(tx + m_full < m)
-        res += (CONJ ? conj(A[m_full]) : A[m_full]) * x[(tx + m_full) * int64_t(incx)];
+        res += (CONJ ? conj(A[m_full]) : A[m_full]) * x[(tx + m_full) * (incx)];
 
     if(NB_X <= warpSize)
     {
@@ -675,19 +676,18 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_warp_reduce_kernel_calc(rocblas_int m,
     if(tx == 0)
     {
         // !alpha handled earlier by early return
-        y[col * int64_t(incy)]
-            = beta ? (To)(alpha * res + beta * y[col * int64_t(incy)]) : (To)(alpha * res);
+        y[col * (incy)] = beta ? (To)(alpha * res + beta * y[col * (incy)]) : (To)(alpha * res);
     }
 }
 
-template <bool CONJ, rocblas_int NB_X, rocblas_int WIN, typename T_index, typename Ti, typename Tex>
+template <bool CONJ, rocblas_int NB_X, rocblas_int WIN, typename T_Index, typename Ti, typename Tex>
 ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
                                                      rocblas_int n,
                                                      Tex         alpha,
                                                      const Ti*   A,
-                                                     rocblas_int lda,
+                                                     T_Index     lda,
                                                      const Ti*   x,
-                                                     rocblas_int incx,
+                                                     T_Index     incx,
                                                      Tex*        workspace)
 {
     // skinny n kernel
@@ -727,13 +727,13 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
         {
             for(int j = 0; j < WIN; j++)
             {
-                xvec[j] = x[(row + j) * T_index(incx)];
+                xvec[j] = x[(row + j) * T_Index(incx)];
             }
             for(int j = 0; j < WIN; j++)
             {
                 for(int k = 0; k < NC; k++)
-                    sum[k] += (CONJ ? conj(A[(i + k) * T_index(lda) + j])
-                                    : A[(i + k) * T_index(lda) + j])
+                    sum[k] += (CONJ ? conj(A[(i + k) * T_Index(lda) + j])
+                                    : A[(i + k) * T_Index(lda) + j])
                               * xvec[j];
             }
         }
@@ -741,13 +741,13 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
         {
             for(int j = 0; j < m_tail; j++)
             {
-                xvec[j] = x[(row + j) * T_index(incx)];
+                xvec[j] = x[(row + j) * T_Index(incx)];
             }
             for(int j = 0; j < m_tail; j++)
             {
                 for(int k = 0; k < NC; k++)
-                    sum[k] += (CONJ ? conj(A[(i + k) * T_index(lda) + j])
-                                    : A[(i + k) * T_index(lda) + j])
+                    sum[k] += (CONJ ? conj(A[(i + k) * T_Index(lda) + j])
+                                    : A[(i + k) * T_Index(lda) + j])
                               * xvec[j];
             }
         }
@@ -758,7 +758,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
         if(tx == 0)
         {
             for(int k = 0; k < NC; k++)
-                workspace[blockIdx.x + T_index(k + i) * gridDim.x] = alpha * sum[k];
+                workspace[blockIdx.x + T_Index(k + i) * gridDim.x] = alpha * sum[k];
         }
     }
     for(; i < n; i++)
@@ -769,12 +769,12 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
         {
             for(int j = 0; j < WIN; j++)
             {
-                xvec[j] = x[(row + j) * T_index(incx)];
+                xvec[j] = x[(row + j) * T_Index(incx)];
             }
             for(int j = 0; j < WIN; j++)
             {
                 sum[0]
-                    += (CONJ ? conj(A[(i + 0) * T_index(lda) + j]) : A[(i + 0) * T_index(lda) + j])
+                    += (CONJ ? conj(A[(i + 0) * T_Index(lda) + j]) : A[(i + 0) * T_Index(lda) + j])
                        * xvec[j];
             }
         }
@@ -782,12 +782,12 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_kernel_calc(rocblas_int m,
         {
             for(int j = 0; j < m_tail; j++)
             {
-                xvec[j] = x[(row + j) * T_index(incx)];
+                xvec[j] = x[(row + j) * T_Index(incx)];
             }
             for(int j = 0; j < m_tail; j++)
             {
                 sum[0]
-                    += (CONJ ? conj(A[(i + 0) * T_index(lda) + j]) : A[(i + 0) * T_index(lda) + j])
+                    += (CONJ ? conj(A[(i + 0) * T_Index(lda) + j]) : A[(i + 0) * T_Index(lda) + j])
                        * xvec[j];
             }
         }
@@ -829,7 +829,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvt_sn_reduce_calc(
     }
 }
 
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemvtsm_kernel_calc(rocblas_int m,
                                                     rocblas_int n,
                                                     Tex         alpha,
@@ -885,12 +885,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemvtsm_kernel_calc(rocblas_int m,
     }
 }
 
-template <rocblas_int DIM_X,
-          rocblas_int DIM_Y,
-          rocblas_int elements_per_thread,
-          typename Ti,
-          typename Tex,
-          typename To>
+template <int DIM_X, int DIM_Y, int elements_per_thread, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(DIM_X* DIM_Y)
 rocblas_gemvn_double_buffered_kernel(rocblas_int    m,
                                      rocblas_int    n,
@@ -923,10 +918,10 @@ rocblas_gemvn_double_buffered_kernel(rocblas_int    m,
         m, n, alpha, A, lda, x, incx, y, incy);
 }
 
-template <bool        CONJ,
-          rocblas_int DIM_X,
-          rocblas_int DIM_Y,
-          rocblas_int elements_per_thread,
+template <bool CONJ,
+          int  DIM_X,
+          int  DIM_Y,
+          int  elements_per_thread,
           typename Ti,
           typename Tex,
           typename To>
@@ -962,12 +957,7 @@ rocblas_gemvt_double_buffered_kernel(rocblas_int    m,
         m, n, alpha, A, lda, x, incx, y, incy);
 }
 
-template <rocblas_int DIM_X,
-          rocblas_int DIM_Y,
-          typename T_lda,
-          typename Ti,
-          typename Tex,
-          typename To>
+template <int DIM_X, int DIM_Y, typename T_Index, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(DIM_X* DIM_Y)
 rocblas_gemvn_kernel(rocblas_int    m,
                      rocblas_int    n,
@@ -975,17 +965,17 @@ rocblas_gemvn_kernel(rocblas_int    m,
                      rocblas_stride stride_alpha,
                      const Ti*      Aa,
                      rocblas_stride shifta,
-                     T_lda          lda,
+                     T_Index        lda,
                      rocblas_stride strideA,
                      const Ti*      xa,
                      rocblas_stride shiftx,
-                     rocblas_int    incx,
+                     T_Index        incx,
                      rocblas_stride stridex,
                      Tex            beta_device_host,
                      rocblas_stride stride_beta,
                      To*            ya,
                      rocblas_stride shifty,
-                     rocblas_int    incy,
+                     T_Index        incy,
                      rocblas_stride stridey)
 {
     rocblas_int num_threads = blockDim.x * blockDim.y * blockDim.z;
@@ -1003,11 +993,11 @@ rocblas_gemvn_kernel(rocblas_int    m,
 
     auto* y = load_ptr_batch(ya, blockIdx.y, shifty, stridey);
 
-    rocblas_gemvn_kernel_calc<DIM_X, DIM_Y, T_lda>(m, n, alpha, A, lda, x, incx, beta, y, incy);
+    rocblas_gemvn_kernel_calc<DIM_X, DIM_Y, T_Index>(m, n, alpha, A, lda, x, incx, beta, y, incy);
 }
 
 // lda always cast to size_t so single kernel
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(NB_X)
 rocblas_gemvt_kernel(rocblas_int    m,
                      rocblas_int    n,
@@ -1043,7 +1033,7 @@ rocblas_gemvt_kernel(rocblas_int    m,
 }
 
 //Optimized kernel for GEMV transpose case when m or n is less than 6000
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename T_Index, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(NB_X)
 rocblas_gemvt_warp_reduce_kernel(rocblas_int    m,
                                  rocblas_int    n,
@@ -1051,17 +1041,17 @@ rocblas_gemvt_warp_reduce_kernel(rocblas_int    m,
                                  rocblas_stride stride_alpha,
                                  const Ti*      Aa,
                                  rocblas_stride shifta,
-                                 rocblas_int    lda,
+                                 T_Index        lda,
                                  rocblas_stride strideA,
                                  const Ti*      xa,
                                  rocblas_stride shiftx,
-                                 rocblas_int    incx,
+                                 T_Index        incx,
                                  rocblas_stride stridex,
                                  Tex            beta_device_host,
                                  rocblas_stride stride_beta,
                                  To*            ya,
                                  rocblas_stride shifty,
-                                 rocblas_int    incy,
+                                 T_Index        incy,
                                  rocblas_stride stridey)
 {
     auto alpha = load_scalar(alpha_device_host, blockIdx.y, stride_alpha);
@@ -1075,16 +1065,11 @@ rocblas_gemvt_warp_reduce_kernel(rocblas_int    m,
 
     auto* y = load_ptr_batch(ya, blockIdx.y, shifty, stridey);
 
-    rocblas_gemvt_warp_reduce_kernel_calc<CONJ, NB_X>(m, n, alpha, A, lda, x, incx, beta, y, incy);
+    rocblas_gemvt_warp_reduce_kernel_calc<CONJ, NB_X, T_Index>(
+        m, n, alpha, A, lda, x, incx, beta, y, incy);
 }
 
-template <bool        CONJ,
-          rocblas_int NB_X,
-          rocblas_int WIN,
-          typename T_index,
-          typename Ti,
-          typename U,
-          typename Tex>
+template <bool CONJ, int NB_X, int WIN, typename T_Index, typename Ti, typename U, typename Tex>
 ROCBLAS_KERNEL(NB_X)
 rocblas_gemvt_sn_kernel(rocblas_int    m,
                         rocblas_int    n,
@@ -1105,10 +1090,10 @@ rocblas_gemvt_sn_kernel(rocblas_int    m,
     const auto* A = cond_load_ptr_batch(alpha, Aa, blockIdx.y, shifta, strideA);
     const auto* x = cond_load_ptr_batch(alpha, xa, blockIdx.y, shiftx, stridex);
 
-    rocblas_gemvt_sn_kernel_calc<CONJ, NB_X, WIN, T_index>(m, n, alpha, A, lda, x, incx, workspace);
+    rocblas_gemvt_sn_kernel_calc<CONJ, NB_X, WIN, T_Index>(m, n, alpha, A, lda, x, incx, workspace);
 }
 
-template <rocblas_int NB, rocblas_int WIN, typename Tex, typename U, typename To>
+template <int NB, int WIN, typename Tex, typename U, typename To>
 ROCBLAS_KERNEL(NB)
 rocblas_gemvt_sn_reduce(rocblas_int    n_sums,
                         U              beta_device_host,
@@ -1125,7 +1110,7 @@ rocblas_gemvt_sn_reduce(rocblas_int    n_sums,
     rocblas_gemvt_sn_reduce_calc<NB, WIN>(n_sums, beta, y, incy, workspace);
 }
 
-template <bool CONJ, rocblas_int NB_X, typename Ti, typename Tex, typename To>
+template <bool CONJ, int NB_X, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(NB_X)
 rocblas_gemvtsm_kernel(rocblas_int    m,
                        rocblas_int    n,
@@ -1161,7 +1146,7 @@ rocblas_gemvtsm_kernel(rocblas_int    m,
     rocblas_gemvtsm_kernel_calc<CONJ, NB_X>(m, n, alpha, A, lda, x, incx, beta, y, incy);
 }
 
-template <rocblas_int NB_X, rocblas_int NB_BATCH, typename Ti, typename Tex, typename To>
+template <int NB_X, int NB_BATCH, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL_ILF void rocblas_gemv_sm_mn_batched_kernel_calc(rocblas_int m,
                                                                rocblas_int n,
                                                                Tex         alpha,
@@ -1220,7 +1205,7 @@ ROCBLAS_KERNEL_ILF void rocblas_gemv_sm_mn_batched_kernel_calc(rocblas_int m,
     }
 }
 
-template <rocblas_int NB_X, rocblas_int NB_BATCH, typename Ti, typename Tex, typename To>
+template <int NB_X, int NB_BATCH, typename Ti, typename Tex, typename To>
 ROCBLAS_KERNEL(NB_X* NB_BATCH)
 rocblas_gemvn_sm_mn_batched_kernel(rocblas_int    m,
                                    rocblas_int    n,
