@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,50 +24,48 @@
 #include "handle.hpp"
 #include "rocblas_hpr.hpp"
 
-template <typename T, typename U>
-__device__ void rocblas_hpr_kernel_calc(
-    bool is_upper, rocblas_int n, U alpha, const T* x, rocblas_int incx, T* AP)
+template <int DIM_X, int DIM_Y, int N_TX, typename T, typename U>
+__forceinline__ __device__ void
+    rocblas_hpr_kernel_calc(bool is_upper, rocblas_int n, U alpha, const T* x, int64_t incx, T* AP)
 {
-    rocblas_int tx = blockIdx.x * blockDim.x + threadIdx.x;
-    rocblas_int ty = blockIdx.y * blockDim.y + threadIdx.y;
+    int tx = (blockIdx.x * DIM_X * N_TX) + threadIdx.x;
+    int ty = blockIdx.y * DIM_Y + threadIdx.y;
 
     int index = is_upper ? ((ty * (ty + 1)) / 2) + tx : ((ty * (2 * n - ty + 1)) / 2) + (tx - ty);
 
-    if(is_upper ? ty < n && tx < ty : tx < n && ty < tx)
-        AP[index] += alpha * x[tx * incx] * conj(x[ty * incx]);
-    else if(tx == ty && tx < n)
+#pragma unroll
+    for(int i = 0; i < N_TX; i++, tx += DIM_X, index += DIM_X)
     {
-        U x_real  = std::real(x[tx * incx]);
-        U x_imag  = std::imag(x[tx * incx]);
-        AP[index] = std::real(AP[index]) + alpha * ((x_real * x_real) + (x_imag * x_imag));
+        if(is_upper ? ty < n && tx < ty : tx < n && ty < tx)
+            AP[index] += alpha * x[tx * incx] * conj(x[ty * incx]);
+        else if(tx == ty && tx < n)
+        {
+            AP[index] = std::real(AP[index]) + alpha * std::norm(x[tx * incx]);
+        }
     }
 }
 
-template <rocblas_int DIM_X, rocblas_int DIM_Y, typename TScal, typename TConstPtr, typename TPtr>
+template <int DIM_X, int DIM_Y, int N_TX, typename TScal, typename TConstPtr, typename TPtr>
 ROCBLAS_KERNEL(DIM_X* DIM_Y)
 rocblas_hpr_kernel(bool           is_upper,
                    rocblas_int    n,
                    TScal          alphaa,
                    TConstPtr      xa,
                    rocblas_stride shift_x,
-                   rocblas_int    incx,
+                   int64_t        incx,
                    rocblas_stride stride_x,
                    TPtr           APa,
                    rocblas_stride shift_A,
                    rocblas_stride stride_A)
 {
-    rocblas_int num_threads = blockDim.x * blockDim.y * blockDim.z;
-    if(DIM_X * DIM_Y != num_threads)
-        return; // need to launch exactly the number of threads as template parameters indicate.
-
     auto alpha = load_scalar(alphaa);
     if(!alpha)
         return;
 
-    auto*       AP = load_ptr_batch(APa, blockIdx.z, shift_A, stride_A);
     const auto* x  = load_ptr_batch(xa, blockIdx.z, shift_x, stride_x);
+    auto*       AP = load_ptr_batch(APa, blockIdx.z, shift_A, stride_A);
 
-    rocblas_hpr_kernel_calc(is_upper, n, alpha, x, incx, AP);
+    rocblas_hpr_kernel_calc<DIM_X, DIM_Y, N_TX>(is_upper, n, alpha, x, incx, AP);
 }
 
 /**
@@ -77,19 +75,19 @@ rocblas_hpr_kernel(bool           is_upper,
  * Where T is the base type (rocblas_float_complex or rocblas_double_complex)
  * and U is the scalar type (float or double)
  */
-template <typename TScal, typename TConstPtr, typename TPtr>
-rocblas_status rocblas_hpr_template(rocblas_handle handle,
+template <typename API_INT, typename TScal, typename TConstPtr, typename TPtr>
+rocblas_status rocblas_hpr_launcher(rocblas_handle handle,
                                     rocblas_fill   uplo,
-                                    rocblas_int    n,
+                                    API_INT        n,
                                     TScal          alpha,
                                     TConstPtr      x,
                                     rocblas_stride offset_x,
-                                    rocblas_int    incx,
+                                    int64_t        incx,
                                     rocblas_stride stride_x,
                                     TPtr           AP,
                                     rocblas_stride offset_A,
                                     rocblas_stride stride_A,
-                                    rocblas_int    batch_count)
+                                    API_INT        batch_count)
 {
     // Quick return if possible. Not Argument error
     if(!n || !batch_count)
@@ -98,17 +96,19 @@ rocblas_status rocblas_hpr_template(rocblas_handle handle,
     // in case of negative inc, shift pointer to end of data for negative indexing tid*inc
     ptrdiff_t shift_x = incx < 0 ? offset_x - ptrdiff_t(incx) * (n - 1) : offset_x;
 
-    static constexpr int HPR_DIM_X = 128;
-    static constexpr int HPR_DIM_Y = 8;
-    rocblas_int          blocksX   = (n - 1) / HPR_DIM_X + 1;
-    rocblas_int          blocksY   = (n - 1) / HPR_DIM_Y + 1;
+    static constexpr int HPR_DIM_X = 64;
+    static constexpr int HPR_DIM_Y = 16;
+    static constexpr int N_TX      = 2; // x items per x thread
+
+    rocblas_int blocksX = (n - 1) / (HPR_DIM_X * N_TX) + 1;
+    rocblas_int blocksY = (n - 1) / HPR_DIM_Y + 1;
 
     dim3 hpr_grid(blocksX, blocksY, batch_count);
     dim3 hpr_threads(HPR_DIM_X, HPR_DIM_Y);
 
     if(rocblas_pointer_mode_device == handle->pointer_mode)
     {
-        ROCBLAS_LAUNCH_KERNEL((rocblas_hpr_kernel<HPR_DIM_X, HPR_DIM_Y>),
+        ROCBLAS_LAUNCH_KERNEL((rocblas_hpr_kernel<HPR_DIM_X, HPR_DIM_Y, N_TX>),
                               hpr_grid,
                               hpr_threads,
                               0,
@@ -125,7 +125,7 @@ rocblas_status rocblas_hpr_template(rocblas_handle handle,
                               stride_A);
     }
     else
-        ROCBLAS_LAUNCH_KERNEL((rocblas_hpr_kernel<HPR_DIM_X, HPR_DIM_Y>),
+        ROCBLAS_LAUNCH_KERNEL((rocblas_hpr_kernel<HPR_DIM_X, HPR_DIM_Y, N_TX>),
                               hpr_grid,
                               hpr_threads,
                               0,
@@ -148,15 +148,15 @@ rocblas_status rocblas_hpr_template(rocblas_handle handle,
 template <typename T, typename U>
 rocblas_status rocblas_hpr_check_numerics(const char*    function_name,
                                           rocblas_handle handle,
-                                          rocblas_int    n,
+                                          int64_t        n,
                                           T              AP,
                                           rocblas_stride offset_a,
                                           rocblas_stride stride_a,
                                           U              x,
                                           rocblas_stride offset_x,
-                                          rocblas_int    inc_x,
+                                          int64_t        inc_x,
                                           rocblas_stride stride_x,
-                                          rocblas_int    batch_count,
+                                          int64_t        batch_count,
                                           const int      check_numerics,
                                           bool           is_input)
 {
@@ -178,53 +178,52 @@ rocblas_status rocblas_hpr_check_numerics(const char*    function_name,
 // Instantiations below will need to be manually updated to match any change in
 // template parameters in the files *hpr*.cpp
 
-// clang-format off
-
-#ifdef INSTANTIATE_HPR_TEMPLATE
-#error INSTANTIATE_HPR_TEMPLATE already defined
+#ifdef INST_HPR_LAUNCHER
+#error INST_HPR_LAUNCHER already defined
 #endif
 
-#define INSTANTIATE_HPR_TEMPLATE(TScal_, TConstPtr_, TPtr_)              \
-template rocblas_status rocblas_hpr_template<TScal_, TConstPtr_, TPtr_>  \
-                                   (rocblas_handle handle,               \
-                                    rocblas_fill   uplo,                 \
-                                    rocblas_int    n,                    \
-                                    TScal_          alpha,               \
-                                    TConstPtr_      x,                   \
-                                    rocblas_stride offset_x,             \
-                                    rocblas_int    incx,                 \
-                                    rocblas_stride stride_x,             \
-                                    TPtr_           AP,                  \
-                                    rocblas_stride  offset_A,             \
-                                    rocblas_stride stride_A,             \
-                                    rocblas_int    batch_count);
+#define INST_HPR_LAUNCHER(TScal_, TConstPtr_, TPtr_)                                      \
+    template rocblas_status rocblas_hpr_launcher<rocblas_int, TScal_, TConstPtr_, TPtr_>( \
+        rocblas_handle handle,                                                            \
+        rocblas_fill   uplo,                                                              \
+        rocblas_int    n,                                                                 \
+        TScal_         alpha,                                                             \
+        TConstPtr_     x,                                                                 \
+        rocblas_stride offset_x,                                                          \
+        int64_t        incx,                                                              \
+        rocblas_stride stride_x,                                                          \
+        TPtr_          AP,                                                                \
+        rocblas_stride offset_A,                                                          \
+        rocblas_stride stride_A,                                                          \
+        rocblas_int    batch_count);
 
-INSTANTIATE_HPR_TEMPLATE(float const*, rocblas_float_complex const*, rocblas_float_complex*)
-INSTANTIATE_HPR_TEMPLATE(double const*, rocblas_double_complex const*, rocblas_double_complex*)
-INSTANTIATE_HPR_TEMPLATE(float const*, rocblas_float_complex const* const*, rocblas_float_complex* const*)
-INSTANTIATE_HPR_TEMPLATE(double const*, rocblas_double_complex const* const*, rocblas_double_complex* const*)
+INST_HPR_LAUNCHER(float const*, rocblas_float_complex const*, rocblas_float_complex*)
+INST_HPR_LAUNCHER(double const*, rocblas_double_complex const*, rocblas_double_complex*)
+INST_HPR_LAUNCHER(float const*, rocblas_float_complex const* const*, rocblas_float_complex* const*)
+INST_HPR_LAUNCHER(double const*,
+                  rocblas_double_complex const* const*,
+                  rocblas_double_complex* const*)
 
-#undef INSTANTIATE_HPR_TEMPLATE
+#undef INST_HPR_LAUNCHER
 
 #ifdef INSTANTIATE_HPR_NUMERICS
 #error INSTANTIATE_HPR_NUMERICS already defined
 #endif
 
-#define INSTANTIATE_HPR_NUMERICS(T_, U_)                                   \
-template rocblas_status rocblas_hpr_check_numerics<T_, U_>                 \
-                                         (const char*    function_name,    \
-                                          rocblas_handle handle,           \
-                                          rocblas_int    n,                \
-                                          T_             AP,               \
-                                          rocblas_stride    offset_a,         \
-                                          rocblas_stride stride_a,         \
-                                          U_             x,                \
-                                          rocblas_stride    offset_x,         \
-                                          rocblas_int    inc_x,            \
-                                          rocblas_stride stride_x,         \
-                                          rocblas_int    batch_count,      \
-                                          const int      check_numerics,   \
-                                          bool           is_input);
+#define INSTANTIATE_HPR_NUMERICS(T_, U_)                                                      \
+    template rocblas_status rocblas_hpr_check_numerics<T_, U_>(const char*    function_name,  \
+                                                               rocblas_handle handle,         \
+                                                               int64_t        n,              \
+                                                               T_             AP,             \
+                                                               rocblas_stride offset_a,       \
+                                                               rocblas_stride stride_a,       \
+                                                               U_             x,              \
+                                                               rocblas_stride offset_x,       \
+                                                               int64_t        inc_x,          \
+                                                               rocblas_stride stride_x,       \
+                                                               int64_t        batch_count,    \
+                                                               const int      check_numerics, \
+                                                               bool           is_input);
 
 INSTANTIATE_HPR_NUMERICS(rocblas_float_complex*, rocblas_float_complex const*)
 INSTANTIATE_HPR_NUMERICS(rocblas_double_complex*, rocblas_double_complex const*)
@@ -232,5 +231,3 @@ INSTANTIATE_HPR_NUMERICS(rocblas_float_complex* const*, rocblas_float_complex co
 INSTANTIATE_HPR_NUMERICS(rocblas_double_complex* const*, rocblas_double_complex const* const*)
 
 #undef INSTANTIATE_HPR_NUMERICS
-
-// clang-format on
