@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,19 +28,27 @@
 #include "logging.hpp"
 #include "rocblas.h"
 
-template <typename T, typename A, typename X>
+template <typename API_INT, typename T, typename TConstPtr, typename TPtr>
 inline rocblas_status rocblas_trmv_arg_check(rocblas_handle    handle,
                                              rocblas_fill      uplo,
                                              rocblas_operation transA,
                                              rocblas_diagonal  diag,
-                                             rocblas_int       n,
-                                             A                 a,
-                                             rocblas_int       lda,
-                                             X                 x,
-                                             rocblas_int       incx,
-                                             rocblas_int       batch_count,
+                                             API_INT           n,
+                                             TConstPtr         A,
+                                             API_INT           lda,
+                                             TPtr              x,
+                                             API_INT           incx,
+                                             API_INT           batch_count,
                                              size_t&           dev_bytes)
 {
+    if constexpr(std::is_same_v<API_INT, int>)
+    {
+        if(batch_count > c_YZ_grid_launch_limit && handle->isYZGridDim16bit())
+        {
+            return rocblas_status_invalid_size;
+        }
+    }
+
     if(uplo != rocblas_fill_lower && uplo != rocblas_fill_upper)
         return rocblas_status_invalid_value;
 
@@ -66,12 +74,30 @@ inline rocblas_status rocblas_trmv_arg_check(rocblas_handle    handle,
         return handle->set_optimal_device_memory_size(dev_bytes);
 
     // pointers are validated if they need to be dereferenced
-    if(!a || !x)
+    if(!A || !x)
         return rocblas_status_invalid_pointer;
 
     return rocblas_status_continue;
 }
 
+template <typename TConstPtr, typename TPtr, typename TWork>
+rocblas_status rocblas_internal_trmv_launcher(rocblas_handle    handle,
+                                              rocblas_fill      uplo,
+                                              rocblas_operation transA,
+                                              rocblas_diagonal  diag,
+                                              rocblas_int       n,
+                                              TConstPtr         A,
+                                              rocblas_stride    offset_A,
+                                              int64_t           lda,
+                                              rocblas_stride    stride_A,
+                                              TPtr              x,
+                                              rocblas_stride    offset_x,
+                                              int64_t           incx,
+                                              rocblas_stride    stride_x,
+                                              TWork             workspace,
+                                              rocblas_stride    stride_w,
+                                              rocblas_int       batch_count);
+
 template <typename T>
 ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
     rocblas_internal_trmv_template(rocblas_handle    handle,
@@ -80,15 +106,15 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                    rocblas_diagonal  diag,
                                    rocblas_int       n,
                                    const T*          A,
-                                   rocblas_stride    offseta,
+                                   rocblas_stride    offset_A,
                                    rocblas_int       lda,
-                                   rocblas_stride    stridea,
+                                   rocblas_stride    stride_A,
                                    T*                x,
-                                   rocblas_stride    offsetx,
+                                   rocblas_stride    offset_x,
                                    rocblas_int       incx,
-                                   rocblas_stride    stridex,
+                                   rocblas_stride    stride_x,
                                    T*                workspace,
-                                   rocblas_stride    stridew,
+                                   rocblas_stride    stride_w,
                                    rocblas_int       batch_count);
 
 template <typename T>
@@ -99,30 +125,30 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                            rocblas_diagonal  diag,
                                            rocblas_int       n,
                                            const T* const*   A,
-                                           rocblas_stride    offseta,
+                                           rocblas_stride    offset_A,
                                            rocblas_int       lda,
-                                           rocblas_stride    stridea,
+                                           rocblas_stride    stride_A,
                                            T* const*         x,
-                                           rocblas_stride    offsetx,
+                                           rocblas_stride    offset_x,
                                            rocblas_int       incx,
-                                           rocblas_stride    stridex,
+                                           rocblas_stride    stride_x,
                                            T*                workspace,
-                                           rocblas_stride    stridew,
+                                           rocblas_stride    stride_w,
                                            rocblas_int       batch_count);
 
 template <typename T, typename U>
 rocblas_status rocblas_trmv_check_numerics(const char*    function_name,
                                            rocblas_handle handle,
                                            rocblas_fill   uplo,
-                                           rocblas_int    n,
+                                           int64_t        n,
                                            T              A,
                                            rocblas_stride offset_a,
-                                           rocblas_int    lda,
+                                           int64_t        lda,
                                            rocblas_stride stride_a,
                                            U              x,
                                            rocblas_stride offset_x,
-                                           rocblas_int    inc_x,
+                                           int64_t        inc_x,
                                            rocblas_stride stride_x,
-                                           rocblas_int    batch_count,
+                                           int64_t        batch_count,
                                            const int      check_numerics,
                                            bool           is_input);
