@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,20 +25,29 @@
 #include "check_numerics_matrix.hpp"
 #include "check_numerics_vector.hpp"
 #include "handle.hpp"
+#include "rocblas_block_sizes.h"
 
-template <typename U, typename V>
+template <typename API_INT, typename U, typename V>
 inline rocblas_status rocblas_trsv_arg_check(rocblas_handle    handle,
                                              rocblas_fill      uplo,
                                              rocblas_operation transA,
                                              rocblas_diagonal  diag,
-                                             rocblas_int       n,
+                                             API_INT           n,
                                              U                 A,
-                                             rocblas_int       lda,
+                                             API_INT           lda,
                                              V                 B,
-                                             rocblas_int       incx,
-                                             rocblas_int       batch_count,
+                                             API_INT           incx,
+                                             API_INT           batch_count,
                                              size_t&           dev_bytes)
 {
+    if constexpr(std::is_same_v<API_INT, int>)
+    {
+        if(batch_count > c_YZ_grid_launch_limit && handle->isYZGridDim16bit())
+        {
+            return rocblas_status_invalid_size;
+        }
+    }
+
     if(uplo != rocblas_fill_lower && uplo != rocblas_fill_upper)
         return rocblas_status_invalid_value;
 
@@ -59,10 +68,6 @@ inline rocblas_status rocblas_trsv_arg_check(rocblas_handle    handle,
         return rocblas_status_success;
     }
 
-    // pointers are validated if they need to be dereferenced
-    if(!A || !B)
-        return rocblas_status_invalid_pointer;
-
     // Need one int worth of global memory to keep track of completed sections
     dev_bytes = sizeof(rocblas_int) * batch_count;
     if(handle->is_device_memory_size_query())
@@ -70,6 +75,10 @@ inline rocblas_status rocblas_trsv_arg_check(rocblas_handle    handle,
         return handle->set_optimal_device_memory_size(dev_bytes);
     }
 
+    // pointers are validated if they need to be dereferenced
+    if(!A || !B)
+        return rocblas_status_invalid_pointer;
+
     return rocblas_status_continue;
 }
 
@@ -77,16 +86,16 @@ template <typename T, typename U>
 rocblas_status rocblas_internal_trsv_check_numerics(const char*       function_name,
                                                     rocblas_handle    handle,
                                                     rocblas_fill      uplo,
-                                                    rocblas_int       n,
+                                                    int64_t           n,
                                                     T                 A,
                                                     rocblas_stride    offset_a,
-                                                    rocblas_int       lda,
+                                                    int64_t           lda,
                                                     rocblas_stride    stride_a,
                                                     U                 x,
                                                     rocblas_stride    offset_x,
-                                                    rocblas_int       inc_x,
+                                                    int64_t           inc_x,
                                                     rocblas_stride    stride_x,
-                                                    rocblas_int       batch_count,
+                                                    int64_t           batch_count,
                                                     const rocblas_int check_numerics,
                                                     bool              is_input);
 
@@ -99,12 +108,12 @@ rocblas_status rocblas_internal_trsv_substitution_template(rocblas_handle    han
                                                            rocblas_int       n,
                                                            ATYPE             dA,
                                                            rocblas_stride    offset_A,
-                                                           rocblas_int       lda,
+                                                           int64_t           lda,
                                                            rocblas_stride    stride_A,
                                                            T const*          alpha,
                                                            XTYPE             dx,
                                                            rocblas_stride    offset_x,
-                                                           rocblas_int       incx,
+                                                           int64_t           incx,
                                                            rocblas_stride    stride_x,
                                                            rocblas_int       batch_count,
                                                            rocblas_int*      w_completed_sec);
