@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 #include "handle.hpp"
 #include "rocblas_block_sizes.h"
 #include "rocblas_syrk_herk.hpp"
-#include "rocblas_syrkx.hpp"
+#include "rocblas_syrkx_herkx.hpp"
 
 /**
   * T is base type, i.e. float, double, rocblas_float_complex, or rocblas_double_complex
@@ -47,41 +47,42 @@ rocblas_status rocblas_internal_syrk_herk_template(rocblas_handle    handle,
                                                    TConstPtr         A,
                                                    rocblas_stride    offset_a,
                                                    rocblas_int       lda,
-                                                   rocblas_stride    stride_a,
+                                                   rocblas_stride    stride_A,
                                                    const TScal*      beta,
                                                    TPtr              C,
                                                    rocblas_stride    offset_c,
                                                    rocblas_int       ldc,
-                                                   rocblas_stride    stride_c,
+                                                   rocblas_stride    stride_C,
                                                    rocblas_int       batch_count)
 {
     // quick returns handled in rocblas_internal_syr2k_her2k_template
     constexpr bool TWOK = false;
-    return rocblas_internal_syr2k_her2k_template<NB, BATCHED, TWOK, HERM, T>(handle,
-                                                                             uplo,
-                                                                             trans_a,
-                                                                             n,
-                                                                             k,
-                                                                             alpha,
-                                                                             A,
-                                                                             offset_a,
-                                                                             lda,
-                                                                             stride_a,
-                                                                             A,
-                                                                             offset_a,
-                                                                             lda,
-                                                                             stride_a,
-                                                                             beta,
-                                                                             C,
-                                                                             offset_c,
-                                                                             ldc,
-                                                                             stride_c,
-                                                                             batch_count);
+    return rocblas_internal_syr2k_her2k_template<rocblas_int, NB, BATCHED, TWOK, HERM, T>(
+        handle,
+        uplo,
+        trans_a,
+        n,
+        k,
+        alpha,
+        A,
+        offset_a,
+        lda,
+        stride_A,
+        A,
+        offset_a,
+        lda,
+        stride_A,
+        beta,
+        C,
+        offset_c,
+        ldc,
+        stride_C,
+        batch_count);
 }
 
 #define ROCBLAS_INTERNAL_SYRK_HERK_PARAMS                                                   \
-    handle, uplo, trans_a, n, k, alpha, A, offset_a, lda, stride_a, beta, C, offset_c, ldc, \
-        stride_c, batch_count
+    handle, uplo, trans_a, n, k, alpha, A, offset_a, lda, stride_A, beta, C, offset_c, ldc, \
+        stride_C, batch_count
 
 template <typename T>
 ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
@@ -94,12 +95,12 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                    const T*          A,
                                    rocblas_stride    offset_a,
                                    rocblas_int       lda,
-                                   rocblas_stride    stride_a,
+                                   rocblas_stride    stride_A,
                                    const T*          beta,
                                    T*                C,
                                    rocblas_stride    offset_c,
                                    rocblas_int       ldc,
-                                   rocblas_stride    stride_c,
+                                   rocblas_stride    stride_C,
                                    rocblas_int       batch_count)
 {
     constexpr bool BATCHED = false;
@@ -131,12 +132,12 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                            const T* const*   A,
                                            rocblas_stride    offset_a,
                                            rocblas_int       lda,
-                                           rocblas_stride    stride_a,
+                                           rocblas_stride    stride_A,
                                            const T*          beta,
                                            T* const*         C,
                                            rocblas_stride    offset_c,
                                            rocblas_int       ldc,
-                                           rocblas_stride    stride_c,
+                                           rocblas_stride    stride_C,
                                            rocblas_int       batch_count)
 {
     constexpr bool BATCHED = true;
@@ -168,12 +169,12 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                    const T*          A,
                                    rocblas_stride    offset_a,
                                    rocblas_int       lda,
-                                   rocblas_stride    stride_a,
+                                   rocblas_stride    stride_A,
                                    const real_t<T>*  beta,
                                    T*                C,
                                    rocblas_stride    offset_c,
                                    rocblas_int       ldc,
-                                   rocblas_stride    stride_c,
+                                   rocblas_stride    stride_C,
                                    rocblas_int       batch_count)
 {
     constexpr bool BATCHED = false;
@@ -199,12 +200,12 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                            const T* const*   A,
                                            rocblas_stride    offset_a,
                                            rocblas_int       lda,
-                                           rocblas_stride    stride_a,
+                                           rocblas_stride    stride_A,
                                            const real_t<T>*  beta,
                                            T* const*         C,
                                            rocblas_stride    offset_c,
                                            rocblas_int       ldc,
-                                           rocblas_stride    stride_c,
+                                           rocblas_stride    stride_C,
                                            rocblas_int       batch_count)
 {
     constexpr bool BATCHED = true;
@@ -226,15 +227,15 @@ rocblas_status rocblas_herk_syrk_check_numerics(const char*       function_name,
                                                 rocblas_handle    handle,
                                                 rocblas_fill      uplo,
                                                 rocblas_operation trans,
-                                                rocblas_int       n,
-                                                rocblas_int       k,
+                                                int64_t           n_64,
+                                                int64_t           k_64,
                                                 TConstPtr         A,
-                                                rocblas_int       lda,
-                                                rocblas_stride    stride_a,
+                                                int64_t           lda_64,
+                                                rocblas_stride    stride_A,
                                                 TPtr              C,
-                                                rocblas_int       ldc,
-                                                rocblas_stride    stride_c,
-                                                rocblas_int       batch_count,
+                                                int64_t           ldc_64,
+                                                rocblas_stride    stride_C,
+                                                int64_t           batch_count_64,
                                                 const int         check_numerics,
                                                 bool              is_input)
 {
@@ -247,13 +248,13 @@ rocblas_status rocblas_herk_syrk_check_numerics(const char*       function_name,
                                                               trans,
                                                               rocblas_fill_full,
                                                               rocblas_client_general_matrix,
-                                                              n,
-                                                              k,
+                                                              n_64,
+                                                              k_64,
                                                               A,
                                                               0,
-                                                              lda,
-                                                              stride_a,
-                                                              batch_count,
+                                                              lda_64,
+                                                              stride_A,
+                                                              batch_count_64,
                                                               check_numerics,
                                                               is_input);
         if(check_numerics_status != rocblas_status_success)
@@ -266,13 +267,13 @@ rocblas_status rocblas_herk_syrk_check_numerics(const char*       function_name,
         rocblas_operation_none,
         uplo,
         HERM ? rocblas_client_hermitian_matrix : rocblas_client_symmetric_matrix,
-        n,
-        n,
+        n_64,
+        n_64,
         C,
         0,
-        ldc,
-        stride_c,
-        batch_count,
+        ldc_64,
+        stride_C,
+        batch_count_64,
         check_numerics,
         is_input);
 
@@ -282,30 +283,28 @@ rocblas_status rocblas_herk_syrk_check_numerics(const char*       function_name,
 // Instantiations below will need to be manually updated to match any change in
 // template parameters in the files syrk*.cpp or herk*.cpp
 
-// clang-format off
-
 #ifdef INSTANTIATE_SYRK_TEMPLATE
 #error INSTANTIATE_SYRK_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_SYRK_TEMPLATE(T_)                             \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status          \
-rocblas_internal_syrk_template<T_>(rocblas_handle    handle,      \
-                                   rocblas_fill      uplo,        \
-                                   rocblas_operation trans_a,     \
-                                   rocblas_int       n,           \
-                                   rocblas_int       k,           \
-                                   const T_*         alpha,       \
-                                   const T_*         A,           \
-                                   rocblas_stride    offset_a,    \
-                                   rocblas_int       lda,         \
-                                   rocblas_stride    stride_a,    \
-                                   const T_*         beta,        \
-                                   T_*               C,           \
-                                   rocblas_stride    offset_c,    \
-                                   rocblas_int       ldc,         \
-                                   rocblas_stride    stride_c,    \
-                                   rocblas_int       batch_count);
+#define INSTANTIATE_SYRK_TEMPLATE(T_)                                                            \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status rocblas_internal_syrk_template<T_>( \
+        rocblas_handle    handle,                                                                \
+        rocblas_fill      uplo,                                                                  \
+        rocblas_operation trans_a,                                                               \
+        rocblas_int       n,                                                                     \
+        rocblas_int       k,                                                                     \
+        const T_*         alpha,                                                                 \
+        const T_*         A,                                                                     \
+        rocblas_stride    offset_a,                                                              \
+        rocblas_int       lda,                                                                   \
+        rocblas_stride    stride_A,                                                              \
+        const T_*         beta,                                                                  \
+        T_*               C,                                                                     \
+        rocblas_stride    offset_c,                                                              \
+        rocblas_int       ldc,                                                                   \
+        rocblas_stride    stride_C,                                                              \
+        rocblas_int       batch_count);
 
 INSTANTIATE_SYRK_TEMPLATE(float)
 INSTANTIATE_SYRK_TEMPLATE(double)
@@ -318,24 +317,24 @@ INSTANTIATE_SYRK_TEMPLATE(rocblas_double_complex)
 #error INSTANTIATE_SYRK_BATCHED_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_SYRK_BATCHED_TEMPLATE(T_)                             \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status                  \
-rocblas_internal_syrk_batched_template<T_>(rocblas_handle    handle,      \
-                                           rocblas_fill      uplo,        \
-                                           rocblas_operation trans_a,     \
-                                           rocblas_int       n,           \
-                                           rocblas_int       k,           \
-                                           const T_*         alpha,       \
-                                           const T_* const*  A,           \
-                                           rocblas_stride    offset_a,    \
-                                           rocblas_int       lda,         \
-                                           rocblas_stride    stride_a,    \
-                                           const T_*         beta,        \
-                                           T_* const*        C,           \
-                                           rocblas_stride    offset_c,    \
-                                           rocblas_int       ldc,         \
-                                           rocblas_stride    stride_c,    \
-                                           rocblas_int       batch_count);
+#define INSTANTIATE_SYRK_BATCHED_TEMPLATE(T_)                                  \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status                   \
+        rocblas_internal_syrk_batched_template<T_>(rocblas_handle    handle,   \
+                                                   rocblas_fill      uplo,     \
+                                                   rocblas_operation trans_a,  \
+                                                   rocblas_int       n,        \
+                                                   rocblas_int       k,        \
+                                                   const T_*         alpha,    \
+                                                   const T_* const*  A,        \
+                                                   rocblas_stride    offset_a, \
+                                                   rocblas_int       lda,      \
+                                                   rocblas_stride    stride_A, \
+                                                   const T_*         beta,     \
+                                                   T_* const*        C,        \
+                                                   rocblas_stride    offset_c, \
+                                                   rocblas_int       ldc,      \
+                                                   rocblas_stride    stride_C, \
+                                                   rocblas_int       batch_count);
 
 INSTANTIATE_SYRK_BATCHED_TEMPLATE(float)
 INSTANTIATE_SYRK_BATCHED_TEMPLATE(double)
@@ -348,24 +347,24 @@ INSTANTIATE_SYRK_BATCHED_TEMPLATE(rocblas_double_complex)
 #error INSTANTIATE_HERK_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_HERK_TEMPLATE(T_)                             \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status          \
-rocblas_internal_herk_template<T_>(rocblas_handle    handle,      \
-                                   rocblas_fill      uplo,        \
-                                   rocblas_operation trans_a,     \
-                                   rocblas_int       n,           \
-                                   rocblas_int       k,           \
-                                   const real_t<T_>* alpha,       \
-                                   const T_*         A,           \
-                                   rocblas_stride    offset_a,    \
-                                   rocblas_int       lda,         \
-                                   rocblas_stride    stride_a,    \
-                                   const real_t<T_>* beta,        \
-                                   T_*               C,           \
-                                   rocblas_stride    offset_c,    \
-                                   rocblas_int       ldc,         \
-                                   rocblas_stride    stride_c,    \
-                                   rocblas_int       batch_count);
+#define INSTANTIATE_HERK_TEMPLATE(T_)                                                            \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status rocblas_internal_herk_template<T_>( \
+        rocblas_handle    handle,                                                                \
+        rocblas_fill      uplo,                                                                  \
+        rocblas_operation trans_a,                                                               \
+        rocblas_int       n,                                                                     \
+        rocblas_int       k,                                                                     \
+        const real_t<T_>* alpha,                                                                 \
+        const T_*         A,                                                                     \
+        rocblas_stride    offset_a,                                                              \
+        rocblas_int       lda,                                                                   \
+        rocblas_stride    stride_A,                                                              \
+        const real_t<T_>* beta,                                                                  \
+        T_*               C,                                                                     \
+        rocblas_stride    offset_c,                                                              \
+        rocblas_int       ldc,                                                                   \
+        rocblas_stride    stride_C,                                                              \
+        rocblas_int       batch_count);
 
 INSTANTIATE_HERK_TEMPLATE(rocblas_float_complex)
 INSTANTIATE_HERK_TEMPLATE(rocblas_double_complex)
@@ -376,24 +375,24 @@ INSTANTIATE_HERK_TEMPLATE(rocblas_double_complex)
 #error INSTANTIATE_HERK_BATCHED_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_HERK_BATCHED_TEMPLATE(T_)                             \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status                  \
-rocblas_internal_herk_batched_template<T_>(rocblas_handle    handle,      \
-                                           rocblas_fill      uplo,        \
-                                           rocblas_operation trans_a,     \
-                                           rocblas_int       n,           \
-                                           rocblas_int       k,           \
-                                           const real_t<T_>* alpha,       \
-                                           const T_* const*  A,           \
-                                           rocblas_stride    offset_a,    \
-                                           rocblas_int       lda,         \
-                                           rocblas_stride    stride_a,    \
-                                           const real_t<T_>* beta,        \
-                                           T_* const*        C,           \
-                                           rocblas_stride    offset_c,    \
-                                           rocblas_int       ldc,         \
-                                           rocblas_stride    stride_c,    \
-                                           rocblas_int       batch_count);
+#define INSTANTIATE_HERK_BATCHED_TEMPLATE(T_)                                  \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status                   \
+        rocblas_internal_herk_batched_template<T_>(rocblas_handle    handle,   \
+                                                   rocblas_fill      uplo,     \
+                                                   rocblas_operation trans_a,  \
+                                                   rocblas_int       n,        \
+                                                   rocblas_int       k,        \
+                                                   const real_t<T_>* alpha,    \
+                                                   const T_* const*  A,        \
+                                                   rocblas_stride    offset_a, \
+                                                   rocblas_int       lda,      \
+                                                   rocblas_stride    stride_A, \
+                                                   const real_t<T_>* beta,     \
+                                                   T_* const*        C,        \
+                                                   rocblas_stride    offset_c, \
+                                                   rocblas_int       ldc,      \
+                                                   rocblas_stride    stride_C, \
+                                                   rocblas_int       batch_count);
 
 INSTANTIATE_HERK_BATCHED_TEMPLATE(rocblas_float_complex)
 INSTANTIATE_HERK_BATCHED_TEMPLATE(rocblas_double_complex)
@@ -405,39 +404,45 @@ INSTANTIATE_HERK_BATCHED_TEMPLATE(rocblas_double_complex)
 #endif
 
 #define INSTANTIATE_HERK_SYRK_NUMERICS(HERM_, TConstPtr_, TPtr_)                        \
-template rocblas_status rocblas_herk_syrk_check_numerics                                \
-                                  <HERM_, TConstPtr_, TPtr_>                            \
-                                  (const char*       function_name,                     \
-                                   rocblas_handle handle,                               \
-                                   rocblas_fill   uplo,                                 \
-                                   rocblas_operation trans,                             \
-                                   rocblas_int    n,                                    \
-                                   rocblas_int    k,                                    \
-                                   TConstPtr_     A,                                    \
-                                   rocblas_int    lda,                                  \
-                                   rocblas_stride stride_a,                             \
-                                   TPtr_          C,                                    \
-                                   rocblas_int    ldc,                                  \
-                                   rocblas_stride stride_c,                             \
-                                   rocblas_int    batch_count,                          \
-                                   const int      check_numerics,                       \
-                                   bool           is_input);
+    template rocblas_status rocblas_herk_syrk_check_numerics<HERM_, TConstPtr_, TPtr_>( \
+        const char*       function_name,                                                \
+        rocblas_handle    handle,                                                       \
+        rocblas_fill      uplo,                                                         \
+        rocblas_operation trans,                                                        \
+        int64_t           n_64,                                                         \
+        int64_t           k_64,                                                         \
+        TConstPtr_        A,                                                            \
+        int64_t           lda_64,                                                       \
+        rocblas_stride    stride_A,                                                     \
+        TPtr_             C,                                                            \
+        int64_t           ldc_64,                                                       \
+        rocblas_stride    stride_C,                                                     \
+        int64_t           batch_count_64,                                               \
+        const int         check_numerics,                                               \
+        bool              is_input);
 
 // instantiate for rocblas_Xherk_Xsyrk and rocblas_Xherk_Xsyrk_strided_batched
 INSTANTIATE_HERK_SYRK_NUMERICS(false, float const*, float*)
 INSTANTIATE_HERK_SYRK_NUMERICS(false, double const*, double*)
-INSTANTIATE_HERK_SYRK_NUMERICS(false,  rocblas_float_complex const*, rocblas_float_complex*)
-INSTANTIATE_HERK_SYRK_NUMERICS( true,  rocblas_float_complex const*, rocblas_float_complex*)
+INSTANTIATE_HERK_SYRK_NUMERICS(false, rocblas_float_complex const*, rocblas_float_complex*)
+INSTANTIATE_HERK_SYRK_NUMERICS(true, rocblas_float_complex const*, rocblas_float_complex*)
 INSTANTIATE_HERK_SYRK_NUMERICS(false, rocblas_double_complex const*, rocblas_double_complex*)
-INSTANTIATE_HERK_SYRK_NUMERICS( true, rocblas_double_complex const*, rocblas_double_complex*)
+INSTANTIATE_HERK_SYRK_NUMERICS(true, rocblas_double_complex const*, rocblas_double_complex*)
 
 // instantiate for rocblas_Xherk_Xsyrk_batched
 INSTANTIATE_HERK_SYRK_NUMERICS(false, float const* const*, float* const*)
 INSTANTIATE_HERK_SYRK_NUMERICS(false, double const* const*, double* const*)
-INSTANTIATE_HERK_SYRK_NUMERICS(false,  rocblas_float_complex const* const*, rocblas_float_complex* const*)
-INSTANTIATE_HERK_SYRK_NUMERICS( true,  rocblas_float_complex const* const*, rocblas_float_complex* const*)
-INSTANTIATE_HERK_SYRK_NUMERICS(false, rocblas_double_complex const* const*, rocblas_double_complex* const*)
-INSTANTIATE_HERK_SYRK_NUMERICS( true, rocblas_double_complex const* const*, rocblas_double_complex* const*)
+INSTANTIATE_HERK_SYRK_NUMERICS(false,
+                               rocblas_float_complex const* const*,
+                               rocblas_float_complex* const*)
+INSTANTIATE_HERK_SYRK_NUMERICS(true,
+                               rocblas_float_complex const* const*,
+                               rocblas_float_complex* const*)
+INSTANTIATE_HERK_SYRK_NUMERICS(false,
+                               rocblas_double_complex const* const*,
+                               rocblas_double_complex* const*)
+INSTANTIATE_HERK_SYRK_NUMERICS(true,
+                               rocblas_double_complex const* const*,
+                               rocblas_double_complex* const*)
 
 #undef INSTANTIATE_HERK_SYRK_NUMERICS
-// clang-format on
