@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,8 +23,8 @@
 
 #pragma once
 
-#include "gemm.hpp"
 #include "rocblas_block_sizes.h"
+#include "rocblas_gemm.hpp"
 #include "rocblas_trtri.hpp"
 
 /*
