@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,10 +22,15 @@
 #include "handle.hpp"
 #include <cstdarg>
 #include <limits>
+
 #ifdef WIN32
 #include <windows.h>
 #endif
 
+#ifdef BUILD_WITH_HIPBLASLT
+#include <hipblaslt/hipblaslt.h>
+#endif
+
 #if BUILD_WITH_TENSILE
 #else
 // see TensileHost.cpp for normal rocblas_initialize definition
@@ -42,7 +47,7 @@ extern "C" void rocblas_shutdown()
 /* read environment variable */
 /* On windows, getenv take a copy of the environment at the beginning of the process */
 /* This behavior is not suited for the purpose of the tests */
-const char* read_env(const char* env_var)
+static const char* read_env(const char* env_var)
 {
 #ifdef WIN32
     const DWORD              nSize = _MAX_PATH;
@@ -141,6 +146,18 @@ static Processor getActiveArch(int deviceId)
     {
         return Processor::gfx1102;
     }
+    else if(deviceString.find("gfx1151") != std::string::npos)
+    {
+        return Processor::gfx1151;
+    }
+    else if(deviceString.find("gfx1200") != std::string::npos)
+    {
+        return Processor::gfx1200;
+    }
+    else if(deviceString.find("gfx1201") != std::string::npos)
+    {
+        return Processor::gfx1201;
+    }
     return static_cast<Processor>(0);
 }
 
@@ -163,6 +180,14 @@ _rocblas_handle::_rocblas_handle()
         stream_order_alloc             = stream_order_alloc_env_val ? true : false;
     }
 
+    //ROCBLAS_DEFAULT_ATOMICS_MODE
+    const char* atomics_mode_env = read_env("ROCBLAS_DEFAULT_ATOMICS_MODE");
+    if(atomics_mode_env)
+    {
+        atomics_mode = strtoul(atomics_mode_env, nullptr, 0) ? rocblas_atomics_allowed
+                                                             : rocblas_atomics_not_allowed;
+    }
+
     // Device memory size
     const char* env = read_env("ROCBLAS_DEVICE_MEMORY_SIZE");
     if(env)
@@ -185,7 +210,8 @@ _rocblas_handle::_rocblas_handle()
             }
             else
             {
-                device_memory_size = DEFAULT_DEVICE_MEMORY_SIZE;
+
+                device_memory_size = getDefaultDeviceMemorySize();
             }
         }
     }
@@ -221,6 +247,38 @@ _rocblas_handle::_rocblas_handle()
 
     // Initialize numerical checking
     init_check_numerics();
+
+#ifdef BUILD_WITH_HIPBLASLT
+    const char* hipblasltEnvVal = read_env("ROCBLAS_USE_HIPBLASLT");
+
+    if(hipblasltEnvVal)
+    {
+        if(strncmp(hipblasltEnvVal, "1", 1) == 0)
+        {
+            hipblasltEnvVar = 1;
+        }
+        else
+        {
+            hipblasltEnvVar = 0;
+        }
+    }
+    else
+    {
+        hipblasltEnvVar = -1;
+    }
+
+    if(useHipBLASLt())
+    {
+        hipblasLtHandle                = std::make_shared<hipblasLtHandle_t>();
+        hipblasStatus_t hipblas_status = hipblasLtCreate(&(*hipblasLtHandle));
+        if(HIPBLAS_STATUS_SUCCESS != hipblas_status)
+        {
+            rocblas_cerr << "rocBLAS internal error: Unable to initialize hipblaslt: "
+                         << hipblas_status << std::endl;
+            rocblas_abort();
+        }
+    }
+#endif
 }
 
 /*******************************************************************************
@@ -295,6 +353,20 @@ _rocblas_handle::~_rocblas_handle()
 #endif
         }
     }
+
+#ifdef BUILD_WITH_HIPBLASLT
+    if(hipblasLtHandle.unique())
+    {
+        hipblasStatus_t hipblas_status = hipblasLtDestroy(*hipblasLtHandle);
+        if(HIPBLAS_STATUS_SUCCESS != hipblas_status)
+        {
+            rocblas_cerr << "rocBLAS internal error: Unable to destroy hipblaslt: "
+                         << hipblas_status << std::endl;
+            rocblas_abort();
+        }
+        hipblasLtHandle.reset();
+    }
+#endif
 }
 
 /*******************************************************************************
@@ -322,7 +394,7 @@ bool _rocblas_handle::device_allocator(size_t size)
 
         //Add an additional device memory on top of default size.
         //This is to support kernels requiring large workspace with numerical checking enabled.
-        size_t total_size = size + DEFAULT_DEVICE_MEMORY_SIZE;
+        size_t total_size = size + getDefaultDeviceMemorySize();
 
         if(!device_memory || (hipFree)(device_memory) == hipSuccess)
         {
