@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,6 +41,9 @@
 #endif
 #include <utility>
 
+// forward declare hipblaslt handle
+typedef void* hipblasLtHandle_t;
+
 // forcing early cleanup
 extern "C" ROCBLAS_EXPORT void rocblas_shutdown();
 
@@ -92,7 +95,10 @@ enum class Processor : int
     gfx1035 = 1035,
     gfx1100 = 1100,
     gfx1101 = 1101,
-    gfx1102 = 1102
+    gfx1102 = 1102,
+    gfx1151 = 1151,
+    gfx1200 = 1200,
+    gfx1201 = 1201
 };
 
 // helper function in handle.cpp
@@ -233,6 +239,78 @@ public:
         return archMajorMinor;
     }
 
+    int getMaxSharedMemPerBlock()
+    {
+        int max_mem = -1;
+        THROW_IF_HIP_ERROR(hipDeviceGetAttribute(
+            &max_mem, hipDeviceAttribute_t(hipDeviceAttributeMaxSharedMemoryPerBlock), device));
+        return max_mem;
+    }
+
+    bool isYZGridDim16bit()
+    {
+        return archMajor == 12;
+    }
+
+    bool isDefaultHipBLASLtArch()
+    {
+        int arch = getArch();
+        if(arch == 1200 || arch == 1201)
+        {
+            return true;
+        }
+        return false;
+    }
+
+    auto getHipblasLtHandle()
+    {
+        return hipblasLtHandle;
+    }
+
+    /*******************************************************************************
+     * This function determines whether or not to use the hipBLASLt backend based
+     * on the state of the environment variable and the current architecture.
+     *
+     * - If the enviornment variable is set, its value determines whether ot not to
+     *   use the hipBLASLt backend.
+     * - If the current architecture is supported, then the `prob_specific_useHipBLASLt`
+     *   input determines whether ot not to use the hipBLASLt backend.
+     * - Otherwise, the hipBLASLt backend is not used.
+     ******************************************************************************/
+    auto useHipBLASLt(bool prob_specific_useHipBLASLt = true)
+    {
+
+#ifdef BUILD_WITH_HIPBLASLT
+        if(hipblasltEnvVar < 0)
+        {
+            if(isDefaultHipBLASLtArch())
+            {
+                return prob_specific_useHipBLASLt;
+            }
+            else
+            {
+                return false;
+            }
+        }
+        return hipblasltEnvVar == 1;
+#else
+        return false;
+#endif
+    }
+
+    inline int getDefaultDeviceMemorySize()
+    {
+
+        if(getArchMajor() == 9 && getArchMajorMinor() >= 94)
+        {
+            return DEFAULT_DEVICE_MEMORY_SIZE_EXTENDED;
+        }
+        else
+        {
+            return DEFAULT_DEVICE_MEMORY_SIZE;
+        }
+    }
+
     // hipEvent_t pointers (for internal use only)
     hipEvent_t startEvent = nullptr;
     hipEvent_t stopEvent  = nullptr;
@@ -350,18 +428,27 @@ public:
     bool is_stream_in_capture_mode()
     {
         hipStreamCaptureStatus capture_status = hipStreamCaptureStatusNone;
-        bool                   status = hipStreamIsCapturing(stream, &capture_status) == hipSuccess;
-        if(!status)
-            rocblas_cerr << "Stream capture check failed" << std::endl;
-        if(capture_status == hipStreamCaptureStatusActive)
-            return true;
-        else
+        //default stream will not be in capture mode
+        if(stream != 0)
+        {
+            hipError_t status = hipStreamIsCapturing(stream, &capture_status);
+            if(status != hipSuccess)
+            {
+                PRINT_IF_HIP_ERROR(status);
+                return false;
+            }
+        }
+
+        if(capture_status == hipStreamCaptureStatusNone)
             return false;
+        else
+            return true; // returns true for both hipStreamCaptureStatusActive & hipStreamCaptureStatusInvalidated
     }
 
 private:
     // device memory work buffer
-    static constexpr size_t DEFAULT_DEVICE_MEMORY_SIZE = 32 * 1024 * 1024;
+    static constexpr size_t DEFAULT_DEVICE_MEMORY_SIZE          = 32 * 1024 * 1024;
+    static constexpr size_t DEFAULT_DEVICE_MEMORY_SIZE_EXTENDED = 128 * 1024 * 1024;
 
     // Variables holding state of device memory allocation
     void*                           device_memory              = nullptr;
@@ -393,6 +480,10 @@ private:
     int       archMajor;
     int       archMajorMinor;
 
+    // hipBLASLt handle is created at handle creation time and remains in effect for the life of the handle.
+    std::shared_ptr<hipblasLtHandle_t> hipblasLtHandle;
+    int                                hipblasltEnvVar = -1;
+
     // Opaque smart allocator class to perform device memory allocations
     // clang-format off
     class [[nodiscard]] _device_malloc : public rocblas_device_malloc_base
