@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -71,9 +71,9 @@ rocblas_status rocblas_internal_dot_launcher_64(rocblas_handle __restrict__ hand
     static constexpr int WIN      = rocblas_dot_WIN<T>();
     int64_t              n_passes = (n_64 - 1) / c_i64_grid_X_chunk + 1;
 
-    if(std::abs(incx_64) <= c_i32_max && std::abs(incy_64) < c_i32_max)
+    if(std::abs(incx_64) <= c_ILP64_i32_max && std::abs(incy_64) <= c_ILP64_i32_max)
     {
-        if(n_64 <= c_i32_max && batch_count_64 < c_i64_grid_YZ_chunk)
+        if(n_64 <= c_ILP64_i32_max && batch_count_64 < c_i64_grid_YZ_chunk)
         {
             // valid to use original 32bit API with truncated 64bit args
             return rocblas_internal_dot_launcher<rocblas_int, NB, CONJ, T, U, V>(handle,
@@ -606,6 +606,8 @@ INST_DOT_EX_LAUNCHER(int64_t, ROCBLAS_DOT_NB, false, float, rocblas_bfloat16 con
 // Mixed precision for dot_ex
 INST_DOT_EX_LAUNCHER_64(false, rocblas_half, rocblas_half const*, float);
 INST_DOT_EX_LAUNCHER_64(false, rocblas_half, rocblas_half const* const*, float);
+INST_DOT_EX_LAUNCHER_64(false, double, float const*, double);
+INST_DOT_EX_LAUNCHER_64(false, double, float const* const*, double);
 
 // real types are "supported" in dotc_ex
 INST_DOT_EX_LAUNCHER_64(true, rocblas_half, rocblas_half const*, float);
@@ -614,6 +616,8 @@ INST_DOT_EX_LAUNCHER_64(true, rocblas_half, rocblas_half const*, rocblas_half);
 INST_DOT_EX_LAUNCHER_64(true, rocblas_half, rocblas_half const* const*, rocblas_half);
 INST_DOT_EX_LAUNCHER_64(true, rocblas_bfloat16, rocblas_bfloat16 const*, float);
 INST_DOT_EX_LAUNCHER_64(true, rocblas_bfloat16, rocblas_bfloat16 const* const*, float);
+INST_DOT_EX_LAUNCHER_64(true, double, float const*, double);
+INST_DOT_EX_LAUNCHER_64(true, double, float const* const*, double);
 INST_DOT_EX_LAUNCHER_64(true, float, float const*, float);
 INST_DOT_EX_LAUNCHER_64(true, float, float const* const*, float);
 INST_DOT_EX_LAUNCHER_64(true, double, double const*, double);
