@@ -24,13 +24,15 @@
 
 #include "rocblas.h"
 
-extern "C" void rocblas_shutdown();
-
 /*****************************************************************************
  * This is the only file in rocBLAS which should #include Tensile headers    *
  * or reference Tensile identifiers. tensile_host.hpp defines the interface. *
  *****************************************************************************/
 
+#ifdef BUILD_WITH_HIPBLASLT
+#include "hipblaslt_host.hpp"
+#endif
+
 #include "tensile_host.hpp"
 //#include <Tensile/AMDGPU.hpp>
 #include <Tensile/Contractions.hpp>
@@ -67,9 +69,10 @@ extern "C" void rocblas_shutdown();
 #include <libgen.h>
 #include <link.h>
 #include <unistd.h>
-#define ROCBLAS_LIB_PATH "/opt/rocm/lib/rocblas"
+#define ROCBLAS_LIB_PATH "/opt/rocm/lib"
 #endif
 
+// cppcheck-suppress preprocessorErrorDirective
 #if __has_include(<filesystem>)
 #include <filesystem>
 namespace fs = std::filesystem;
@@ -98,6 +101,45 @@ namespace
     }
 #endif
 
+    std::string rocblas_exepath()
+    {
+#ifdef WIN32
+        std::vector<TCHAR> result(MAX_PATH + 1);
+        // Ensure result is large enough to accommodate the path
+        DWORD length = 0;
+        for(;;)
+        {
+            length = GetModuleFileNameA(nullptr, result.data(), result.size());
+            if(length < result.size() - 1)
+            {
+                result.resize(length + 1);
+                break;
+            }
+            result.resize(result.size() * 2);
+        }
+
+        fs::path exepath(result.begin(), result.end());
+        exepath = exepath.remove_filename();
+        // Add trailing "/" to exepath if required
+        exepath += exepath.empty() ? "" : "/";
+        return exepath.string();
+#else
+        std::string pathstr;
+        char*       path = realpath("/proc/self/exe", 0);
+        if(path)
+        {
+            char* p = strrchr(path, '/');
+            if(p)
+            {
+                p[1]    = 0;
+                pathstr = path;
+            }
+            free(path);
+        }
+        return pathstr;
+#endif
+    }
+
     /******************************************************
      * Map a rocBLAS type to a corresponding Tensile type *
      ******************************************************/
@@ -694,6 +736,7 @@ namespace
                     // Fall back on hard-coded path if static library or not found
 
 #ifndef ROCBLAS_STATIC_LIB
+
 #ifdef WIN32
                     // wchar_t wpath[MAX_PATH + 1] = {0};
                     // if(GetModuleFileNameW(GetModuleHandle("rocblas.dll"), wpath, MAX_PATH + 1))
@@ -717,9 +760,8 @@ namespace
                     if(rocblas_so_path.size())
                         base_path = std::string{dirname(&rocblas_so_path[0])};
 #endif
-#endif // ifndef ROCBLAS_STATIC_LIB
 
-                    // Find the location of the libraries
+                    // Find the location of the Tensile libraries relative to shared library
                     if(TestPath(base_path + "/../../Tensile/library"))
                         base_path += "/../../Tensile/library";
                     else if(TestPath(base_path + "/library"))
@@ -730,6 +772,22 @@ namespace
                         base_path += "/../rocblas/library";
                     else
                         base_path += "/rocblas/library";
+
+#else
+                    // First location relative to standard static library install, then relative to current executable
+                    if(TestPath(base_path + "/rocblas/library"))
+                        base_path += "/rocblas/library";
+                    else
+                    {
+                        std::string exe_path = rocblas_exepath();
+                        if(TestPath(exe_path + "rocblas/library"))
+                            base_path = exe_path + "rocblas/library";
+                        else if(TestPath(exe_path + "../../Tensile/library"))
+                            base_path = exe_path + "../../Tensile/library";
+                        else
+                            base_path += "/rocblas/library";
+                    }
+#endif // ROCBLAS_STATIC_LIB
                 }
                 return 0;
             }();
@@ -1044,6 +1102,17 @@ inline bool fallbackTensileProblem(Tensile::ContractionProblem& tensile_prob)
     return false;
 }
 
+template <typename TiA, typename To, typename Tc, typename TiB, typename TcA, typename TcB>
+bool useHipBLASLt(const RocblasContractionProblem<TiA, To, Tc, TiB, TcA, TcB>& prob)
+{
+#ifdef BUILD_WITH_HIPBLASLT
+    bool problemSpecific = prob.batch_A == 0; // Only use hipblaslt for non-batch problems.
+    return prob.handle->useHipBLASLt(problemSpecific);
+#else
+    return false;
+#endif
+}
+
 /******************************************************************************
  * runContractionProblem calls Tensile to run a contraction problem described *
  * by RocblasContractionProblem                                               *
@@ -1054,6 +1123,31 @@ rocblas_status
                           rocblas_gemm_algo                                            algo,
                           int32_t solution_index)
 {
+#ifdef BUILD_WITH_HIPBLASLT
+    if(useHipBLASLt(prob))
+    {
+        try
+        {
+            rocblas_internal_ostream msg;
+            auto hipblasltResult = runContractionProblemHipBlasLT(prob, algo, solution_index);
+
+            if(hipblasltResult == rocblas_status_success
+               || (algo == rocblas_gemm_algo_solution_index && solution_index > 0))
+            {
+                return hipblasltResult;
+            }
+        }
+        catch(...)
+        {
+            rocblas_internal_ostream msg;
+            print_once(msg << "\nrocBLAS warning: hipBlasLT exception encountered. ");
+        }
+
+        rocblas_internal_ostream msg;
+        print_once(msg << "\nrocBLAS warning: hipBlasLT failed, falling back to tensile. ");
+    }
+#endif
+
     rocblas_status                                status = rocblas_status_internal_error;
     std::shared_ptr<Tensile::ContractionSolution> solution;
 
@@ -1126,16 +1220,25 @@ rocblas_status
                 {
                     if(!(prob.flags & rocblas_gemm_flags_check_solution_index))
                     {
-                        adapter.launchKernels(
+                        hipError_t hip_status = adapter.launchKernels(
                             solution->solve(tensile_prob, GetTensileInputs(prob), *hardware),
                             handle->get_stream(),
                             handle->startEvent,
                             handle->stopEvent);
+                        if(hip_status != hipSuccess)
+                            status = rocblas_internal_convert_hip_to_rocblas_status(hip_status);
+                        else
+                            status = rocblas_status_success;
+                    }
+                    else
+                    {
+                        status = rocblas_status_success;
                     }
-                    status = rocblas_status_success;
                 }
                 else
+                {
                     status = rocblas_status_invalid_value;
+                }
             }
         }
     }
@@ -1161,6 +1264,13 @@ rocblas_status getAllSolutions(const RocblasContractionProblem<TiA, To, Tc, TiB,
                                rocblas_int* list_array,
                                rocblas_int* list_size)
 {
+#ifdef BUILD_WITH_HIPBLASLT
+    if(useHipBLASLt(prob))
+    {
+        return getAllSolutionsHipBlasLT(prob, option, list_array, list_size);
+    }
+#endif
+
     rocblas_status                                          status = rocblas_status_internal_error;
     std::set<std::shared_ptr<Tensile::ContractionSolution>> solutions;
     try
